/***************************************************************************
 *                                                                         *
 *  draw.c                                                                 *
 *                                                                         *
 *  Some simple -unoptimized- draw-routines                                *
 *                                                                         *
 ***************************************************************************/

#include "z64pfs.h"
#include "draw.fnt"   // The font

/*--------------------------------------------------------------------------
   Global variables
 --------------------------------------------------------------------------*/

u16 fntCol;

/*--------------------------------------------------------------------------
   DrawChar
 --------------------------------------------------------------------------*/

void DrawChar(u16 x,u16 y,char chr)
{
   int loopX,loopY;

   for (loopY = 0; loopY < FontHt; loopY++)
      for (loopX = 0; loopX < FontWd; loopX++)
	 if ((Font8x16[(u8)chr*FontHt+loopY] >> (7-loopX)) & 1)
	    cfb[(loopY+y)*ScreenWd+loopX+x] = fntCol;
}

/*--------------------------------------------------------------------------
   DrawStr
 --------------------------------------------------------------------------*/

void DrawStr(u16 x,u16 y,const char *str)
{
   while (*str)
   {
      DrawChar(x,y,*str++);
      x += 9;
   }
   osWritebackDCacheAll();
}

/*--------------------------------------------------------------------------
   DrawStrCol
 --------------------------------------------------------------------------*/

void DrawStrCol(u16 x,u16 y,const char *str,u16 col)
{
   fntCol = col;
   DrawStr(x,y,str);
}

/*--------------------------------------------------------------------------
   DrawU32
 --------------------------------------------------------------------------*/

void DrawU32(u16 x,u16 y,u32 number,u8 chars)
{
   const char HexTab[] = "0123456789ABCDEF";

   char hexStr[9];
   char *hexPos = hexStr;
   int loop;

   for (loop = (--chars)*4; loop >= 0; loop -= 4)
      *hexPos++ = HexTab[(number >> loop) & 0x0F];
   *hexPos = 0;
   DrawStr(x,y,hexStr);
}

/*--------------------------------------------------------------------------
   DrawHLine
 --------------------------------------------------------------------------*/

void DrawHLine(u16 x,u16 y,u16 size,u16 col)
{
   u16 *cfbPtr;

   cfbPtr = cfb+(y*ScreenWd+x);
   for (; size; size--)
   {
      *cfbPtr = col;
      cfbPtr++;
   }
}

/*--------------------------------------------------------------------------
   DrawVLine
 --------------------------------------------------------------------------*/

void DrawVLine(u16 x,u16 y,u16 size,u16 col)
{
   u16 *cfbPtr;

   cfbPtr = cfb+(y*ScreenWd+x);
   for (; size; size--)
   {
      *cfbPtr = col;
      cfbPtr += ScreenWd;
   }
}

/*--------------------------------------------------------------------------
   DrawFilledBox
 --------------------------------------------------------------------------*/

void DrawFilledBox(u16 x,u16 y,u16 wd,u16 ht,u16 col)
{
   u16 loop;

   for (loop = 0; loop < ht; loop++)
      DrawHLine(x,y+loop,wd,col);
}

/*--------------------------------------------------------------------------
   DrawWindow - background isnt saved
 --------------------------------------------------------------------------*/

void DrawWindow(u16 x,u16 y,u16 wd,u16 ht,const char *title,u16 barCol)
{
   DrawHLine(x,y,wd-1,Col_Gray3);
   DrawVLine(x,y+1,ht-2,Col_Gray3);
   DrawHLine(x+1,y+1,wd-3,Col_White);
   DrawVLine(x+1,y+2,ht-3,Col_White);
   DrawHLine(x+1,y+ht-2,wd-2,Col_Gray1);
   DrawVLine(x+wd-2,y+1,ht-3,Col_Gray1);
   DrawHLine(x,y+ht-1,wd,Col_Black);
   DrawVLine(x+wd-1,y,ht,Col_Black);

   DrawFilledBox(x+2,y+2,wd-4,ht-4,Col_Gray4);
   DrawFilledBox(x+3,y+3,wd-6,11,barCol);

   x += (wd-strlen(title)*9) >> 1;
   DrawStrCol(x,y+5,title,Col_White);
}

