/***************************************************************************
 *                                                                         *
 *  z64pps.c                                                               *
 *                                                                         *
 *  The PC-based parallel-port sender                                      *
 *                                                                         *
 *  Version 0.50                                                           *
 *  Ravemax/Dextrose                                                       *
 *                                                                         *
 ***************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <dos.h>
#include <ctype.h>

/*--------------------------------------------------------------------------
   Types
 --------------------------------------------------------------------------*/

typedef unsigned short   U16;
typedef unsigned long    U32;

/*--------------------------------------------------------------------------
   Constant strings
 --------------------------------------------------------------------------*/

const char *const TitleStr =
   "\nZ64PPS v0.5 (c) Ravemax of Dextrose\n";

const char *const UsageStr =
   "Usage: Z64PPS rom [-<option>,..]\n\n"
   "<Options>\n"
   "  p?: Port ? (default: 0x378)\n"
   "  a?: Destinationaddress ? (default: 0)\n";

const char *const Err_UnknownOption_Str =
   "Unknown option: '%c'\n\n";

const char *const Err_FileIO_Str =
   "Error: '%s' not found!\n\n";

/*--------------------------------------------------------------------------
   Global variables
 --------------------------------------------------------------------------*/

int lptBase = 0x378;
U32 destAddr = 0L;

/*--------------------------------------------------------------------------
   ParseCommandline()
 --------------------------------------------------------------------------*/

void ParseCommandline(char startFrom,int argc,char *argv[])
{
   while (1)
   {
      argc--;
      if (argc < startFrom)
	 break;

      switch(tolower(argv[argc][1]))
      {
	 case 'p' :
	    lptBase = (int)strtol(argv[argc]+2,NULL,0);
	    break;
	 case 'd' :
	    destAddr = strtoul(argv[argc]+2,NULL,0);
	    break;
	 case '?' :
	    puts(UsageStr);
	    exit(1);
	 default :
	    printf(Err_UnknownOption_Str,argv[argc][1]);
	    exit(2);
      }
   }
}

/*--------------------------------------------------------------------------
   FileSize()
 --------------------------------------------------------------------------*/

long FileSize(FILE *file)
{
   long tsize,temp = ftell(file);

   fseek(file,0,SEEK_END);
   tsize = ftell(file);
   fseek(file,temp,SEEK_SET);
   return(tsize);
}

/*--------------------------------------------------------------------------
   ParallelSend - no checks..
 --------------------------------------------------------------------------*/

void ParallelSend(char *buffer,short size)
{
   for (; size; size--)
   {
      while (inportb(lptBase+1) & 128);
      outportb(lptBase,*buffer | 0x10);
//      delay(10); // Should work, because of flag-checks, w/o delays
      while (!(inportb(lptBase+1) & 128));
      outportb(lptBase,(*buffer >> 4) & 0x0F);
//      delay(10);
      buffer++;
   }
}

/*--------------------------------------------------------------------------
   main()
 --------------------------------------------------------------------------*/

int main(int argc,char *argv[])
{
   #define BufferSize   16384

   FILE *f;
   long romSize;
   char buffer[BufferSize];
   short numRead;

   puts(TitleStr);

   /*
    *  Commandline
    */

   if (argc < 2)
   {
      puts(UsageStr);
      return(1);
   }
   ParseCommandline(2,argc,argv);

   /*
    *  Open ROM
    */

   if ((f = fopen(argv[1],"rb")) == NULL)
   {
      printf(Err_FileIO_Str,argv[1]);
      return(3);
   }

   romSize = FileSize(f);

   /*
    *  Send
    */

   ParallelSend((char *)&destAddr,4);  // "Header"
   ParallelSend((char *)&romSize,4);

   while (!feof(f))
   {
      numRead = fread(buffer,1,BufferSize,f);
      ParallelSend(buffer,numRead);
      printf(".");
   }

   puts("\n");
   fclose(f);
   return(0);
}
