/*---ASPIDEV.CPP---*/
#include"aspi\aspidev.h"
#include<stddef.h>
#include<string.h>
CASPI_Device::CASPI_Device()
:m_pASPI(NULL)
{
  m_Thread.HostNum=m_Thread.TargetID=m_Thread.LUN=0;
}
CASPI_Device::CASPI_Device(ASPI_Driver *pASPI)
:m_pASPI(pASPI)
{
  m_Thread.HostNum=m_Thread.TargetID=m_Thread.LUN=0;
}
CASPI_Device::CASPI_Device(ASPI_Driver *pASPI,const Thread &thread)
:m_pASPI(pASPI),m_Thread(thread)
{
}
ASPI_Driver *CASPI_Device::SetASPI_Driver(ASPI_Driver *pASPI)
{
  ASPI_Driver *tmp;
  tmp=m_pASPI;
  m_pASPI=pASPI;
  return tmp;
}
ASPI_Driver *CASPI_Device::GetASPI_Driver()
{
  return m_pASPI;
}
Thread CASPI_Device::SetThread(const Thread &thread)
{
  Thread tmp;
  tmp=m_Thread;     // Save previous thread
  m_Thread=thread;  // Set new thread
  return tmp;       // Return previous thread
}
Thread CASPI_Device::GetThread()
{
  return m_Thread;
}
WORD CASPI_Device::TestUnitReady()
{
  SenseData sd;
  return TestUnitReady(sd);
}
WORD CASPI_Device::TestUnitReady(SenseData &sd)
{
  SCSI_CDB_TestUnitReady cdb;
  CommandInfo ci;
  WORD ret;
  cdb.OpCode=0x00;
  cdb.Resv1=0;
  cdb.LUN=GetThread().LUN;
  cdb.Resv2=0;
  cdb.Resv3=0;
  cdb.Resv4=0;
  cdb.Control=0;
  ci.SetThread(GetThread());
  ci.SetCDB(&cdb,sizeof(cdb));
  ci.SetDataBuffer(NULL,0);
  ret=GetASPI_Driver()->SCSI_ExecuteNoData(ci);
  sd=ci.Sense;
  return ret;
}
WORD CASPI_Device::ModeSense(int PageCode,BlkDes *BD,BYTE *AllocPtr,BYTE AllocLen)
{
  static SCSI_CDB_MODE_SENSE cdb;
  static CommandInfo ci;
  static BYTE ModeSenseParamList[12+256];  // Mode Sense Parameter List
  WORD ret;
  cdb.OpCode=0x1a;  // MODE SENSE
  cdb.Resv1=0;
  cdb.LUN=GetThread().LUN;
  cdb.PageCode=PageCode;
  cdb.PC=0;  // Page Control:Current Values
  cdb.Resv2=0;
  cdb.AllocLen=AllocLen+12;
  cdb.Control=0;
  ci.SetThread(GetThread());
  ci.SetCDB(&cdb,sizeof(cdb));
  ci.SetDataBuffer(ModeSenseParamList,12+AllocLen);
  ret=GetASPI_Driver()->SCSI_ExecuteInputData(ci);
  if(ret!=0)  // Error
    return ret;
  // Copy block descriptor
  ((BYTE*)(&BD->NumOfBlk))[0]=ModeSenseParamList[7];  // NumOfBlk(LSB)
  ((BYTE*)(&BD->NumOfBlk))[1]=ModeSenseParamList[6];  // NumOfBlk
  ((BYTE*)(&BD->NumOfBlk))[2]=ModeSenseParamList[5];  // NumOfBlk(MSB)
  ((BYTE*)(&BD->NumOfBlk))[3]=0;
  ((BYTE*)(&BD->BlkLen))[0]=ModeSenseParamList[11];   // BlkLen(LSB)
  ((BYTE*)(&BD->BlkLen))[1]=ModeSenseParamList[10];   // BlkLen
  ((BYTE*)(&BD->BlkLen))[2]=ModeSenseParamList[9];    // BlkLen(MSB)
  ((BYTE*)(&BD->BlkLen))[3]=0;
  // Copy page
  memcpy(AllocPtr,ModeSenseParamList+12,AllocLen);
  return 0;
}
WORD CASPI_Device::ModeSenseReadPage(int PageCode,BYTE *AllocPtr,BYTE AllocLen)
{
  WORD ret;
  BlkDes BD;
  ret=ModeSense(PageCode,&BD,AllocPtr,AllocLen);
  return ret;
}
WORD CASPI_Device::ModeSelectSetBlkDes(BlkDes *BD)
{
  static SCSI_CDB_MODE_SELECT cdb;
  static CommandInfo ci;
  static BYTE ModeSelectParamList[12];  // Mode Select Parameter List
  WORD ret;
  cdb.OpCode=0x15;  // MODE SELECT
  cdb.SP=0;
  cdb.Resv1=0;
  cdb.PF=1;
  cdb.LUN=GetThread().LUN;
  cdb.Resv2=0;
  cdb.Resv3=0;
  cdb.ParamListLen=12;
  cdb.Control=0;
  ci.SetThread(GetThread());
  ci.SetCDB(&cdb,sizeof(cdb));
  ci.SetDataBuffer(ModeSelectParamList,12);
  // Initial header
  memset(ModeSelectParamList,0,3);
  ModeSelectParamList[3]=8;
  // Initial block descriptor
  ModeSelectParamList[4]=BD->DensityCode;              // Density code
  ModeSelectParamList[5]=((BYTE*)(&BD->NumOfBlk))[2];  // NumOfBlk(MSB)
  ModeSelectParamList[6]=((BYTE*)(&BD->NumOfBlk))[1];  // NumOfBlk
  ModeSelectParamList[7]=((BYTE*)(&BD->NumOfBlk))[0];  // NumOfBlk(LSB)
  ModeSelectParamList[8]=0;                            // Reserved
  ModeSelectParamList[9]=((BYTE*)(&BD->BlkLen))[2];    // BlkLen(MSB)
  ModeSelectParamList[10]=((BYTE*)(&BD->BlkLen))[1];   // BlkLen
  ModeSelectParamList[11]=((BYTE*)(&BD->BlkLen))[0];   // BlkLen(LSB)
  ret=GetASPI_Driver()->SCSI_ExecuteOutputData(ci);
  return ret;
}
WORD CASPI_Device::ModeSelectSetPage(BYTE *AllocPtr,BYTE AllocLen)
{
  static SCSI_CDB_MODE_SELECT cdb;
  static CommandInfo ci;
  static BYTE ModeSelectParamList[12+256];  // Mode Select Parameter List
  WORD ret;
  cdb.OpCode=0x15;  // MODE SELECT
  cdb.SP=0;
  cdb.Resv1=0;
  cdb.PF=1;
  cdb.LUN=GetThread().LUN;
  cdb.Resv2=0;
  cdb.Resv3=0;
  cdb.ParamListLen=AllocLen+4;
  cdb.Control=0;
  ci.SetThread(GetThread());
  ci.SetCDB(&cdb,sizeof(cdb));
  ci.SetDataBuffer(ModeSelectParamList,4+AllocLen);
  // Initial header
  memset(ModeSelectParamList,0,4);
  memcpy(ModeSelectParamList+4,AllocPtr,AllocLen);
  ret=GetASPI_Driver()->SCSI_ExecuteOutputData(ci);
  return ret;
}
