/*---ASPIDRV.CPP---*/
#include"aspi\aspidrv.h"
#include<dos.h>
#include<stddef.h>
#include<string.h>
// ASPI device name
const static char ASPI_Name[]="SCSIMGR$";
// Byte alignment
#pragma option -a-
// SCSI Request Block(SRB)
// SRB header
struct SRB_Header
{
  enum CmdList       // Command code list
  {
    HostInquiry     =0,
    GetDeviceType   =1,
    ExecuteSCSI_IO  =2,
    AbortSCSI_IO    =3,
    ResetSCSI_Device=4,
  };
  BYTE Command;      // Command Code
  BYTE Status;       // Status Byte
  BYTE HostNum;      // Host Adaptor Number
  BYTE RequestFlag;  // SCSI Request Flags
  BYTE Resv[4];      // Reserved
};
// Host Adapter Inquiry Command SRB
struct HostInquirySRB
{
  SRB_Header Header;   // Header
  BYTE NumOfHosts;     // Number of Host Adapter
  BYTE HostSCSI_ID;    // Host Adapter SCSI ID
  BYTE ManagerID[16];  // SCSI Manager ID
  BYTE HostID[16];     // Host Adapter Identifier
  BYTE HostParam[16];  // Host Adaptor Unique Parameter
  WORD SuppExt;        // Supported Extensions
};
// Get Device Type Command SRB
struct GetDeviceTypeSRB
{
  SRB_Header Header;  // Header
  BYTE TargetID;      // Target ID
  BYTE LUN;           // LUN
  BYTE Type;          // Peripheral Device Type
};
// Execute SCSI I/O Command SRB
struct ExecuteSCSI_IO_SRB
{
  SRB_Header Header;     // Header
  BYTE TargetID;         // Target ID
  BYTE LUN;              // LUN
  DWORD DataAllocLen;    // Data Allocation Length
  BYTE SenseAllocLen;    // Sense Allocation Length
  FARPTR DataBufferPtr;  // Data Buffer Pointer
  FARPTR SRB_LinkPtr;    // SRB Link Pointer
  BYTE SCSI_CDB_Len;     // SCSI CDB Length
  BYTE HostStatus;       // Host Adaptor Status
  BYTE TargetStatus;     // Target Status
  FARPTR PostAddr;       // Post Routine Address
  BYTE ASPI_Work[34];    // Reserved for ASPI Workspace
  BYTE CDB_Sense[512];   // For CDB and Sense data
};
// Abort SCSI I/O Command SRB
struct AbortSCSI_IO_SRB
{
  SRB_Header Header;   // Header
  FARPTR SRB_Pointer;  // SRB Pointer to Abort
};
// Reset SCSI Device Command SRB
struct ResetSCSI_Device_SRB
{
  SRB_Header Header;   // Header
  BYTE TargetID;       // Target ID
  BYTE LUN;            // LUN
  BYTE Resv[14];       // Reserved
  BYTE HostStatus;     // Host Adaptor Status
  BYTE TargetStatus;   // Target Status
  FARPTR PostAddr;     // Post Routine Address
  BYTE ASPI_Work[2];   // Reserved for ASPI Workspace
};
// CommandInfo member functions
CommandInfo::CommandInfo()
{
  Reset();
}
void CommandInfo::Reset()
{
  SetThread(0,0,0);
  CDB=NULL;
  SCSI_CDB_Len=0;
  DataBufferPtr=NULL;
  DataAllocLen=0;
}
void CommandInfo::SetThread(const Thread &thread)
{
  InputThread=thread;
}
void CommandInfo::SetThread(BYTE HostNum,BYTE TargetID,BYTE LUN)
{
  InputThread.HostNum=HostNum;
  InputThread.TargetID=TargetID;
  InputThread.LUN=LUN;
}
void CommandInfo::SetCDB(FARPTR cdb,BYTE scsi_cdb_len)
{
  CDB=cdb;
  SCSI_CDB_Len=scsi_cdb_len;
}
void CommandInfo::SetDataBuffer(FARPTR BufferPtr,DWORD AllocLen)
{
  DataBufferPtr=BufferPtr;
  DataAllocLen=AllocLen;
}
// SCSI_CDB_TestUnitReady member functions
SCSI_CDB_TestUnitReady::SCSI_CDB_TestUnitReady()
{
  Reset();
}
void SCSI_CDB_TestUnitReady::Reset()
{
  OpCode=0x00;
  Resv1=0;
  LUN=0;
  Resv2=0;
  Resv3=0;
  Resv4=0;
  Control=0;
}
// SCSI_CDB_Inquiry member functions
SCSI_CDB_Inquiry::SCSI_CDB_Inquiry()
{
  Reset();
}
void SCSI_CDB_Inquiry::Reset()
{
  OpCode=0x12;
  LUN=0;
  EVPD=0;
  Resv1=0;
  PageCode=0;
  Resv2=0;
  DataAllocLen=0;
  Control=0;
}
// ASPI_Driver Member functions
Std_ASPI_Driver::Std_ASPI_Driver()
{
  REGS reg;
  SREGS sreg;
  unsigned short FileHandle;
  m_ASPI_Entry=NULL;
  // Open file "SCSIMGR$"
  reg.x.ax=0x3d00;
  sreg.ds=FP_SEG(ASPI_Name);
  reg.x.dx=FP_OFF(ASPI_Name);
  int86x(0x21,&reg,&reg,&sreg);
  if(reg.x.flags&1!=0)
    return;  // Unable to open,quit
  FileHandle=reg.x.ax;
  // IOCTL read
  reg.x.ax=0x4402;
  sreg.ds=FP_SEG(&m_ASPI_Entry);
  reg.x.dx=FP_OFF(&m_ASPI_Entry);
  reg.x.cx=4;
  reg.x.bx=FileHandle;
  int86x(0x21,&reg,&reg,&sreg);
  // Close
  reg.x.ax=0x3e00;
  reg.x.bx=FileHandle;
  int86x(0x21,&reg,&reg,&sreg);
}
int Std_ASPI_Driver::IsInstalled()
{
  return(m_ASPI_Entry!=NULL);
}
int Std_ASPI_Driver::GetNumOfHosts()
{
  static HostInquirySRB srb;
  srb.Header.Command=SRB_Header::HostInquiry;
  srb.Header.HostNum=0;
  srb.Header.RequestFlag=0;
  (*(DWORD*)(&srb.Header.Resv))=0;
  (*m_ASPI_Entry)(&srb);  // Call ASPI
  return srb.NumOfHosts;
}
void Std_ASPI_Driver::HostInquiry(BYTE HostNum,HostInquiryInfo *info)
{
  static HostInquirySRB srb;
  srb.Header.Command=SRB_Header::HostInquiry;
  srb.Header.HostNum=HostNum;
  srb.Header.RequestFlag=0;
  (*(DWORD*)(&srb.Header.Resv))=0;
  (*m_ASPI_Entry)(&srb);  // Call ASPI
  info->HostSCSI_ID=srb.HostSCSI_ID;
  memcpy(info->ManagerID,srb.ManagerID,16);
  info->ManagerID[16]=0;
  memcpy(info->HostID,srb.HostID,16);
  info->HostID[16]=0;
  memcpy(info->HostParam,srb.HostParam,16);
  info->HostParam[16]=0;
  info->SuppExt=srb.SuppExt;
}
BYTE Std_ASPI_Driver::GetDeviceType(BYTE Host,BYTE TargetID,BYTE LUN,BYTE *type)
{
  static GetDeviceTypeSRB srb;
  srb.Header.Command=SRB_Header::GetDeviceType;
  srb.Header.HostNum=Host;
  srb.Header.RequestFlag=0;
  (*(DWORD*)(&srb.Header.Resv))=0;
  srb.TargetID=TargetID;
  srb.LUN=LUN;
  (*m_ASPI_Entry)(&srb);  // Call ASPI
  if(srb.Header.Status==0x01)  // SCSI Completed without Error
  {
    *type=srb.Type;
    return 0;
  }
  else
  {
    *type=0;
    return 1;
  }
}
WORD Std_ASPI_Driver::SCSI_ExecuteInputData(CommandInfo &cmd)
{
  // Request Flags
  // Bit:
  //  7 6 5 4 3 2 1 0
  //  0 0 0 0 1 0 0 0 = 0x08
  //  | | | | | | | |
  //  | | | | | | | +-- 0 = Posting Disable
  //  | | | | | | +---- 0 = Linking Disable
  //  | | | | | +------ 0 = Residual Byte Length Reporting Disable
  //  | | | +-+-------- 01 = SCSI Target to Host
  //  +-+-+------------ Reserved
  return SCSI_Execute(cmd,0x08);
}
WORD Std_ASPI_Driver::SCSI_ExecuteOutputData(CommandInfo &cmd)
{
  // Request Flags
  // Bit:
  //  7 6 5 4 3 2 1 0
  //  0 0 0 1 0 0 0 0 = 0x10
  //  | | | | | | | |
  //  | | | | | | | +-- 0 = Posting Disable
  //  | | | | | | +---- 0 = Linking Disable
  //  | | | | | +------ 0 = Residual Byte Length Reporting Disable
  //  | | | +-+-------- 10 = SCSI Target to Host
  //  +-+-+------------ Reserved
  return SCSI_Execute(cmd,0x10);
}
WORD Std_ASPI_Driver::SCSI_ExecuteNoData(CommandInfo &cmd)
{
  // Request Flags
  // Bit:
  //  7 6 5 4 3 2 1 0
  //  0 0 0 1 1 0 0 0 = 0x18
  //  | | | | | | | |
  //  | | | | | | | +-- 0 = Posting Disable
  //  | | | | | | +---- 0 = Linking Disable
  //  | | | | | +------ 0 = Residual Byte Length Reporting Disable
  //  | | | +-+-------- 11 = No data transfer
  //  +-+-+------------ Reserved
  return SCSI_Execute(cmd,0x18);
}
WORD Std_ASPI_Driver::SCSI_ResetDevice(const Thread &thread)
{
  static ResetSCSI_Device_SRB srb;
  srb.Header.Command=SRB_Header::ResetSCSI_Device;
  srb.Header.HostNum=thread.HostNum;
  srb.Header.RequestFlag=0;
  (*(DWORD*)(&srb.Header.Resv))=0;
  srb.TargetID=thread.TargetID;
  srb.LUN=thread.LUN;
  memset(&srb.Resv,0,14);
  srb.PostAddr=NULL;  // No Posting
  (*m_ASPI_Entry)(&srb);  // Call ASPI
  // Wait until process ok
  while(srb.Header.Status==0)
    ;
  // Return Host/Target Status
  return *( (WORD*)(&srb.HostStatus) );
}
Std_ASPI_Driver::~Std_ASPI_Driver()
{
}
WORD Std_ASPI_Driver::SCSI_Execute(CommandInfo &cmd,BYTE ReqFlag)
{
  static ExecuteSCSI_IO_SRB srb;
  srb.Header.Command=SRB_Header::ExecuteSCSI_IO;
  srb.Header.HostNum=cmd.InputThread.HostNum;
  srb.Header.RequestFlag=ReqFlag;
  (*(DWORD*)(&srb.Header.Resv))=0;
  srb.TargetID=cmd.InputThread.TargetID;
  srb.LUN=cmd.InputThread.LUN;
  srb.DataAllocLen=cmd.DataAllocLen;
  srb.SenseAllocLen=255;
  srb.DataBufferPtr=cmd.DataBufferPtr;
  srb.SRB_LinkPtr=NULL;  // No linking
  srb.SCSI_CDB_Len=cmd.SCSI_CDB_Len;
  srb.PostAddr=NULL;  // No Posting
  _fmemcpy( ((BYTE*)&srb)+64,cmd.CDB,cmd.SCSI_CDB_Len );  // Copy CDB
  (*m_ASPI_Entry)(&srb);  // Call ASPI
  // Wait until process ok
  while(srb.Header.Status==0)
    ;
  // Return Host/Target Status
  cmd.HostStatus=srb.HostStatus;
  cmd.TargetStatus=srb.TargetStatus;
  // Return sense data
  memcpy(&cmd.Sense,((BYTE*)&srb)+64+cmd.SCSI_CDB_Len,sizeof(SenseData));
  // Return Host/Target Status
  WORD ret;
  ret=*( (WORD*)(&srb.HostStatus) );
  return ret;
}
void Std_ASPI_Driver::SCSI_ExecuteNoWait(BYTE *SRB,CommandInfo &cmd,BYTE ReqFlag)
{
  ExecuteSCSI_IO_SRB *pSRB;
  pSRB=(ExecuteSCSI_IO_SRB *)SRB;
  pSRB->Header.Command=SRB_Header::ExecuteSCSI_IO;
  pSRB->Header.HostNum=cmd.InputThread.HostNum;
  pSRB->Header.RequestFlag=ReqFlag;
  (*(DWORD*)(&pSRB->Header.Resv))=0;
  pSRB->TargetID=cmd.InputThread.TargetID;
  pSRB->LUN=cmd.InputThread.LUN;
  pSRB->DataAllocLen=cmd.DataAllocLen;
  pSRB->SenseAllocLen=255;
  pSRB->DataBufferPtr=cmd.DataBufferPtr;
  pSRB->SRB_LinkPtr=NULL;  // No linking
  pSRB->SCSI_CDB_Len=cmd.SCSI_CDB_Len;
  pSRB->PostAddr=NULL;  // No Posting
  _fmemcpy( ((BYTE*)pSRB)+64,cmd.CDB,cmd.SCSI_CDB_Len );  // Copy CDB
  (*m_ASPI_Entry)(pSRB);  // Call ASPI
}
//
int aspi_inquiry_unit(ASPI_Driver &aspidrv,Thread &thread,BYTE *buf)
{
  SCSI_CDB_Inquiry cdb_inquiry;
  CommandInfo ci;
  cdb_inquiry.LUN=thread.LUN;
  cdb_inquiry.DataAllocLen=96;
  ci.SetThread(thread);
  ci.SetCDB(&cdb_inquiry,sizeof(cdb_inquiry));
  ci.SetDataBuffer(buf,96);
  WORD ret;
  ret=aspidrv.SCSI_ExecuteInputData(ci);
  return ret;
}
void cdb_write16bit(void *dst,WORD src)
{
  ((BYTE*)dst)[0]=((BYTE*)&src)[1];
  ((BYTE*)dst)[1]=((BYTE*)&src)[0];
}
void cdb_write32bit(void *dst,DWORD src)
{
  ((BYTE*)dst)[0]=((BYTE*)&src)[3];
  ((BYTE*)dst)[1]=((BYTE*)&src)[2];
  ((BYTE*)dst)[2]=((BYTE*)&src)[1];
  ((BYTE*)dst)[3]=((BYTE*)&src)[0];
}

