/*---TEXTLCD.CPP---*/
#include"lcd\textlcd.h"
#include<stdarg.h>
#include<stdio.h>
// Text_LCD_config member function
Text_LCD_config::Text_LCD_config()
{
  display_on=1;
  cursor_on=1;
  blink_char=0;
  display_lines=1;
}
// Text_LCD member functions
Text_LCD::Text_LCD(Text_LCD_interface *pif):m_pif(pif)
{
}
void Text_LCD::clear_screen()
{
  while(is_LCD_ready()==0);
  m_pif->write_cmd(1);
}
void Text_LCD::home()
{
  while(is_LCD_ready()==0);
  m_pif->write_cmd(2);
}
void Text_LCD::entry_mode_set(int I_D,int S)
{
  unsigned char f;
  f=0x4;
  if(I_D)
    f|=0x2;
  if(S)
    f|=0x1;
  while(is_LCD_ready()==0);
  m_pif->write_cmd(f);
}
void Text_LCD::display_control(int D,int C,int B)
{
  unsigned char f;
  f=0x8;
  if(D)
    f|=0x4;
  if(C)
    f|=0x2;
  if(B)
    f|=0x1;
  while(is_LCD_ready()==0);
  m_pif->write_cmd(f);
}
void Text_LCD::shift(int S_C,int R_L)
{
  unsigned char f;
  f=0x10;
  if(S_C)
    f|=0x8;
  if(R_L)
    f|=0x4;
  while(is_LCD_ready()==0);
  m_pif->write_cmd(f);
}
void Text_LCD::function_set(int DL,int N,int F)
{
  unsigned char f;
  f=0x20;
  if(DL)
    f|=0x10;
  if(N)
    f|=0x8;
  if(F)
    f|=0x4;
  while(is_LCD_ready()==0);
  m_pif->write_cmd(f);
}
void Text_LCD::set_CG_RAM_addr(unsigned char addr)
{
  while(is_LCD_ready()==0);
  m_pif->write_cmd(0x40|(addr&0x3f));
}
void Text_LCD::set_DD_RAM_addr(unsigned char addr)
{
  while(is_LCD_ready()==0);
  m_pif->write_cmd(0x80|(addr&0x7f));
}
void Text_LCD::set_font(int fnt_code,char *fnt_buf)
{
  int i;
  for(i=0;i<8;i++)
  {
    set_CG_RAM_addr(fnt_code*8+i);
    write_char(fnt_buf[i]);
  }
}
void Text_LCD::write_char(unsigned char char_code)
{
  while(is_LCD_ready()==0);
  m_pif->write_data(char_code);
}
void Text_LCD::write_str(unsigned char *str)
{
  while(*str!=0)
  {
    while(is_LCD_ready()==0);
    m_pif->write_data(*str);
    str++;
  }
}
void Text_LCD::read_str(unsigned char *str,int sz)
{
  while(sz!=0)
  {
    while(is_LCD_ready()==0);
    *str=m_pif->read_data();
    str++;
    sz--;
  }
}
void Text_LCD::set_config(const Text_LCD_config &cfg)
{
  int D,C,B,N;
  D=(cfg.display_on!=0);
  C=(cfg.cursor_on!=0);
  B=(cfg.blink_char!=0);
  if(cfg.display_lines==2)
    N=1;
  else
    N=0;
  entry_mode_set(1,0);
  display_control(D,C,B);
  function_set(1,N,0);
}
void Text_LCD::gotoxy(int x,int y)
{
  set_DD_RAM_addr((x-1)+(y-1)*0x40);
}
int Text_LCD::printf(const char *fmt,...)
{
  char buffer[80];
  va_list argptr;
  int cnt;
  va_start(argptr, fmt);
  cnt = vsprintf(buffer, fmt, argptr);
  va_end(argptr);
  write_str(buffer);
  return(cnt);
}
int Text_LCD::is_LCD_ready()
{
  return (m_pif->read_status()&0x80)==0;
}
