/*---IPS_DEC.CPP---*/
#include"ips_dec.h"
#include<string.h>
// state
enum
{
  ips_err,ips_end,get_header,
  get_addr,get_byte_cnt,get_fill_data
};
// member functions
IPS_decoder::IPS_decoder()
{
  m_state=get_header;
  m_addr=0;
}
int IPS_decoder::get_request_data_size()
{
  switch(m_state)
  {
    case get_header:
      return 5;
    case get_addr:
      return 3;
    case get_byte_cnt:
      return 2;
    case get_fill_data:
      return 3;
    default:
      return 0;
  }
}
void IPS_decoder::send_data(const unsigned char far *data,IPS_command &cmd)
{
  switch(m_state)
  {
    case get_header:
      if(_fmemcmp(data,"PATCH",5)!=0)
      {
        // no "PATCH" header,error
        m_state=ips_err;
        cmd.type=IPS_error;
      }
      // header ok,get address
      m_state=get_addr;
      cmd.type=IPS_next;
      return;
    case get_addr:
      if(_fmemcmp(data,"EOF",3)==0)
      {
        // found "EOF",end IPS
        m_state=ips_end;
        cmd.type=IPS_end;
        return;
      }
      // not "EOF", convert ADR byte order to x86 format
      ((unsigned char*)(&m_addr))[3]=0;
      ((unsigned char*)(&m_addr))[2]=data[0];
      ((unsigned char*)(&m_addr))[1]=data[1];
      ((unsigned char*)(&m_addr))[0]=data[2];
      m_state=get_byte_cnt;
      cmd.type=IPS_next;
      return;
    case get_byte_cnt:
      // get data length
      unsigned byte_cnt;
      ((unsigned char*)(&byte_cnt))[1]=data[0];
      ((unsigned char*)(&byte_cnt))[0]=data[1];
      if(byte_cnt!=0)
      {
        // data length !=0, do copy data
        m_state=get_addr;
        cmd.type=IPS_copy;
        cmd.addr=m_addr;
        cmd.copy_cnt=byte_cnt;
      }
      else
      {
        // data length == 0, do fill data
        m_state=get_fill_data;
        cmd.type=IPS_next;
      }
      return;
    case get_fill_data:
      // get fill count/fill data
      unsigned fill_cnt;
      unsigned char fill_data;
      ((unsigned char*)(&fill_cnt))[1]=data[0];
      ((unsigned char*)(&fill_cnt))[0]=data[1];
      fill_data=data[2];
      m_state=get_addr;
      cmd.type=IPS_fill;
      cmd.addr=m_addr;
      cmd.fill.cnt=fill_cnt;
      cmd.fill.data=fill_data;
      return;
    default:
      // error
      m_state=ips_err;
      cmd.type=IPS_error;
      return;
  }
}
