#include"loadbios.h"
#include"z64_gbl.h"
#include"z64const.h"
#include"z64_util.h"
#include"flashrom\sst.h"
#include<string.h>
int read_bios_file_to_dram()
{
  int ret;
  // Read BIOS file
  show_center_str(1,"READING...");
  ret=p_filesys->open_bios_file_read();
  if(ret!=0)
  {
    load_file_error();
    return 1;
  }
  process_file_init_progress(bios_size);
  N64_loop loop;
  for(loop.init(bios_size,file_buf_size);loop.not_end();loop.next())
  {
    // read file data
		if(p_filesys->read_bios_file(file_buf,file_buf_size)!=0)
		{
      p_filesys->close_bios_file();
      load_file_error();
      return 1;  // Load error
    }
    // write DRAM
    p_emu->write_DRAM(loop.get_word_addr(),file_buf,file_buf_size/2);
    // print progress
		unsigned long nsize;
    nsize=loop.get_byte_addr()+loop.get_blk_size();
    process_file_update_progress(nsize);
	}
  p_filesys->close_bios_file();
  // Verify
  show_center_str(1,"VERIFY...");
  ret=p_filesys->open_bios_file_read();
  if(ret!=0)
  {
    load_file_error();
    return 1;
  }
  process_file_init_progress(bios_size);
  for(loop.init(bios_size,file_buf_size);loop.not_end();loop.next())
  {
    // read file data
		if(p_filesys->read_bios_file(file_buf,file_buf_size)!=0)
		{
      p_filesys->close_bios_file();
      load_file_error();
      return 1;  // Load error
    }
    // read DRAM
    p_emu->read_DRAM(loop.get_word_addr(),ref_file_buf,file_buf_size/2);
    // compare
    if(_fmemcmp(file_buf,ref_file_buf,file_buf_size)!=0)
    {
      p_filesys->close_bios_file();
      show_error_msg_wait_enter("\nVERIFY FILE ERROR");
      return 1;  // Verify error
    }
    // print progress
		unsigned long nsize;
    nsize=loop.get_byte_addr()+loop.get_blk_size();
    process_file_update_progress(nsize);
	}
  p_filesys->close_bios_file();
  return 0;
}
int write_dram_to_bios_file()
{
  show_center_str(1,"WRITING...");
  int ret;
  ret=p_filesys->open_bios_file_create();
  if(ret!=0)
  {
    write_file_error();
    return 1;
  }
  process_file_init_progress(bios_size);
  N64_loop loop;
  for(loop.init(bios_size,file_buf_size);loop.not_end();loop.next())
  {
    // read DRAM
    p_emu->read_DRAM(loop.get_word_addr(),file_buf,file_buf_size/2);
    // write file data
		if(p_filesys->write_bios_file(file_buf,file_buf_size)!=0)
		{
      p_filesys->close_bios_file();
      write_file_error();
      return 1;  // Load error
    }
    // print progress
		unsigned long nsize;
    nsize=loop.get_byte_addr()+loop.get_blk_size();
    process_file_update_progress(nsize);
	}
  p_filesys->close_bios_file();
  return 0;
}
void write_dram_to_flash()
{
  show_center_str(1,"WRITING...");
  flash_sst flash;
  flash.set_segment(0xe000);
  unsigned sector_size;
  sector_size=flash.get_sector_size();
  unsigned long addr_ptr;
  _asm cli
  process_file_init_progress(bios_size-64L*1024);
  for(addr_ptr=0;addr_ptr<bios_size-64L*1024;addr_ptr+=sector_size)
  {
    unsigned sector_idx;
    sector_idx=addr_ptr/sector_size;
    p_emu->read_DRAM(addr_ptr/2,file_buf,sector_size/2);
    int ret;
    ret=flash.write(file_buf,sector_idx,1);
    if(ret!=0)
    {
      show_center_str(1,"FATAL ERROR");
      show_center_str(2,"WRITE FLASH FAILED");
      error_sound();
      while(1);
    }
    process_file_update_progress(addr_ptr+sector_size);
  }
  show_center_str(1,"UPDATE COMPLETED");
  show_center_str(2,"TURN OFF Z64");
  while(1);
}
void copy_bios_file()
{
  int ret;
  ret=read_bios_file_to_dram();
  if(ret!=0)
    return;
  ret=wait_change_disk();
  if(ret)
    return;
  write_dram_to_bios_file();
}
void update_bios_file()
{
  int ret;
  ret=read_bios_file_to_dram();
  if(ret!=0)
    return;
  write_dram_to_flash();
}
