/*---N64_EMU.H---*/
#ifndef _INC_N64_EMU
#define _INC_N64_EMU
// N64 ROM emulator
class N64_ROM_emulator
{
  public:
    N64_ROM_emulator();
    void set_chip_ver(unsigned ver);
    unsigned get_chip_ver();
    // set emulator control register
    void set_ctrl_reg(unsigned int work_mode);
    // set ROM/DRAM read/write address
    void set_address(unsigned long addr);
    // access constant value
    enum
    {
      N64_ROM=0,N64_DRAM=1,PC_ROM=2,PC_DRAM=3,  // version 1
      N64_16M_ROM_SRAM,N64_32M_ROM,             // version 2
      USER_BIOS
    };
    // set access control
    void set_access_ctrl(unsigned mode);
    // clear/read N64 request flag
    void clear_N64_req_flag();
    int read_N64_req_flag();
    // read/write data from/to ROM/DRAM
    // word_cnt must<=256(512 bytes) when read ROM
    void read_data_word(unsigned char far *buf,unsigned word_cnt);
    void write_data_word(unsigned char far *buf,unsigned word_cnt);
    // read ROM
    void read_ROM
         (unsigned long addr,unsigned char far *buf,unsigned word_cnt);
    // read/write cartride SRAM
    void read_cart_SRAM
         (unsigned long addr,unsigned char far *buf,unsigned word_cnt);
    void write_cart_SRAM
         (unsigned long addr,unsigned char far *buf,unsigned word_cnt);
    // read/write DRAM mapped SRAM
    void read_DRAM_mapped_SRAM
         (unsigned long addr,unsigned char far *buf,unsigned word_cnt);
    void write_DRAM_mapped_SRAM
         (unsigned long addr,unsigned char far *buf,unsigned word_cnt);
    // read/write DRAM
    void read_DRAM
         (unsigned long addr,unsigned char far *buf,unsigned word_cnt);
    void write_DRAM
	 (unsigned long addr,unsigned char far *buf,unsigned word_cnt);

    // set N64/PC access EEPROM
    // it will delay 100ms to wait relay
    void set_N64_access_EEPROM();
    void set_PC_access_EEPROM();
    // reset EEPROM,disable->enable EEPROM
    void reset_EEPROM();
    // send wait clock for command
    void send_command_wait_clock();
    // send EEPROM physical data bit
    void send_EEPROM_physical_bit(int n);
    // send logical data bit
    void send_EEPROM_logical_bit(int n);
    // send logical data byte
    void send_EEPROM_logical_byte(unsigned char n);
    // receive EEPROM physical data bit
    int recv_EEPROM_physical_bit();
    // receive EEPROM logical data bit
    int recv_EEPROM_logical_bit();
    // receive EEPROM logical data byte
    unsigned char recv_EEPROM_logical_byte();
    // wait EEPROM command acknowledge
    int wait_EEPROM_command_ack();
    // test EEPROM
    int test_EEPROM();
    // read EEPROM page(8 bytes)
    void read_EEPROM_page(int page,unsigned char *buf);
    void read_EEPROM_page(int page,unsigned char *buf,int page_cnt);
    // write EEPROM page(8 bytes)
    void write_EEPROM_page(int page,unsigned char *buf);
    void write_EEPROM_page(int page,unsigned char *buf,int page_cnt);
    // Is N64 power on?
    int is_N64_power_on();
    // Is cartridge plugged?
    int test_cart();
  private:
    // wait 100ms for relay
    void wait_relay();
    unsigned m_chip_ver;
};
// N64 ROM emulator loop
class N64_loop
{
  public:
    void init(unsigned long total_bytes,unsigned blk_bytes);
    int not_end();
    void next();
    unsigned get_blk_size();
    unsigned long get_byte_addr();
    unsigned long get_word_addr();
  private:
    unsigned long m_total_bytes;
    unsigned m_blk_bytes;
    unsigned long m_byte_addr;
};
#endif
