/*---PATCH.CPP---*/
// patch Z64 DRAM case-by-case
#include"n64_emu.h"
#include"z64_gbl.h"
#include"z64_ips.h"
#include"z64_util.h"
#include<dos.h>
#include<fcntl.h>
#include<dirent.h>
#include<dir.h>
#include<string.h>
int check_header_file(const char *fn_buf,const char *hdr)
{
  static char file_hdr[64];
  int fh;
  if(_dos_open(fn_buf,O_RDONLY,&fh)!=0)
  {
    return 0;
  }
  unsigned nr;
  _dos_read(fh,file_hdr,64,&nr);
  if(nr!=64)
  {
    _dos_close(fh);
    return 0;
  }
  _dos_close(fh);
  if(memcmp(hdr,file_hdr,64)==0)  // header data ok
    return 1;
  return 0;
}
int get_patch_filename(const char *hdr,char *fn_buf)
{
  static char prefix[]="A:\\";
  static char drive[MAXDRIVE];
  static char dir[MAXDIR];
  static char file[MAXFILE];
  static char ext[MAXEXT];
  static char tmp_fn_buf[64];
  DIR *pdir;
  dirent *ent;
  // read every .HDR files
  pdir=opendir(prefix);
  if(pdir==NULL)
    return 0;
  while(1)
  {
    ent=readdir(pdir);
    if(ent==NULL)
      break;
    fnsplit(ent->d_name,drive,dir,file,ext);
    // found header file
    if(stricmp(ext,".HDR")!=0)
      continue;
    // read file data
    strcpy(tmp_fn_buf,prefix);
    strcat(tmp_fn_buf,file);
    strcat(tmp_fn_buf,".HDR");
    if(check_header_file(tmp_fn_buf,hdr))
    {
      // header file ok
      strcpy(fn_buf,prefix);
      strcat(fn_buf,file);
      //strcat(fn_buf,".IPS");
      closedir(pdir);
      return 1;
    }
  }
  closedir(pdir);
  return 0;
}
void patch_DRAM()
{
  static char patch_file_name[64];
  static char header_DRAM[64];
  static char header_ROM[64];
  static char full_patch_file_name[64];
  // what is in DRAM/ROM?
  // read 64 bytes from DRAM/ROM
  p_emu->read_DRAM(0,header_DRAM,64/2);
  p_emu->read_ROM(0,header_ROM,64/2);
  // if there is a "special" cartridge in slot, do not patch
  if(get_patch_filename(header_ROM,patch_file_name))
  {
    return;
  }
  // if ROM == DRAM data, do not patch
  if(memcmp(header_DRAM,header_ROM,64)==0)
  {
    return;
  }
  // if file needs patch,do patch
  if(get_patch_filename(header_DRAM,patch_file_name))
  {
    if(p_emu->get_chip_ver()==1)  // for version 1 only
    {
      strcpy(full_patch_file_name,patch_file_name);
      strcat(full_patch_file_name,".IP1");
      if(file_exist(full_patch_file_name))
      {
        do_ips(full_patch_file_name);
        goto patch_ok;
      }
    }
    if(p_emu->get_chip_ver()==2)  // for version 2 only
    {
      strcpy(full_patch_file_name,patch_file_name);
      strcat(full_patch_file_name,".IP2");
      if(file_exist(full_patch_file_name))
      {
        do_ips(full_patch_file_name);
        goto patch_ok;
      }
    }
    // other IPS file
    {
      strcpy(full_patch_file_name,patch_file_name);
      strcat(full_patch_file_name,".IPS");
      if(file_exist(full_patch_file_name))
      {
        do_ips(full_patch_file_name);
        goto patch_ok;
      }
    }
  }
patch_ok:
  // copy boot code
	int i;
	for(i=0;i<4096;i+=512)
		p_emu->read_ROM(i/2,file_buf+i,512/2);
  p_emu->write_DRAM(64/2,file_buf+64,4032/2);
}
