/*
 *  ---Z64.CPP---
 *  Z64 main program
 *  include files
 *
 *  Modified 1.11 Version - PFS-support
 */

#include"z64const.h"
#include"z64_io.h"
#include"lcd_if.h"
#include"z64_util.h"
#include"z64_file.h"
#include"n64_emu.h"
#include"z64_menu.h"
#include"z64del.h"
#include"flash\flash.h"
#include"z64_test.h"
#include"z64_ips.h"
#include"loadbios.h"
#include<stdio.h>
#include<string.h>
#include<bios.h>
#include<dos.h>
#include<fcntl.h>
#include<io.h>
void patch_DRAM();
//
void DoSaveGame(const char *title,unsigned long rom_size_bytes);///
void SaveGameFromRam(const char *title,unsigned long rom_size_bytes);///
// Global objects
Z64_display_device *p_disp;
Z64_keypad_device *p_key;
Z64_file_system *p_filesys;
N64_ROM_emulator *p_emu;
flash_controller *p_fc;
unsigned char far file_buf[file_buf_size];            // file buf
unsigned char far ref_file_buf[file_buf_size];        // Reference file buf
						      // for test/verify
unsigned char data_buf[data_buf_size];
unsigned char eeprom_buf[eeprom_buf_size];
unsigned char prev_eeprom_file_buf[eeprom_buf_size];  // Previous EEPROM
						      // file buffer
unsigned char resv_cart_eeprom_buf[eeprom_buf_size];  // Reserved cartridge
						      // EEPROM buffer
// previous SRAM data
unsigned char huge prev_SRAM_buf[SRAM_size];

// file menu functions
const opt_download_mempak = 0;
const opt_backup_game	  = 1;
const opt_play_cart	  = 2;
const opt_load_file	  = 3;

// select file functions
extern int show_info_shift;
void backup_game();
void playing(const char *title,const char *prompt,int check_disk=0);
void playing_cartridge(const char *title);
int load_file(Z64_file_info &info);
int verify_file(Z64_file_info &info);
void select_load_bios(Z64_file_info &info);
void playing_file(Z64_file_info &info);
void auto_backup_eeprom(Z64_file_info &info);
void restore_cart_eeprom();
void select_file_to_del();

/*--------------------------------------------------------------------------
   Up/Download Mempak
 --------------------------------------------------------------------------*/

void memdump(unsigned long start)
{
   int mloop,k;
   long aktofs = start;

   show_center_str(1,"MEMDUMP");
   show_center_str(2,"DOWN=NEXT, UP=END");

   while (p_key->read_key() == UpKey);
   while ((k = p_key->read_key()) != UpKey)
      if (k == DownKey)
      {
	 p_emu->read_DRAM(aktofs/2,file_buf,6/2); /* /2 = wegen Word */
	 p_disp->clear();
	 for (mloop = 0; mloop < 6; mloop++)
	 {
	    p_disp->gotoxy(1+mloop*3,1);
	    p_disp->printf("%2X,",(unsigned char)file_buf[mloop]);
	    p_disp->gotoxy(1+mloop,2);
	    p_disp->printf("%c",(char)file_buf[mloop]);
	 }
	 p_disp->gotoxy(8,2);
	 p_disp->printf("*%lX",aktofs);
	 aktofs += 6;
      }
}

unsigned long fbyte_to_ulong(unsigned char far *data)
{
   unsigned long temp;

   *((char *)&temp)   = data[3];
   *((char *)&temp+1) = data[2];
   *((char *)&temp+2) = data[1];
   *((char *)&temp+3) = data[0];

   return(temp);
}

void memdump_dword(unsigned long start)
{
   int mloop,k;
   long aktofs = start;

   show_center_str(1,"MEMDUMP-DWORD");
   show_center_str(2,"DOWN=NEXT, UP=END");

   while (p_key->read_key() == UpKey);
   while ((k = p_key->read_key()) != UpKey)
      if (k == DownKey)
      {
	 p_emu->read_DRAM(aktofs/2,file_buf,2);
	 p_disp->clear();
	 p_disp->gotoxy(1,1);
	 p_disp->printf("%ld,",fbyte_to_ulong(file_buf));
	 p_disp->gotoxy(1,2);
	 p_disp->printf("*%lX",aktofs);
	 aktofs += 32;
      }
}

const unsigned long pfsid_addr   = 0x200000/2;
const unsigned long chunk_addr   = 0x200018/2;
const unsigned long pfsdata_addr = 0x200218/2;

int load_z64pfs(const char *const title,const char *upload_fname)
{
   const char *const mempak_rom_fname =
      "Z64PFS.N64";
   const char *const hw1_found_str =
      "ERROR: HW2/3 REQUIRED";
   const char *const z64pfs_missing_str =
      "ERROR: NO Z64PFS.N64";
   const char *const transfer_str =
      "TRANSFER...";

   int fh;
   Z64_file_info zfinfo;
   unsigned long fsize;

   /*
    *  Check if HW2/3
    */

   if ((upload_fname == NULL) && (p_emu->get_chip_ver() == 1))
   {
      show_error_msg_wait_enter(hw1_found_str);
      return(1);
   }

   /*
    *  Set variables
    */

   if (_dos_open(mempak_rom_fname,O_RDONLY,&fh))
   {
      show_error_msg_wait_enter(z64pfs_missing_str);
      return(1);
   }
   lseek(fh,0L,SEEK_END);
   fsize = tell(fh);
   _dos_close(fh);

   zfinfo.set_rom_size(fsize);
   zfinfo.set_file_name(mempak_rom_fname);
   zfinfo.set_type(PFS);
   zfinfo.set_title(title);

   /*
    *  Load ROM
    */

   wait_power_off(title);
   wait_cart();
   load_file(zfinfo);

   if (upload_fname == NULL)
   {
      ((unsigned short *)file_buf)[0] = 0x1234;   // Not the ID, dont
      p_emu->write_DRAM(pfsid_addr,file_buf,1);
   }
   else
   {
      unsigned long cur_dram_pos = pfsid_addr;
      unsigned cur_numbytes;

      _dos_open(upload_fname,O_RDONLY,&fh);
      while (1)
      {
	 _dos_read(fh,file_buf,file_buf_size,&cur_numbytes);
	 if (!cur_numbytes)
	    break;
	 cur_numbytes /= 2;
	 p_emu->write_DRAM(cur_dram_pos,file_buf,cur_numbytes);
	 cur_dram_pos += cur_numbytes;
      }
      _dos_close(fh);
   }
   memdump(pfsid_addr*2);
   p_emu->set_access_ctrl(N64_ROM_emulator::USER_BIOS);
   success_sound();
   wait_power_on(title);
   playing(title,transfer_str,1);

   return(0);
}

void download_mempak(void)
{
   const char *dl_mempak_str = "  DOWNLOAD MEMPAK";

   /*
    *  Dump mempak
    */

   if (load_z64pfs(dl_mempak_str,NULL))
      return;

   /*
    * Check if transfer was successful
    */

   const unsigned long pfsID = 0x31534650;

   memdump(pfsid_addr*2);

   p_emu->read_DRAM(pfsid_addr,file_buf,4/2);
   if (((unsigned long *)file_buf)[0] != pfsID)
      return;

   /*
    *  Set filename+create file
    */

   int loop = 0;
   int fh;
   char fname_buf[13];

   do
   {
      sprintf(fname_buf,"%.8X.PFS",loop);
      loop++;
   } while (_dos_creatnew(fname_buf,_A_NORMAL,&fh));

   show_center_str(2,fname_buf);

   /*
    *  Write PFS to file
    */

   unsigned long datasize[16];
   unsigned long cur_datasize,cur_numbytes,cur_dataofs = pfsdata_addr;
   unsigned numwritten;

   p_emu->read_DRAM(pfsid_addr,file_buf,536/2);
   _dos_write(fh,file_buf,536,&numwritten);

   for (loop = 0; loop < 16; loop++)
      datasize[loop] = fbyte_to_ulong(&file_buf[24+loop*32]);

   for (loop = 0; loop < 16; loop++)
   {
      if (datasize[loop])
      {
	 cur_datasize = datasize[loop];
	 while (cur_datasize)
	 {
	    if (cur_datasize > file_buf_size)
	    {
	       cur_numbytes = file_buf_size;
	       cur_datasize -= file_buf_size;
	    }
	    else
	    {
	       cur_numbytes = cur_datasize;
	       cur_datasize = 0;
	    }
  /*
	    while (p_key->read_key() == UpKey);
	    p_disp->clear();
	    p_disp->gotoxy(1,1);
	    p_disp->printf("%ld,",cur_numbytes);
	    p_disp->gotoxy(1,2);
	    p_disp->printf("%lX,",cur_dataofs*2);

	    while (p_key->read_key() != UpKey);
	    memdump(cur_dataofs*2);
    */
	    p_emu->read_DRAM(cur_dataofs,file_buf,cur_numbytes/2);
	    _dos_write(fh,file_buf,cur_numbytes,&numwritten);
	    cur_dataofs += cur_numbytes/2;
	 }
      }
   }

   _dos_close(fh);
   success_sound();
}

void upload_mempak(const char *pfs_fname)
{
   const char *sure_str      = "SURE?";
   const char *up_mempak_str = "   UPLOAD MEMPAK";

   p_disp->clear();
   p_disp->gotoxy(1,1);
   p_disp->printf("%s",up_mempak_str);
   p_disp->gotoxy(5,2);
   p_disp->printf("%s",sure_str);


   if (ask_yes_no(11,2,1) == 0)
      load_z64pfs(up_mempak_str,pfs_fname);
}

/*-------------------------------------------------------------------------*/

class select_load_file_menu:public select_file_menu
{
  public:
    void run();
    int handle_key(int k,int pos,Z64_file_info &info);
  private:
		long both_key_clk;
};

void select_load_file_menu::run()
{
  both_key_clk=biostime(0,0);
	load_file_menu_list l;
  select_file_menu::run(&l,opt_load_file);
}
int select_load_file_menu::handle_key(int k,int pos,Z64_file_info &info)
{
  if(k==BothKey)
  {
    if(biostime(0,0)-both_key_clk>36)
    {
			success_sound();
      select_file_to_del();
      set_reload();
      p_disp->clear();
      return 0;
		}
  }
  else
  {
    both_key_clk=biostime(0,0);
  }
  if(k!=EnterKey)
		return 0;
  // k== Enter
  // load bios
  if(pos>=opt_load_file && info.get_type()==BIOS_file)
  {
    select_load_bios(info);
    set_reload();
  }

  // upload PFS-file
  else if(pos >= opt_load_file && info.get_type() == PFS)
  {
     upload_mempak(info.get_file_name());
  }

  // load game file
  else if(pos>=opt_load_file)
  {
    int ret;
    wait_power_off(get_center_title(info.get_title()));
    wait_cart();
    ret=load_file(info);
    if(ret!=0)
			return 0;
    // set N64 access DRAM
    if(p_emu->get_chip_ver()==1)  // ver 1
    {
      p_emu->set_access_ctrl(N64_ROM_emulator::N64_DRAM);
    }
    else  // ver 2 or higher
    {
      if(info.get_rom_size()<=16L*1024*1024)
      {
	// only 16M ROM, simulate SRAM
	p_emu->set_access_ctrl(N64_ROM_emulator::N64_16M_ROM_SRAM);
      }
      else
      {
	// more than 16M ROM
	p_emu->set_access_ctrl(N64_ROM_emulator::N64_32M_ROM);
      }
    }
    wait_power_off(get_center_title(info.get_title()));
    success_sound();
    //wait_power_on(get_center_title(info.get_title()));
		// -- start
    if(wait_power(get_center_title(info.get_title()),3)) ///
    {
      DoSaveGame(info.get_title(),info.get_rom_size());
      set_reload();
      return 0;
    }
		// -- end
    playing_file(info);
    if(p_key->read_key()==BothKey)
    {
      verify_file(info);
    }
    auto_backup_eeprom(info);
    restore_cart_eeprom();
    if(p_emu->get_chip_ver()>1)
      auto_backup_SRAM(info);
		// Disable N64 to access DRAM
    p_emu->set_access_ctrl(N64_ROM_emulator::PC_ROM);
    success_sound();
  }
  // Download mempak
  else if (pos == opt_download_mempak)
  {
     download_mempak();
     set_reload();  // Ntig da eine Datei erstellt wurde
  }
  // play cartidge
  else if(pos==opt_play_cart)
	{
    char title[20];
    wait_power_off(get_center_str("PLAY CARTRIDGE"));
    wait_cart();
    p_emu->read_ROM(0x10,title,10);
    p_emu->set_access_ctrl(N64_ROM_emulator::N64_ROM);
    wait_power_on(get_center_str("PLAY CARTRIDGE"));
		playing_cartridge(title);
  }
  // backup game
  else if(pos==opt_backup_game)
  {
    backup_game();
    set_reload();
  }
  p_disp->clear();
	return 0;
}
// Reserve/restore EEPROM
void resv_cart_eeprom()
{
  show_center_str(2,"RESERVE RECORD...");
  // Read cartridge EEPROM
  p_emu->set_PC_access_EEPROM();
	p_emu->reset_EEPROM();
  if(p_emu->test_EEPROM())
  {
    p_emu->set_N64_access_EEPROM();
    return;  // EEPROM not installed
  }
  p_emu->read_EEPROM_page(0,resv_cart_eeprom_buf,eeprom_buf_size/8);
}
void restore_cart_eeprom()
{
  show_center_str(2,"RESTORE RECORD...");
  // Write to cartridge EEPROM
  p_emu->set_PC_access_EEPROM();
	p_emu->reset_EEPROM();
  if(p_emu->test_EEPROM())
  {
    p_emu->set_N64_access_EEPROM();
    return;  // EEPROM not installed
  }
	p_emu->write_EEPROM_page(0,resv_cart_eeprom_buf,eeprom_buf_size/8);
}
// file load functions
int load_file(Z64_file_info &info)
{
  // load rom file
  p_disp->clear();
  show_file_title(info.get_title());
  show_center_str(2,"LOADING...");
  if(p_filesys->open_rom_file_read(info)!=0)
	{
    load_file_error();
    return 1;  // Open file error
  }
  unsigned long rom_size_bytes;
  rom_size_bytes = info.get_rom_size();
  process_file_init_progress(rom_size_bytes);
  N64_loop loop;
  for(loop.init(rom_size_bytes,file_buf_size);loop.not_end();loop.next())
  {
    // read file data
    unsigned long remain_size;
    unsigned req_read_size;
    remain_size=rom_size_bytes-loop.get_byte_addr();
    req_read_size=file_buf_size;
    if(remain_size<file_buf_size)
    {
      req_read_size=remain_size;
      printf("read small buffer:%u\n",req_read_size);
    }
      if(p_filesys->read_rom_file(file_buf,req_read_size,info.get_type())!=0)
		{
      p_filesys->close_rom_file();
      load_file_error();
      return 1;  // Load error
    }
    // write DRAM
    p_emu->write_DRAM(loop.get_word_addr(),file_buf,file_buf_size/2);
    // print progress
		unsigned long nsize;
    nsize=loop.get_byte_addr()+loop.get_blk_size();
    process_file_update_progress(nsize);
    // Check user break
    if(p_key->read_key()==EnterKey)
		{
			clear_file_update_progress();
      p_filesys->close_rom_file();
			return 1;  // Break
		}
	}
	p_filesys->close_rom_file();
  // Padding 00 until 1M+4096Byte
  const long pad_null_length=1024L*1024L+4096;
  if(rom_size_bytes<pad_null_length)
  {
    long pad_ptr;
    pad_ptr=rom_size_bytes;
    _fmemset(file_buf,0,file_buf_size);
    while(pad_ptr<pad_null_length)
    {
      p_emu->write_DRAM(pad_ptr/2,file_buf,file_buf_size/2);
      pad_ptr+=file_buf_size;
    }
  }

  // return if PFS-manager
  if (info.get_type() == PFS)
     return 0;

  // patch security
  patch_DRAM();
  // ask to load IPS file
  ask_load_IPS_file(info);
  // before load EEPROM file,reserve cartridge EEPROM
  resv_cart_eeprom();
  // load EEPROM file
  if(p_filesys->is_eeprom_file_exist(info))
  {
    show_center_str(2,"LOAD RECORD...");
    // Read from file
    if(p_filesys->open_eeprom_file_read(info)!=0)  // NO EEPROM file,skip it
    {
      load_file_error();
			return 1;  // Open error
    }
    if(p_filesys->read_eeprom_file(eeprom_buf,eeprom_buf_size)!=0)
    {
      p_filesys->close_eeprom_file();
      load_file_error();
      return 1;  // read error
    }
    p_filesys->close_eeprom_file();
    // Write to cartridge EEPROM
		p_emu->set_PC_access_EEPROM();
		p_emu->reset_EEPROM();
    if(p_emu->test_EEPROM())
		{
      // EEPROM not installed
      p_emu->set_N64_access_EEPROM();
      no_eeprom_warning();
    }
    else
    {
			p_emu->write_EEPROM_page(0,eeprom_buf,eeprom_buf_size/8);
    }
    p_emu->set_N64_access_EEPROM();
    memcpy(prev_eeprom_file_buf,eeprom_buf,eeprom_buf_size);
  }
  // load SRAM file
  if((p_emu->get_chip_ver()>1) && p_filesys->is_sram_file_exist(info))
  {
    show_center_str(2,"LOAD SRAM...");
    // Read from file
    if(p_filesys->open_sram_file_read(info)!=0)  // NO EEPROM file,skip it
    {
      load_file_error();
			return 1;  // Open error
    }
    load_SRAM_file();
  }
  // save SRAM
  if(p_emu->get_chip_ver()>1)
  {
    save_original_DRAM_mapped_SRAM();
  }
  return 0;
}
int verify_file(Z64_file_info &info)
{
	// load rom file
	p_disp->clear();
  show_file_title(info.get_title());
	show_center_str(1,"VERIFY...");
  if(p_filesys->open_rom_file_read(info)!=0)
  {
    load_file_error();
    return 1;  // Open file error
  }
  unsigned long rom_size_bytes;
	rom_size_bytes=info.get_rom_size();
  process_file_init_progress(rom_size_bytes);
  N64_loop loop;
  for(loop.init(rom_size_bytes,file_buf_size);loop.not_end();loop.next())
  {
    // read file data
    if(p_filesys->read_rom_file(file_buf,file_buf_size,info.get_type())!=0)///
    {
      p_filesys->close_rom_file();
      load_file_error();
			return 1;  // Load error
		}
    // read DRAM
		p_emu->read_DRAM(loop.get_word_addr(),ref_file_buf,file_buf_size/2);
    // test
    if(_fmemcmp(file_buf,ref_file_buf,file_buf_size)!=0)
    {
      mem_err(loop.get_byte_addr(),file_buf_size,file_buf,ref_file_buf);
    }
    // print progress
		unsigned long nsize;
    nsize=loop.get_byte_addr()+loop.get_blk_size();
    process_file_update_progress(nsize);
  }
  p_filesys->close_rom_file();
  show_msg_wait_enter("VERIFY OK");
  return 0;
}
void copy_bios_file();
void update_bios_file();
void select_load_bios(Z64_file_info &info)
{
  static option load_bios_option[]=
  {
    {2,"UPDATE"},
    {9,"COPY"},
    {14,"CANCEL"}
  };
  p_disp->clear();
  show_file_title(info.get_title());
  const opt_num=sizeof(load_bios_option)/sizeof(load_bios_option[0]);
  int sel;
  sel=select_option(load_bios_option,opt_num,2);
  if(sel==2)
    return;
  if(sel==1)
  {
    copy_bios_file();
    return;
  }
  if(sel==0)
  {
    update_bios_file();
    return;
  }
}
struct mid
{
  int midInfoLevel;
	long midSerialNum;
  char midVolLabel[11];
	char midFileSysType[8];
};
struct mid *get_media_id(int drv,struct mid *p_mid)
{
  REGS reg;
  reg.x.ax=0x440d;
  reg.x.bx=drv;
	reg.x.cx=0x0866;
  reg.x.dx=FP_OFF(p_mid);
  p_mid->midInfoLevel=0;
  int86(0x21,&reg,&reg);
  if(reg.x.cflag)
    return NULL;
  return p_mid;
}
void wait_disk(mid *pm)
{
  const char prompt[]="INSERT DISK";
	p_disp->clear();
  show_center_str(2,prompt);
	error_sound();
  p_fc->set_status(1);
  while(1)
  {
    p_key->read_key();
    int is_changed,new_status;
    p_fc->get_status(is_changed,new_status);
    if(is_changed)
    {
      if(new_status)
      {
        show_center_str(2,prompt);
        if(p_filesys->is_device_ready())
        {
          if(pm==NULL)
            break;
          mid m;
          if(get_media_id(0,&m)!=NULL && memcmp(&m,pm,sizeof(mid))==0)
            break;
         }
      }
      else
        show_center_str(2,"");
    }
  }
}
void playing(const char *title,const char *prompt,int check_disk)
{
  mid m,*pm;
  if(check_disk)
  {
    pm=get_media_id(0,&m);
  }
  p_disp->clear();
  show_file_title(title);
  show_center_str(2,prompt);
  p_fc->set_status(1);
  int len;
	len=strlen(prompt);
  int x_pt;
	x_pt=(20-len)/2+len+1;
  if(x_pt>20)
    x_pt=20;
  // check power
  long key_pressed_clk;
  int show_key_msg;
  show_key_msg=0;
  while(p_emu->is_N64_power_on()==1)
  {
    if(check_disk && p_filesys->is_device_ready()==0)
    {
      wait_disk(pm);
      p_disp->clear();
      show_file_title(title);
      show_center_str(2,prompt);
      p_fc->set_status(1);
    }
    if(p_key->read_key()!=NoKey && show_key_msg==0)
		{
      key_pressed_clk=biostime(0,0);
			show_key_msg=1;
      show_center_str("TURN OFF N64\nTO SELECT GAME");
    }
    else if(show_key_msg && biostime(0,0)-key_pressed_clk>36)
    {
      show_key_msg=0;
      p_disp->clear();
      show_file_title(title);
      show_center_str(2,prompt);
      p_fc->set_status(1);
    }
    if(show_key_msg==0)
    {
      int is_changed,new_status;
      p_fc->get_status(is_changed,new_status);
      if(is_changed)
      {
        p_disp->gotoxy(x_pt,2);
				if(new_status)
          p_disp->printf("_");
        else
          p_disp->printf(" ");
      }
    }
  }
  // Disable N64 to access DRAM
  p_emu->set_access_ctrl(N64_ROM_emulator::PC_ROM);
}
void playing_cartridge(const char *title)
{
  playing(title,"PLAYING CARTRIDGE...");
}
void playing_file(Z64_file_info &info)
{
  playing(info.get_title(),"PLAYING FILE...",1);
}
void auto_backup_eeprom(Z64_file_info &info)
{
	const char *cmp_eeprom_buf;
  if(!p_filesys->is_eeprom_file_exist(info))
	{
    cmp_eeprom_buf=resv_cart_eeprom_buf;
  }
  else
  {
    cmp_eeprom_buf=prev_eeprom_file_buf;
  }
  // EEPROM file present
  p_disp->clear();
  show_file_title(info.get_title());
  show_center_str(2,"CHECKING RECORD...");
  // Is EPROM data changed?
  // read EEPROM chip
  p_emu->set_PC_access_EEPROM();
  p_emu->reset_EEPROM();
  if(p_emu->test_EEPROM())
  {
    return;  // No EEPROM chip $$$
	}
  p_emu->read_EEPROM_page(0,eeprom_buf,eeprom_buf_size/8);
	p_emu->set_N64_access_EEPROM();
  // Compare
  if(_fmemcmp(eeprom_buf,cmp_eeprom_buf,eeprom_buf_size)==0)
    return;
  // Changed,ask to update EEPROM file
  p_disp->clear();
  show_file_title(info.get_title());
  p_disp->gotoxy(1,2);
  p_disp->printf("UPDATE RECORD?");
  prompt_sound();
  int ret;
  ret=ask_yes_no(15,2);
  if(ret==0)  // Yes
  {
    if(p_filesys->open_eeprom_file_write(info)!=0)
    {
      write_file_error();
      return;  // Open error
		}
    if(p_filesys->write_eeprom_file(eeprom_buf,eeprom_buf_size)!=0)
		{
      p_filesys->close_eeprom_file();  // write error
      write_file_error();
      return;
    }
    p_filesys->close_eeprom_file();
    p_disp->clear();
    show_file_title(info.get_title());
    show_center_str(2,"UPDATE RECORD OK");
  }
}
// select file to load
void select_file_to_load()
{
  select_load_file_menu m;
  m.run();
}
void select_file_to_del()
{
  select_del_file_menu m;
	m.run();
}
// is buffer empty?
int is_buf_no_data(unsigned char *buf,unsigned size)
{
  int i;
  for(i=0;i<size;i++)
    if(buf[i]!=0xff)
      return 0;
  return 1;
}
// get rom size
unsigned long get_rom_size()
{
  // Get ROM size
  unsigned long rom_max_size;
  if(p_emu->get_chip_ver()>1)
    rom_max_size=32L*1024*1024;  // Max 32MB
  else
    rom_max_size=16L*1024*1024;  // Max 16MB
  printf("rom max size=%ld\n",rom_max_size);
  unsigned long rom_size_bytes;
  rom_size_bytes=8L*1024*1024;  // assume 8MB
  while(rom_size_bytes<rom_max_size)
	{
    // try assumed ROM size
		p_emu->read_ROM(rom_size_bytes/2,data_buf,data_buf_size/2);
    if(is_buf_no_data(data_buf,data_buf_size))
      return rom_size_bytes;         // Ok,correct size
    else
      rom_size_bytes+=4L*1024*1024;  // No correct,add 4MB
  }
  return rom_size_bytes;
}
// test if cartridge SRAM exist
int test_cart_SRAM_exist()
{
  if(p_emu->get_chip_ver()==1)
    return 0;
  printf("test cart sram\n");
  unsigned m,n;
  p_emu->read_cart_SRAM(0,(char*)&m,1);   // read original data
  m=~m;
  p_emu->write_cart_SRAM(0,(char*)&m,1);  // write complement data
  p_emu->read_cart_SRAM(0,(char*)&n,1);   // read it back
  m=~m;
  p_emu->write_cart_SRAM(0,(char*)&m,1);  // retore original data
  printf("m=%04x,n=%04x\n",m,n);
  if(~m==n)
  {
    return 1;
  }
  return 0;
}
// backup game
void backup_game()
{
  wait_power_off(get_center_str("BACKUP CARTRIDGE"));
  wait_cart();
  // Get cartridge title
  char title[20];
  p_emu->read_ROM(0x10,title,10);
  // Get cartridge size
  unsigned long rom_size_bytes;
	rom_size_bytes=get_rom_size();
  // Prompt user
	p_disp->clear();
  show_file_title(title);
  p_disp->gotoxy(1,2);
  p_disp->printf("%3dM BACKUP?",rom_size_bytes/megabit);
  int ret;
  ret=ask_yes_no(14,2,1);
  if(ret!=0)
    return;
  // Check disk space
  p_disp->clear();
  show_center_str(2,"CHECK DISK SPACE");
  long disk_space;
  disk_space=p_filesys->get_disk_space();
  if(disk_space==-1)
  {
    // disk not ready/error
    disk_not_ready();
    return;
	}
  if(rom_size_bytes+eeprom_buf_size>disk_space)
	{
    // space not enough
    disk_space_not_enough();
    return;
  }
  // test cartridge chksum
	/*if(test_cart())
	{
		show_error_msg_wait_enter("CARTRIDGE ROM\nREAD ERROR");
		return;
	}*/
  // Ok,begin to backup
  Z64_new_file nf;
  p_filesys->get_new_file(nf);
  if(p_filesys->open_rom_file_create(nf)!=0)
  {
    p_filesys->delete_new_file(nf);
    write_file_error();
		return;  // Unable to open
  }
	p_disp->clear();
  show_file_title(title);
  p_disp->gotoxy(1,2);
  p_disp->printf("BACK UP...");
  // Backup ROM
  process_file_init_progress(rom_size_bytes);
  N64_loop loop;
  for(loop.init(rom_size_bytes,file_buf_size);loop.not_end();loop.next())
  {
    // read ROM
    unsigned file_buf_ptr;
    for(file_buf_ptr=0;file_buf_ptr<file_buf_size;file_buf_ptr+=data_buf_size)
    {
      p_emu->read_ROM(loop.get_word_addr()+file_buf_ptr/2,file_buf+file_buf_ptr,data_buf_size/2);
    }
    // Write file
    if(p_filesys->write_rom_file(file_buf,file_buf_size)!=0)
    {
			p_filesys->close_rom_file();
      p_filesys->delete_new_file(nf);
			write_file_error();
      return;  // Write error
    }
    // print progress
    unsigned long nsize;
    nsize=loop.get_byte_addr()+loop.get_blk_size();
    if((nsize%32768)==0)
      process_file_update_progress(nsize);
    // Check user break
    if(p_key->read_key()==EnterKey)
    {
      clear_file_update_progress();
      p_filesys->close_rom_file();
      p_filesys->delete_new_file(nf);
      return;  // Break
    }
  }
  // Backup ROM ok
  p_filesys->close_rom_file();
  // Backup EEPROM
  p_emu->set_PC_access_EEPROM();
  p_emu->reset_EEPROM();
  if(p_emu->test_EEPROM())
  {
    p_emu->set_N64_access_EEPROM();  // No EEPROM,don't save
  }
  else
  {
    // EEPROM present,save it
    show_center_str(2,"BACKUP RECORD...");
    if(p_filesys->open_eeprom_file_create(nf)!=0)
    {
      p_emu->set_N64_access_EEPROM();
      p_filesys->delete_new_file(nf);
      write_file_error();
      return;  // Unable to open
    }
    p_emu->read_EEPROM_page(0,eeprom_buf,eeprom_buf_size/8);
		if(p_filesys->write_eeprom_file(eeprom_buf,eeprom_buf_size)!=0)
    {
			p_emu->set_N64_access_EEPROM();
      p_filesys->close_eeprom_file();
      p_filesys->delete_new_file(nf);
      write_file_error();
      return; // Unable to write
    }
    p_filesys->close_eeprom_file();
    p_emu->set_N64_access_EEPROM();
  }
  if(test_cart_SRAM_exist())
  {
    // SRAM present,save it
    show_center_str(2,"BACKUP SRAM...");
    if(p_filesys->open_sram_file_create(nf)!=0)
    {
      p_filesys->delete_new_file(nf);
      write_file_error();
      return;  // Unable to open
    }
    if(write_cart_SRAM_to_SRAM_file())
    {
      p_filesys->delete_new_file(nf);
      return;
    }
  }
  // Backup completed
  p_disp->clear();
  show_file_title(title);
  success_sound();
  show_msg_wait_enter("BACKUP COMPLETED");
}
#pragma -N-
int handler(int,int,int,int)
{
  // report any disk error
	hardresume(_HARDERR_FAIL);
	while(1);
}
void show_exe_time()
{
  // display EXE file time
  int fh;
  _dos_open(_argv[0],O_RDONLY,&fh);
  struct ftime ft;
  getftime(fh,&ft);
  _dos_close(fh);
  printf("EXE time:%u:%u:%u,",
         ft.ft_hour, ft.ft_min,
         ft.ft_tsec * 2);
  printf("date:%u/%u/%u\n",
         ft.ft_month, ft.ft_day,
         ft.ft_year+1980);
}
// main
//#define _USE_VGA
void main(int argc,char **argv)
{
#ifdef _USE_VGA
	Z64_display_VGA disp;
#else
	Text_LCD_if_8255 if_8255;
  Z64_display_LCD disp(&if_8255);
#endif
  p_disp=&disp;
#ifdef _USE_VGA
  Z64_keypad_keyboard keypad;
#else
  Z64_keypad_8255 keypad;
#endif
  p_key=&keypad;
  // get chip version;
  unsigned chip_ver;
  chip_ver=get_chip_ver();
  printf("* Modified version - PFS-support!");
  printf("chip=%d\n",chip_ver);
  // init z64 chip
  N64_ROM_emulator emu;
  p_emu=&emu;
  p_emu->set_chip_ver(chip_ver);
  // Boot prompt
#ifndef _USE_VGA
	if(argc>1 && argv[1][0]=='b')
  {
    // test LCD
    test_LCD(if_8255);
    disp.init();
    disp.clear();
    // self test
    if(p_key->read_key()==DownKey)
    {
      self_test();  // do complete test
    }
    else
    {
      test_z64_chip();  // test control chip only
    }
    // show boot message
    char ver_buf[64];
    char chip_char;
    chip_char=p_emu->get_chip_ver()-1+'A';
    show_center_str(1,"MR. BACKUP  Z64");
    sprintf(ver_buf,"V1.11/PFS - HW%",chip_char);
    show_center_str(2,ver_buf);
    return;
  }
#endif
  show_exe_time();
  // test zip
  if(_getdrive()==1)
  {
    printf("disk drive error\n");
    clear_line(1);
    show_center_str(2,"DISK DRIVE ERROR");
    test_error();
  }
  // ok,init
  Z64_file_system filesys;
  p_filesys=&filesys;
  p_fc=new flash_controller(3,9);
  harderr(handler);
  p_filesys->is_device_ready();
	p_filesys->get_disk_space();
  success_sound();
  select_file_to_load();
}
// --  start
#include <dir.h>
void Refresh()
{
  REGS reg;
  reg.x.ax=0x0d00;
  int86(0x21,&reg,&reg);
  char buf[64];
  getcwd(buf,sizeof(buf));
}
void DoSaveGame(const char *title,unsigned long rom_size_bytes)
{
	int is_changed,new_status;
  int dev_rdy=p_filesys->is_device_ready();
  while(dev_rdy==p_filesys->is_device_ready())
  {
	p_fc->get_status(is_changed,new_status);
    if(is_changed)
    {
	if(new_status)
	show_center_str(2,"REMOVE DISK...");
	else
		    show_center_str(2,"");
		}
    if(p_key->read_key()!=NoKey)
      return;
  }
  dev_rdy=p_filesys->is_device_ready();
  while(dev_rdy==p_filesys->is_device_ready())
  {
	p_fc->get_status(is_changed,new_status);
    if(is_changed)
    {
	    if(new_status)
	show_center_str(2,"INSERT DISK...");
	    else
	      show_center_str(2,"");
	}
	  if(p_key->read_key()!=NoKey)
      return;
  }
  Refresh();
	SaveGameFromRam(title,rom_size_bytes);
}
void SaveGameFromRam(const char *title,unsigned long rom_size_bytes)
{
  //wait_power_off(get_center_str("BACKUP CARTRIDGE"));
  //wait_cart();
  // Get cartridge title
  //char title[20];
  //p_emu->read_ROM(0x10,title,10);
  // Get cartridge size
  //unsigned long rom_size_bytes;
  //rom_size_bytes=get_file_size();
  // Prompt user
  p_disp->clear();
  show_file_title(title);
  p_disp->gotoxy(1,2);
  p_disp->printf("%3dM SAVE?",rom_size_bytes/megabit);//rom_size_bytes/megabit);
  int ret;
  ret=ask_yes_no(14,2,1);
  if(ret!=0)
		return;
  // Check disk space
  p_disp->clear();
  show_center_str(2,"CHECK DISK SPACE");
  long disk_space;
  disk_space=p_filesys->get_disk_space();
  if(disk_space==-1)
  {
    // disk not ready/error
    disk_not_ready();
    return;
  }
  if(rom_size_bytes+eeprom_buf_size>disk_space)
  {
    // space not enough
    disk_space_not_enough();
    return;
  }
  // test cartridge chksum
	//if(test_cart())
	//{
  //  show_error_msg_wait_enter("CARTRIDGE ROM\nREAD ERROR");
  //  return;
  //}
  // Ok,begin to backup
  Z64_new_file nf;
  p_filesys->get_new_file(nf);
  if(p_filesys->open_rom_file_create(nf)!=0)
  {
    p_filesys->delete_new_file(nf);
    write_file_error();
    return;  // Unable to open
  }
  p_disp->clear();
  show_file_title(title);
  p_disp->gotoxy(1,2);
  p_disp->printf("SAVING...");
  // Backup ROM
  process_file_init_progress(rom_size_bytes);
  N64_loop loop;
	for(loop.init(rom_size_bytes,file_buf_size);loop.not_end();loop.next())
  {
    // read ROM
    unsigned file_buf_ptr;
    for(file_buf_ptr=0;file_buf_ptr<file_buf_size;file_buf_ptr+=data_buf_size)
    {
      //p_emu->read_ROM(loop.get_word_addr()+file_buf_ptr/2,file_buf+file_buf_ptr,data_buf_size/2);
      p_emu->read_DRAM(loop.get_word_addr()+file_buf_ptr/2,file_buf+file_buf_ptr,data_buf_size/2);
    }
    // Write file
    if(p_filesys->write_rom_file(file_buf,file_buf_size)!=0)
    {
      p_filesys->close_rom_file();
      p_filesys->delete_new_file(nf);
      write_file_error();
      return;  // Write error
    }
    // print progress
    unsigned long nsize;
    nsize=loop.get_byte_addr()+loop.get_blk_size();
		if((nsize%32768)==0)
      process_file_update_progress(nsize);
    // Check user break
    if(p_key->read_key()==EnterKey)
    {
      clear_file_update_progress();
      p_filesys->close_rom_file();
      p_filesys->delete_new_file(nf);
      return;  // Break
    }
  }
  // Backup ROM ok
  p_filesys->close_rom_file();
  // Backup completed
  p_disp->clear();
	show_file_title(title);
	success_sound();
	show_msg_wait_enter("SAVE COMPLETED");
}
// --  end
