/*---Z64_FILE.H---*/
#ifndef _INC_Z64_FILE
#define _INC_Z64_FILE
#include"dosfile.h"
#include"aspi\aspidrv.h"
// file format
enum {
   Z64,V64,BIOS_file,
   PFS
};

// ROM file infomation(ROM/V64)
class Z64_file_info
{
  public:
    Z64_file_info();
    void set_title(const char *buf);
    const char *get_title() const;
    void set_rom_size(unsigned long size);
    unsigned long get_rom_size() const;
    void set_file_name(const char *fn);
    const char *get_file_name() const;
    void set_country(char c);
    char get_country() const;
    void set_type(int type);
    int get_type() const;
    void set_bios_ver(unsigned ver);
    unsigned get_bios_ver() const;
  private:
    char m_title[20];
    unsigned long m_rom_size;
    char m_fn[16];
    char m_country;
    int m_type;
    unsigned m_bios_ver;
};
// new ROM file infomation
class Z64_new_file
{
  public:
    Z64_new_file();
    void set_file_name(const char *fn);
    const char *get_file_name();
  private:
    char m_fn[16];
};
// ROM file system
class Z64_file_system
{
  public:
    Z64_file_system();
    int is_device_ready();
    long get_disk_space();
    int get_file_list(Z64_file_info *info_buf,int buf_num,int &info_num);
    int open_rom_file_read(Z64_file_info &info_buf);
    int read_rom_file(char far *buf,unsigned len,int type);///
    void close_rom_file();
    int is_eeprom_file_exist(Z64_file_info &info_buf);
    int open_eeprom_file_read(Z64_file_info &info_buf);
    int read_eeprom_file(char far *buf,unsigned len);
    void close_eeprom_file();
    int get_new_file(Z64_new_file &nf);
    int open_rom_file_create(Z64_new_file &nf);
    int write_rom_file(const char far *buf,unsigned len);
    int open_eeprom_file_write(Z64_file_info &info_buf);
    int open_eeprom_file_create(Z64_new_file &nf);
    int write_eeprom_file(const char far *buf,unsigned len);
    int is_sram_file_exist(Z64_file_info &info);
    int open_sram_file_read(Z64_file_info &info);
    int open_sram_file_write(Z64_file_info &info);
    int open_sram_file_create(Z64_new_file &nf);
    int read_sram_file(char far *buf,unsigned len);
    int write_sram_file(const char far *buf,unsigned len);
    void close_sram_file();
    int delete_new_file(Z64_new_file &nf);
    void delete_file(Z64_file_info &info);
    int open_bios_file_read();
    int open_bios_file_create();
    int seek_bios_file(long offset);
    int read_bios_file(char far *buf,unsigned len);
    int write_bios_file(const char far *buf,unsigned len);
    int close_bios_file();
    int test_bios_file(unsigned *ver);
    int get_bios_chksum(unsigned *sum);
  private:
    Std_ASPI_Driver m_aspidrv;
    int use_zip;
    int m_rom_open;
    int m_rom_fh;
    int m_eeprom_open;
    int m_eeprom_fh;
    int m_bios_open;
    int m_bios_fh;
    dos_file m_sram_file;
};
#endif
