/*---Z64_IO.CPP---*/
#include"z64_io.h"
#include<conio.h>
#include<stdarg.h>
#include<stdio.h>
#include<stdlib.h>
#include<bios.h>
#include<dos.h>
// Z64_display_VGA
void Z64_display_VGA::init()
{
  clrscr();
}
void Z64_display_VGA::clear()
{
  clrscr();
}
void Z64_display_VGA::gotoxy(int x,int y)
{
  ::gotoxy(x,y);
}
void Z64_display_VGA::printf(const char *fmt,...)
{
  char buffer[80];
  va_list argptr;
  int cnt;
  va_start(argptr, fmt);
  vsprintf(buffer, fmt, argptr);
  cprintf(buffer);
  va_end(argptr);
}
void Z64_display_VGA::read(char *buf,int sz)
{
  int i;
  for(i=0;i<sz;i++)
    buf[i]=' ';
}
// Z64_display_LCD
Z64_display_LCD::Z64_display_LCD(Text_LCD_interface *pif):m_lcd(pif)
{
}
void Z64_display_LCD::init()
{
  Text_LCD_config cfg;
  cfg.cursor_on=0;
  cfg.display_lines=2;
  m_lcd.set_config(cfg);
  m_lcd.clear_screen();
}
void Z64_display_LCD::clear()
{
  m_lcd.clear_screen();
}
void Z64_display_LCD::gotoxy(int x,int y)
{
  m_lcd.gotoxy(x,y);
}
void Z64_display_LCD::printf(const char *fmt,...)
{
  char buffer[80];
  va_list argptr;
  va_start(argptr, fmt);
  vsprintf(buffer, fmt, argptr);
  va_end(argptr);
  m_lcd.printf(buffer);
}
void Z64_display_LCD::read(char *buf,int sz)
{
  m_lcd.read_str(buf,sz);
}
void Z64_display_LCD::set_font(int code,char *buf)
{
  int i;
  for(i=0;i<8;i++)
  {
    m_lcd.set_CG_RAM_addr(i+code*8);
    m_lcd.write_char(buf[i]);
  }
}
// Z64_keypad_keyboard
int Z64_keypad_keyboard::read_key()
{
  if(kbhit()==0)
    return NoKey;
  int k;
  k=getch();
  if(k==27)
    exit(0);
  if(k==13)
    return EnterKey;
  if(k==32)
    return BothKey;
  if(k==0 || k==0xe0)
  {
    k=getch();
    if(k==72)
      return UpKey;
    if(k==80)
      return DownKey;
  }
  return NoKey;
}
// Z64_keyboard_8255
Z64_keypad_8255::Z64_keypad_8255():m_wait_release_key(0)
{
}
int Z64_keypad_8255::read_key()
{
  if(kbhit() && getch()==27)
    exit(0);
  unsigned char k;
  k=read_keyswitch_debounce()&0x7;
  if(m_wait_release_key)
  {
    if(k==0x7)  // Key released
      m_wait_release_key=0;                 // Accept new key
    if((k&0x6)==0)
      return BothKey;
    return NoKey;
  }
  else
  {
    if(k!=0x7)  // Key pressed
    {
      key_beep();
      m_wait_release_key=1;                 // Wait user release key
      if((k&1)==0)
        return EnterKey;
      if((k&4)==0)
        return UpKey;
      if((k&2)==0)
        return DownKey;
    }
    return NoKey;
  }
}
unsigned char Z64_keypad_8255::read_keyswitch()
{
  return inp(0x16a) & 0x1f;
}
unsigned char Z64_keypad_8255::read_keyswitch_debounce()
{
  const test_cnt_max=100;
  int test_cnt;
  unsigned char keysw,prev_keysw=0;
  for(test_cnt=0;test_cnt<test_cnt_max;test_cnt++)
  {
    keysw=read_keyswitch();
    if(keysw!=prev_keysw)  // If key switch is changed
    {
      prev_keysw=keysw;    // Retry this debounce loop
      test_cnt=0;
    }
  }
  return keysw;            // Key switch is stable,ok
}
void Z64_keypad_8255::key_beep()
{
  long clk;
  clk=biostime(0,0);
  while(biostime(0,0)==clk);
  sound(4000);
  clk=biostime(0,0);
  while(biostime(0,0)==clk);
  nosound();
}
