/*---Z64_IPS.CPP---*/
#include"ips_dec.h"
#include"z64const.h"
#include"z64_gbl.h"
#include"z64_util.h"
#include<stdio.h>
#include<dos.h>
#include<io.h>
#include<fcntl.h>
#include<string.h>
// write signed high/low byte to DRAM
void write_DRAM_low_byte(unsigned long word_addr,unsigned char data)
{
  unsigned char prev_data[2];
  p_emu->read_DRAM(word_addr,prev_data,1);
  prev_data[0]=data;
  p_emu->write_DRAM(word_addr,prev_data,1);
}
void write_DRAM_high_byte(unsigned long word_addr,unsigned char data)
{
  unsigned char prev_data[2];
  p_emu->read_DRAM(word_addr,prev_data,1);
  prev_data[1]=data;
  p_emu->write_DRAM(word_addr,prev_data,1);
}
// fill DRAM data(v64->z64)
void do_v64_fill_mem(unsigned long addr,unsigned size,unsigned char data)
{
  // expand 8 bit to 16 bit
  unsigned char data_16bit[2];
  data_16bit[0]=data;
  data_16bit[1]=data;
  while(size!=0)
  {
    // odd address
    if(addr&1)
    {
      write_DRAM_low_byte(addr/2,data);
      addr++;
      size--;
      continue;
    }
    // last single byte
    if(size==1)
    {
      write_DRAM_high_byte(addr/2,data);
      addr++;
      size--;
      continue;
    }
    // handle 16 bit at once
    p_emu->write_DRAM(addr/2,data_16bit,1);
    addr+=2;
    size-=2;
  }
}
// swap high/low byte
void swap_high_low_byte(unsigned char far *buf,unsigned size)
{
  if(size==0)
    return;
  _asm{
    pushf
    push ds
    push es
    push si
    push di
    mov cx,size
    shr cx,1
    les di,buf
    lds si,buf
    cld
  }
  x:
  _asm{
    lodsw
    xchg ah,al
    stosw
    loop x
    pop di
    pop si
    pop es
    pop ds
    popf
  }
}
// copy memory data(v64->z64)
void do_v64_copy_mem(unsigned long addr,unsigned size,unsigned char far *buf)
{
  //unsigned char *orgp;
  //orgp=buf;
  //printf("do copy:%ld %d\n",addr,size);
  // odd address
  if(addr&1)
  {
    //printf("odd addr,addr=%ld dp=%u\n",addr,buf-orgp);
    write_DRAM_low_byte(addr/2,*buf);
    addr++;
    size--;
    buf++;
  }
  // swap high/low byte in data buffer
  unsigned swap_size;
  swap_size=size;
  if(swap_size&1)
  {
    size=1;       // remains one byte
    swap_size--;
  }
  else
  {
    size=0;
  }
  //printf("swap %d\n",swap_size);
  swap_high_low_byte(buf,swap_size);
  //printf("w swaped addr %ld,size %d dp=%u\n",addr,swap_size,buf-orgp);
  p_emu->write_DRAM(addr/2,buf,swap_size/2);  // write swapped data buffer
  if(size)
  {
    // write last byte
    addr+=swap_size;
    buf+=swap_size;
    //printf("write last addr %ld dp=%u\n",addr,buf-orgp);
    write_DRAM_high_byte(addr/2,*buf);
  }
}
//#include<time.h>
void do_ips(const char *fn)
{
  int fh;
  if(_dos_open(fn,O_RDONLY,&fh)!=0)
  {
    return;
  }
  printf("do_ips(%s)\n",fn);
  IPS_decoder dec;
  //clock_t clk;
  //clk=clock();
  while(1)
  {
    //printf(":");
    //while(p_key->read_key()==NoKey);
    int req_cnt;
    req_cnt=dec.get_request_data_size();
    if(req_cnt==0)
    {
      break;
    }
    unsigned nr;
    _dos_read(fh,file_buf,req_cnt,&nr);
    if(nr!=req_cnt)
    {
      printf("ips read err\n");
      break;
    }
    IPS_command cmd;
    dec.send_data(file_buf,cmd);
    switch(cmd.type)
    {
      case IPS_end:
        printf("ips end\n");
        break;
      case IPS_error:
        printf("ips err\n");
        break;
      case IPS_next:
        continue;
      case IPS_copy:
        //printf("ips copy addr=%ld size=%d\n",cmd.addr,cmd.copy_cnt);
        while(cmd.copy_cnt!=0)
        {
          unsigned max_blk_size;
          if(cmd.copy_cnt>file_buf_size)
            max_blk_size=file_buf_size;
          else
            max_blk_size=cmd.copy_cnt;
          unsigned nr;
          _dos_read(fh,file_buf,max_blk_size,&nr);
          if(nr!=max_blk_size)
          {
            printf("read err\n");
            goto quit_loop;
          }
          do_v64_copy_mem(cmd.addr,max_blk_size,file_buf);
          cmd.addr+=max_blk_size;
          cmd.copy_cnt-=max_blk_size;
        }
        continue;
      case IPS_fill:
        //printf("ips fill addr=%ld size=%d data=%02x\n",cmd.addr,cmd.fill.cnt,cmd.fill.data);
        do_v64_fill_mem(cmd.addr,cmd.fill.cnt,cmd.fill.data);
        continue;
    }
    if(cmd.type==IPS_end || cmd.type==IPS_error)
      break;
  }
  quit_loop:
  //clock_t e_clk;
  //e_clk=clock()-clk;
  //printf("ut=%f\n",e_clk/CLOCKS_PER_SEC);
  _dos_close(fh);
}
void ask_load_IPS_file(Z64_file_info &info)
{
  static char fn_buf[64];
  strcpy(fn_buf,info.get_file_name());
  strcat(fn_buf,".IPS");
  if(file_exist(fn_buf)==0)
    return;
  clear_line(2);
  p_disp->gotoxy(3,2);
  p_disp->printf("LOAD IPS?");
  int yn;
  yn=ask_yes_no(13,2);
  if(yn==0)
  {
    show_center_str(2,"LOADING IPS...");
    do_ips(fn_buf);
  }
}
