/*---Z64_MENU.CPP---*/
#include"z64_file.h"
#include"z64_util.h"
#include"z64_io.h"
#include"n64_emu.h"
#include"flash\flash.h"
#include"z64const.h"
#include"z64_gbl.h"
#include"z64_menu.h"
#include<string.h>
#include<stdio.h>

enum {
   opt_download_mempak = 0,
   opt_backup_game,
   opt_play_cart,
   opt_load_file
};
const opt_exit_del  = 0;
const opt_del_file  = 1;

int show_info_shift = 0;

const char *const download_mempak_str = "<<DOWNLOAD MEMPAK>>";
const char *const play_cart_str       = "<<PLAY CARTRIDGE>>";
const char *const backup_game_str     = "<<BACKUP CARTRIDGE>>";

const char *get_opt_str(int ptr)
{
  if (ptr == opt_download_mempak)
    return(download_mempak_str);
  else if(ptr==opt_play_cart)
    return play_cart_str;
  else if(ptr==opt_backup_game)
    return backup_game_str;
  else
    return "???";
}
void select_file_show_info(const char *prompt,Z64_file_info *info_buf,int ptr,int info_num)
{
  if(ptr >= opt_load_file)
  {
    const char *title;
    title=get_center_title(info_buf[ptr-opt_load_file].get_title());
    if(show_info_shift)
      shift_display(1,title);
    else
      show_center_str(1,title);
    p_disp->gotoxy(1,2);
    p_disp->printf(prompt);
    p_disp->gotoxy(10,2);
    p_disp->printf("%3dM",info_buf[ptr-opt_load_file].get_rom_size()/megabit);
    p_disp->gotoxy(15,2);
    p_disp->printf("%2d",ptr+1-opt_load_file);
    p_disp->gotoxy(18,2);
    p_disp->printf("/%2d",info_num);
  }
  else
  {
    const char *str;
    str=get_opt_str(ptr);
    if(show_info_shift)
    {
      show_center_str(2,"");
      str=get_center_str(str);
      shift_display(1,str);
    }
    else
    {
      p_disp->clear();
      show_center_str(1,str);
    }
    p_fc->set_status(1);
  }
}
void select_file_flash_prompt(const char *prompt,int ptr,flash_controller &fc)
{
  int is_changed,new_status;
  fc.get_status(is_changed,new_status);
  if(ptr>=opt_load_file)
  {
    if(is_changed)
    {
      p_disp->gotoxy(1,2);
      if(new_status)
      {
        p_disp->printf(prompt);
      }
      else
      {
        int i;
        for(i=0;i<strlen(prompt);i++)
	  p_disp->printf(" ");
      }
    }
  }
  else
  {
    if(is_changed)
    {
      const char *str;
      str=get_opt_str(ptr);
      if(new_status)
      {
        show_center_str(1,str);
      }
      else
      {
        p_disp->clear();
      }
    }
  }
}
//
menu::menu()
{
  m_plist=NULL;
  m_pos=0;
}
void menu::set_list(menu_list *plist)
{
  m_plist=plist;
}
void menu::set_pos(int pos)
{
  m_pos=pos;
}
void menu::pop_up()
{
  m_plist->pop_up(m_pos);
}
void menu::handle_up_key()
{
  m_plist->leave(m_pos);
  m_pos--;
  if(m_pos<m_plist->get_min_pos())
    m_pos=m_plist->get_max_pos();
  m_plist->enter(m_pos);
}
void menu::handle_down_key()
{
  m_plist->leave(m_pos);
  m_pos++;
  if(m_pos>m_plist->get_max_pos())
    m_pos=m_plist->get_min_pos();
  m_plist->enter(m_pos);
}
void menu::handle_move(int pos)
{
  m_plist->leave(m_pos);
  m_pos=pos;
  m_plist->enter(m_pos);
}
void menu::handle_idle()
{
  m_plist->idle(m_pos);
}
int menu::get_pos()
{
  return m_pos;
}
//
file_menu_list::file_menu_list():m_pinfo(NULL),m_info_num(0),m_prompt(NULL)
{
}
void file_menu_list::set_prompt(const char *prompt)
{
  m_prompt=prompt;
}
void file_menu_list::set_file_list(Z64_file_info *pinfo,int info_num)
{
  m_pinfo=pinfo;
  m_info_num=info_num;
}
int file_menu_list::get_min_pos()
{
  return 0;
}
int file_menu_list::get_max_pos()
{
  return m_info_num-1;
}
void file_menu_list::pop_up(int pos)
{
  const char *title;
  title=get_center_title(m_pinfo[pos].get_title());
  if(show_info_shift)
    shift_display(1,title);
  else
    show_center_str(1,title);
  // display prompt
  p_disp->gotoxy(1,2);
  p_disp->printf(m_prompt);
  // display id
  p_disp->gotoxy(8,2);
  if ((m_pinfo[pos].get_type() != BIOS_file) &&
      (m_pinfo[pos].get_type() != PFS))
    p_disp->printf("%c",m_pinfo[pos].get_country());
  else
    p_disp->printf(" ");

  // display size (ROM), version (BIOS) or type (PFS)
  p_disp->gotoxy(9,2);
  if (m_pinfo[pos].get_type() == PFS)
    p_disp->printf("(PFS)");
  else
  if (m_pinfo[pos].get_type() == BIOS_file)
  {
    unsigned ver;
    ver=m_pinfo[pos].get_bios_ver();
    p_disp->printf("V%d.%02d",ver>>8,ver&0xff);
  }
  else
  {
    p_disp->gotoxy(9,2);
    p_disp->printf(" %3dM",m_pinfo[pos].get_rom_size()/megabit);
  }

  // display index
  p_disp->gotoxy(15,2);
  p_disp->printf("%2d",pos+1);
  p_disp->gotoxy(18,2);
  p_disp->printf("/%2d",m_info_num);
}
void file_menu_list::leave(int /*pos*/)
{
}
void file_menu_list::enter(int pos)
{
  pop_up(pos);
}
void file_menu_list::idle(int /*pos*/)
{
  int is_changed,new_status;
  p_fc->get_status(is_changed,new_status);
  if(is_changed)
  {
    p_disp->gotoxy(1,2);
    if(new_status)
    {
      p_disp->printf(m_prompt);
    }
    else
    {
      int i;
      for(i=0;i<strlen(m_prompt);i++)
        p_disp->printf(" ");
    }
  }
}
//
load_file_menu_list::load_file_menu_list()
{
  set_prompt("LOAD");
}
int load_file_menu_list::get_max_pos()
{
  return opt_load_file+file_menu_list::get_max_pos();
}
void load_file_menu_list::pop_up(int pos)
{
  if(pos>=opt_load_file)
    file_menu_list::pop_up(pos-opt_load_file);
  else
  {
    const char *str;
    str=get_opt_str(pos);
    if(show_info_shift)
    {
      show_center_str(2,"");
      str=get_center_str(str);
      shift_display(1,str);
    }
    else
    {
      p_disp->clear();
      show_center_str(1,str);
    }
    p_fc->set_status(1);
  }
}
void load_file_menu_list::leave(int pos)
{
  file_menu_list::leave(pos);
}
void load_file_menu_list::enter(int pos)
{
  pop_up(pos);
}
void load_file_menu_list::idle(int pos)
{
  if(pos>=opt_load_file)
    file_menu_list::idle(pos-opt_load_file);
  else
  {
    int is_changed,new_status;
    p_fc->get_status(is_changed,new_status);
    if(is_changed)
    {
      const char *str;
      str=get_opt_str(pos);
      if(new_status)
      {
        show_center_str(1,str);
      }
      else
      {
        p_disp->clear();
      }
      if (pos == opt_backup_game)
      {
	char buf[32];
        sprintf(buf,"DISK FREE:%4ldM",m_disk_space/megabit);
        show_center_str(2,buf);
      }
    }
  }
}
void load_file_menu_list::disk_ready()
{
  m_disk_space=p_filesys->get_disk_space();
  if(m_disk_space<0)
    m_disk_space=0;
}
void load_file_menu_list::disk_not_ready()
{
  m_disk_space=0;
}
//
del_file_menu_list::del_file_menu_list()
{
  set_prompt("DELETE");
}
int del_file_menu_list::get_max_pos()
{
  return opt_del_file+file_menu_list::get_max_pos();
}
void del_file_menu_list::pop_up(int pos)
{
  if(pos>=opt_del_file)
    file_menu_list::pop_up(pos-opt_del_file);
  else
  {
    const char *str;
    str="<<EXIT>>";
    if(show_info_shift)
    {
      show_center_str(2,"");
      str=get_center_str(str);
      shift_display(1,str);
    }
    else
    {
      p_disp->clear();
      show_center_str(1,str);
    }
    p_fc->set_status(1);
  }
}
void del_file_menu_list::leave(int pos)
{
  file_menu_list::leave(pos);
}
void del_file_menu_list::enter(int pos)
{
  pop_up(pos);
}
void del_file_menu_list::idle(int pos)
{
  if(pos>=opt_del_file)
    file_menu_list::idle(pos-opt_del_file);
  else
  {
    int is_changed,new_status;
    p_fc->get_status(is_changed,new_status);
    if(is_changed)
    {
      const char *str;
      str="<<EXIT>>";
      if(new_status)
      {
        show_center_str(1,str);
      }
      else
      {
        p_disp->clear();
      }
      if(pos==opt_exit_del)
      {
        long space;
        space=p_filesys->get_disk_space();
        if(space<0)
          space=0;
        char buf[32];
        sprintf(buf,"DISK FREE:%4ldM",space/megabit);
        show_center_str(2,buf);
      }
    }
  }
}
//
void select_file_menu::run(file_menu_list *pfml,int opt_num)
{
  const int buf_num=99;
  static Z64_file_info info_buf[buf_num];
  int info_num;
  int dev_rdy;
  m_reload=0;
	dev_rdy=p_filesys->is_device_ready();
  if(dev_rdy)
    pfml->disk_ready();
  else
		pfml->disk_not_ready();
	p_disp->clear();
  p_filesys->get_file_list(info_buf,buf_num,info_num);
  int pos;
  pos=0;
  if(pos<0 || pos>=info_num)
    pos=0;
  int ptr;
	if(info_num>=1)
    ptr=opt_num+pos;
  else
    ptr=opt_num-1;
  pfml->set_file_list(info_buf,info_num);
	menu m;
  m.set_list(pfml);
  m.set_pos(ptr);
	show_info_shift=1;
  m.pop_up();
  show_info_shift=0;
  p_fc->set_status(1);
  while(1)
	{
    // handle user key
    int k;
    k=p_key->read_key();
    if(k==NoKey)
		{
      if(p_fc->get_prev_status()==1)
			{
        // Test disk change
        int new_dev_rdy;
        new_dev_rdy=p_filesys->is_device_ready();
        if(new_dev_rdy!=dev_rdy)
        {
          dev_rdy=new_dev_rdy;
          if(dev_rdy==0)
          {
            info_num=0;
            pfml->set_file_list(info_buf,0);
            show_info_shift=1;
            show_center_str(2,"");
            m.handle_move(opt_num-1);
            show_info_shift=0;
            pfml->disk_not_ready();
          }
          else
          {
            show_center_str(1,"");
            show_center_str(2,"LOADING DISK...");
            p_filesys->get_file_list(info_buf,buf_num,info_num);
            p_filesys->get_disk_space();
            show_center_str(2,"");
            pfml->set_file_list(info_buf,info_num);
            pfml->disk_ready();
            if(info_num!=0)
            {
              show_info_shift=1;
              show_center_str(2,"");
              m.handle_move(opt_num);
              show_info_shift=0;
              success_sound();
            }
          }
        }
      }
      m.handle_idle();
    }
    Z64_file_info info;
		if(m.get_pos()>=opt_num)
      info=info_buf[m.get_pos()-opt_num];
    if(handle_key(k,m.get_pos(),info))
      return;
    if(m_reload)
    {
      m_reload=0;
      dev_rdy=p_filesys->is_device_ready();
      if(dev_rdy)
        pfml->disk_ready();
      else
        pfml->disk_not_ready();
      p_filesys->get_file_list(info_buf,buf_num,info_num);
      pfml->set_file_list(info_buf,info_num);
      if(m.get_pos()>opt_num-1)
        m.set_pos(opt_num-1);
    }
    if(k==UpKey)
    {
      if(m.get_pos()==0 && info_num!=0)
        show_info_shift=1;
      if(m.get_pos()==opt_num && info_num!=0)
        show_info_shift=1;
      show_center_str(2,"");
      m.handle_up_key();
      show_info_shift=0;
    }
    if(k==DownKey)
		{
      if(m.get_pos()==opt_num-1  && info_num!=0)
        show_info_shift=1;
      if(m.get_pos()==info_num+opt_num-1 && info_num!=0)
        show_info_shift=1;
      show_center_str(2,"");
      m.handle_down_key();
      show_info_shift=0;
    }
    if(k==EnterKey)
      m.pop_up();
  }
}
