/*---Z64_MENU.H---*/
#ifndef _INC_Z64_MENU
#define _INC_Z64_MENU
//
class menu_list
{
  public:
    virtual int get_min_pos()=0;
    virtual int get_max_pos()=0;
    virtual void pop_up(int pos){}
    virtual void leave(int pos){}
    virtual void enter(int pos){}
    virtual void idle(int pos){}
};
//
class menu
{
  public:
    menu();
    void set_list(menu_list *plist);
    void set_pos(int pos);
    void pop_up();
    void handle_up_key();
    void handle_down_key();
    void handle_move(int pos);
    void handle_idle();
    int get_pos();
  private:
    menu_list *m_plist;
    int m_pos;
};
//
class file_menu_list:public menu_list
{
  public:
    file_menu_list();
    void set_prompt(const char *prompt);
    void set_file_list(Z64_file_info *pinfo,int info_num);
    int get_min_pos();
    int get_max_pos();
    void pop_up(int pos);
    void leave(int pos);
    void enter(int pos);
    void idle(int pos);
    virtual void disk_ready(){}
    virtual void disk_not_ready(){}
  private:
    const char *m_prompt;
    Z64_file_info *m_pinfo;
    int m_info_num;
};
//
class load_file_menu_list:public file_menu_list
{
  public:
    load_file_menu_list();
    int get_max_pos();
    void pop_up(int pos);
    void leave(int pos);
    void enter(int pos);
    void idle(int pos);
    void disk_ready();
    void disk_not_ready();
  private:
    long m_disk_space;
};
//
class del_file_menu_list:public file_menu_list
{
  public:
    del_file_menu_list();
    int get_max_pos();
    void pop_up(int pos);
    void leave(int pos);
    void enter(int pos);
    void idle(int pos);
};
//
class select_file_menu
{
  public:
    void run(file_menu_list *pfml,int opt_num);
    void set_reload(){m_reload=1;}
    virtual int handle_key(int k,int pos,Z64_file_info &info){return 0;};
  private:
		int m_reload;
};
#endif
