/*---Z64_TEST.CPP---*/
#include"z64_test.h"
#include"z64_util.h"
#include"z64_io.h"
#include"n64_emu.h"
#include"z64_gbl.h"
#include"lcd_if.h"
#include<conio.h>
#include<stdio.h>
#include<stdlib.h>
#include<string.h>
#include<dos.h>
int get_chip_ver()
{
  printf("get chip version:\n");
  // read/write command register test
  unsigned i;
  for(i=0;i<16;i++)
  {
    outpw(0x162,i+0x1000);  // Select Access Source = 1
    unsigned r;
    r=inpw(0x164);
    if((r&0x0f)!=i)
    {
      // unable to read back cmd reg, version = 1
      printf("chip version=1.0\n");
      printf("i=%04x,r=%04x\n",i,r);
      return 1;
    }
  }
  // able to reac back cmd reg, ver = 2
  printf("chip version=2.0\n");
  return 2;
}
void test_error()
{
  printf("test error,press ESC to quit\n");
  error_sound();
  while(1)
  {
    if(kbhit() && getch()==27)
      exit(0);
  }
}
void test_chip_error()
{
  show_center_str(2,"Z64 CHIP ERROR");
  test_error();
}
void test_addr_port(unsigned pat1,unsigned pat2)
{
  outpw(0x162,2);
  unsigned data1,data2;
  printf("write pattern %04x %04x\n",pat1,pat2);
  inpw(0x162);
  outpw(0x166,pat1);
  outpw(0x164,pat2);
  data1=inpw(0x166);
  data2=inpw(0x164);
  printf("166:%04x\n",data1);
  printf("164:%04x\n",data2);
  if(pat1!=data1 || pat2!=data2)
  {
    test_chip_error();
  }
}
void test_chip_ver2_err
(unsigned pat,unsigned long addr1,unsigned long addr2,unsigned r)
{
  printf("p=%04x a=%lx\n",pat,addr1);
  printf("access %lx\n",addr2);
  printf("r=%04x\n",r);
  test_chip_error();
}
void test_chip_ver2()
{
  struct pattern_addr
  {
    unsigned int pattern;
    unsigned long addr;
    unsigned int mask;
  };
  static pattern_addr test_pat[]=
  {
    {0x7003,0x10000000,0x0010},
    {0x7013,0x80000000,0x0010},
    {0x7023,0x06000000,0x0010},
    {0x7033,0x05000000,0x0010},
    {0x7003,0x11000000,0x0020},
    {0x7043,0x08000000,0x0020},
    {0x7083,0x05000000,0x0020},
    {0x70c3,0x06000000,0x0020},
  };
  const test_cnt=sizeof(test_pat)/sizeof(test_pat[0]);
  int i,j;
  char data_buf[2];
  // for each pattern
  pattern_addr *ppat;
  ppat=test_pat;
  for(i=0;i<test_cnt;i++)
  {
    // access each address
    for(j=0;j<test_cnt;j++)
    {
      // set mapping address
      p_emu->set_ctrl_reg(ppat[i].pattern);
      // access data
      p_emu->set_address(ppat[j].addr/2);
      p_emu->read_data_word(data_buf,1);
      // get access flag
      unsigned r,f;
      r=inpw(0x164);
      f=(r&ppat[i].mask);
      if(ppat[i].addr==ppat[j].addr)
      {
        // address match, EN_F_16M should active(0)
        if(f!=0)
        {
          printf("access flag should be 0\n");
          test_chip_ver2_err
          (ppat[i].pattern,ppat[i].addr,ppat[j].addr,r);
        }
      }
      else
      {
        // address not match, EN_F_16M sounld inactive(1)
        if(f==0)
        {
          printf("access flag should be 1\n");
          test_chip_ver2_err
          (ppat[i].pattern,ppat[i].addr,ppat[j].addr,r);
        }
      }
    }
  }
  // test request flag
  p_emu->clear_N64_req_flag();
  if(p_emu->read_N64_req_flag())
  {
    printf("unable to clear N64 req flag\n");
    test_chip_error();
  }
  printf("test N64 req flag 1:\n");
  p_emu->clear_N64_req_flag();
  // triggle address=0x0f00
  p_emu->set_ctrl_reg(0x6003);
  // access data
  p_emu->set_address(0x0f000000/2);
  p_emu->read_data_word(data_buf,1);
  if(p_emu->read_N64_req_flag()==0)
  {
    printf("unable to triggle 0x0f000000\n");
    test_chip_error();
  }
  printf("test N64 req flag 2:\n");
  p_emu->clear_N64_req_flag();
  // triggle address=0x1e00
  p_emu->set_ctrl_reg(0xe003);
  // access data
  p_emu->set_address(0x1e000000/2);
  p_emu->read_data_word(data_buf,1);
  if(p_emu->read_N64_req_flag()==0)
  {
    printf("unable to triggle 0x1e00000000\n");
    test_chip_error();
  }
}
void quick_test_mem_err()
{
  printf("mem err\n");
  show_center_str(2,"MEM ERROR");
  error_sound();
  while(p_key->read_key()==NoKey);
}
int quick_test_mem(unsigned long addr)
{
  unsigned m,n;
  m=0x55aa;
  p_emu->write_DRAM(addr,(char*)&m,1);
  p_emu->read_DRAM(addr,(char*)&n,1);
  if(m!=n)
  {
    quick_test_mem_err();
    return 1;
  }
  m=0xaa55;
  p_emu->write_DRAM(addr,(char*)&m,1);
  p_emu->read_DRAM(addr,(char*)&n,1);
  if(m!=n)
  {
    quick_test_mem_err();
    return 1;
  }
  return 0;
}
void test_z64_chip()
{
  show_center_str(1,"TEST CHIP");
  clear_line(2);
  printf("test ctrl chip:\n");
  test_addr_port(0x55aa,0xaa55);
  test_addr_port(0xaa55,0x55aa);
  if(p_emu->get_chip_ver()>1)
  {
    test_chip_ver2();  // test chip version 2.0
  }
  // quick test memory
  if(quick_test_mem(0/2))
    return;
  if(p_emu->get_chip_ver()>1)  // 2.0 chip, test 32M
  {
    printf("test 32M\n");
    if(quick_test_mem(16L*1024*1024/2))
    {
      printf("should install 32M DRAM\n");
      return;
    }
    printf("32M Ok\n");
  }
}
void mem_err(unsigned long mem_base,unsigned buf_size,char far *org_buf,char far *ref_buf)
{
  unsigned int data_a,data_b;
  unsigned i;
  for(i=0;i<buf_size;i+=2)
  {
    data_a=*(unsigned int*)(org_buf+i);
    data_b=*(unsigned int*)(ref_buf+i);
    if(data_a!=data_b)
    {
      printf("mem err:pos=0x%lx wr=%04X rd=%04X\n",mem_base+i,data_a,data_b);
      p_disp->clear();
      p_disp->gotoxy(3,1);
      p_disp->printf("MEM ERR %08lX",mem_base+i);
      p_disp->gotoxy(3,2);
      p_disp->printf("WR %04X  RD %04X",data_a,data_b);
      test_error();
    }
  }
}
void fill_long_data(char *buf,unsigned long data,unsigned size)
{
  int i;
  for(i=0;i<size;i+=4)
  {
    *((unsigned long*)(buf+i))=data;
  }
}
unsigned long get_mem_size()
{
  if(p_emu->get_chip_ver()>1)
    return 32L*1024*1024;
  else
    return 16L*1024*1024;
}
void test_mem_data(char *org_buf,char *ref_buf,int size,unsigned char data)
{
  printf("test data %02X\n",data);
  clear_line(2);
  p_disp->gotoxy(2,2);
  p_disp->printf("ADR:");
  p_disp->gotoxy(16,2);
  p_disp->printf("%02X%02X",data,data);
  unsigned long mem_size;
  mem_size=get_mem_size();
  N64_loop loop;
  for(loop.init(mem_size,size);loop.not_end();loop.next())
  {
    p_emu->write_DRAM(loop.get_word_addr(),org_buf,size/2);
    p_emu->read_DRAM(loop.get_word_addr(),ref_buf,size/2);
    if(memcmp(org_buf,ref_buf,size)!=0)
    {
      mem_err(loop.get_byte_addr(),size,org_buf,ref_buf);
    }
    if(((loop.get_byte_addr()+loop.get_blk_size())%32768)==0)
    {
      cprintf("%ld\r",loop.get_byte_addr()+loop.get_blk_size());
      p_disp->gotoxy(7,2);
      p_disp->printf("%08ld",loop.get_byte_addr()+loop.get_blk_size());
    }
  }
  printf("\nOk\n");
}
void self_test()
{
//  extern char z64_version[];  // defined in Z64.CPP
  // Show version/initial screen
  p_disp->clear();
  show_center_str(1,"SYSTEM SELF TEST");
  char ver_buf[64];
  sprintf(ver_buf,"VER 1.11/PFS");
  show_center_str(2,ver_buf);
  // do self test
  test_z64_chip();
  test_io();
  if(test_cart())
  {
    test_error();
  }
  test_eeprom();
  // test mem
  const pagesize=512;
  static char org_buf[pagesize];
  static char ref_buf[pagesize];
  show_center_str(1,"TEST MEM");
  printf("test mem\n");
  // Data test
  // 55
  fill_long_data(org_buf,0xaaaa5555,sizeof(org_buf));
  test_mem_data(org_buf,ref_buf,pagesize,0x55);
  /*  // don't test it to speed up!
  // aa
  fill_long_data(org_buf,0x5555aaaa,sizeof(org_buf));
  test_mem_data(org_buf,ref_buf,pagesize,0xaa);
  */
  // pattern test
  printf("pattern test\n");
  printf("writing...\n");
  clear_line(2);
  p_disp->gotoxy(2,2);
  p_disp->printf("ADR:");
  p_disp->gotoxy(16,2);
  p_disp->printf("PATW");
  unsigned long mem_size;
  mem_size=get_mem_size();
  int i;
  int j;
  unsigned idx_64k;
  unsigned r_64k;
  unsigned base;
  i=0;
  N64_loop loop;
  for(loop.init(mem_size,256);loop.not_end();loop.next())
  {
    idx_64k=loop.get_word_addr()/65536L;
    r_64k=loop.get_word_addr()%65536L;
    base=idx_64k+r_64k;
    for(i=0;i<256/2;i++)
      ((unsigned*)(org_buf))[i]=base+i;
    p_emu->write_DRAM(loop.get_word_addr(),org_buf,256/2);
    if(((loop.get_byte_addr()+loop.get_blk_size())%32768)==0)
    {
      cprintf("%ld\r",loop.get_byte_addr()+loop.get_blk_size());
      p_disp->gotoxy(7,2);
      p_disp->printf("%08ld",loop.get_byte_addr()+loop.get_blk_size());
    }
  }
  printf("\nreading...\n");
  p_disp->gotoxy(16,2);
  p_disp->printf("PATR");
  i=0;
  for(loop.init(mem_size,256);loop.not_end();loop.next())
  {
    idx_64k=loop.get_word_addr()/65536L;
    r_64k=loop.get_word_addr()%65536L;
    base=idx_64k+r_64k;
    for(i=0;i<256/2;i++)
      ((unsigned*)(org_buf))[i]=base+i;
    p_emu->read_DRAM(loop.get_word_addr(),ref_buf,256/2);
    if(memcmp(org_buf,ref_buf,256)!=0)
    {
      mem_err(loop.get_byte_addr(),256,org_buf,ref_buf);
    }
    if(((loop.get_byte_addr()+loop.get_blk_size())%32768)==0)
    {
      cprintf("%ld\r",loop.get_byte_addr()+loop.get_blk_size());
      p_disp->gotoxy(7,2);
      p_disp->printf("%08ld",loop.get_byte_addr()+loop.get_blk_size());
    }
  }
  printf("\nOk\n");
  test_ok();
}
// test 8255
void test_8255()
{
  printf("test 8255\n");
  const io=0x168;
  unsigned char data;
  // set port a/b output,c input
  outp(io+3,0x89);       // A,B output C input
  // test A
  printf("test port A:\n");
  // 55
  printf("write 55:");
  outp(io,0x55);
  data=inp(io);
  printf("read %02X\n",data);
  if(data!=0x55)
  {
    printf("error\n");
    test_error_sound(1);
    exit(0);
  }
  // aa
  printf("write AA:");
  outp(io,0xAA);
  data=inp(io);
  printf("read %02X\n",data);
  if(data!=0xAA)
  {
    printf("error\n");
    test_error_sound(1);
    exit(0);
  }
  // test B
  printf("test port B:\n");
  // 55
  printf("write 55:");
  outp(io+1,0x55);
  data=inp(io+1);
  printf("read %02X\n",data);
  if(data!=0x55)
  {
    printf("error\n");
    test_error_sound(1);
    exit(0);
  }
  // aa
  printf("write AA:");
  outp(io+1,0xAA);
  data=inp(io+1);
  printf("read %02X\n",data);
  if(data!=0xAA)
  {
    printf("error\n");
    test_error_sound(1);
    exit(0);
  }
}
// test LCD
void test_LCD(Text_LCD_if_8255 &if_8255)
{
  test_8255();
  printf("test LCD...");
  unsigned char r;
  if_8255.write_cmd(0x40);
  r=if_8255.read_status();
  if((r&0x80)!=0x80)
  {
    // error
    printf("error\n");
    printf("r=%x\n",r);
    test_error_sound(2);
    exit(0);
  }
  printf("ok\n");
}
void disp_io_sts
(unsigned char data,unsigned char mask,const char *str1,const char *str2
,const char *str3,const char *str4)
{
  if(data&mask)
  {
    printf(str1);
    p_disp->printf(str3);
  }
  else
  {
    printf(str2);
    p_disp->printf(str4);
  }
  printf(" ");
  p_disp->printf(" ");
}
void test_io()
{
  p_emu->set_access_ctrl(N64_ROM_emulator::N64_ROM);
  printf("test IO,press ENTER to end\n");
  show_center_str(1,"TEST IO:ENTER TO END");
  clear_line(2);
  while(1)
  {
    unsigned char data;
    data=inp(0x16a);
    printf("\rDATA=%02X ",data);
    p_disp->gotoxy(1,2);
    disp_io_sts(data,0x02,"--","Dn","  ","Dn");
    disp_io_sts(data,0x04,"--","Up","  ","Up");
    disp_io_sts(data,0x10,"N64:NO ","N64:YES","   ","N64");
    disp_io_sts(data,0x40,"N64PWR:ON ","N64PWR:OFF","NPWR","    ");
    disp_io_sts(data,0x80,"CART:NO ","CART:YES","    ","CARD");
    if((data&0x01)==0)
    {
      printf("\n");
      return;
    }
  }
}
#pragma option -2
unsigned long byte2long(const char *buf)
{
  unsigned long l;
  ((unsigned char*)(&l))[0]=buf[3];
  ((unsigned char*)(&l))[1]=buf[2];
  ((unsigned char*)(&l))[2]=buf[1];
  ((unsigned char*)(&l))[3]=buf[0];
  return l;
}
#define ROL(i, b) (((i)<<(b)) | ((i)>>(32-(b))))
int test_cart()
{
  if(p_emu->test_cart())
  {
    return 0;
  }
  const buf_size=512;
  static char buf[buf_size];
  const unsigned long rom_magic=0x80371240;
  const unsigned long checksum_startvalue=0xf8ca4ddc;
  show_center_str(1,"TEST CART");
  show_center_str(2,"READING");
  p_emu->read_ROM(0,buf,2);
  unsigned long read_magic;
  read_magic=byte2long(buf);
  printf("magic=%08lX\n",read_magic);
  if(read_magic!=rom_magic)
  {
    printf("magic number err\n");
    show_center_str(2,"READ ERROR");
    test_error();
  }
  // test checksum
  const long chk_start=0x1000;
  const long chk_len  =0x100000;
  const long chk_end  =chk_start+chk_len;
  long read_ptr;
  unsigned long i;
  unsigned long c1, k1, k2;
  unsigned long t1, t2, t3, t4;
  unsigned long t5, t6;
  t1 = checksum_startvalue;
  t2 = checksum_startvalue;
  t3 = checksum_startvalue;
  t4 = checksum_startvalue;
  t5 = checksum_startvalue;
  t6 = checksum_startvalue;
  for(read_ptr=chk_start;read_ptr<chk_end;read_ptr+=buf_size)
  {
    p_emu->read_ROM(read_ptr/2,buf,buf_size/2);
    p_disp->gotoxy(2,2);
    p_disp->printf("READING %07ld",read_ptr);
    //
    for( i=0; i<buf_size; i+=4 ) {
      c1 = byte2long(buf+i);
      k1 = t6 + c1;
      if( k1 < t6 ) t4++;
      t6 = k1;
      t3 ^= c1;
      k2 = c1 & 0x1f;
      k1 = ROL(c1, k2);
      t5 += k1;
      if( c1 < t2 ) {
        t2 ^= k1;
      } else {
        t2 ^= t6 ^ c1;
      }
      t1 += c1 ^ t5;
    }
    //
  }
  unsigned long sum1, sum2;
  sum1 = t6 ^ t4 ^ t3;
  sum2 = t5 ^ t2 ^ t1;
  printf("s1    =%08lx\n",sum1);
  printf("s2    =%08lx\n",sum2);
  unsigned long rs1,rs2;
  p_emu->read_ROM(0x10/2,buf,8/2);
  rs1=byte2long(buf);
  rs2=byte2long(buf+4);
  printf("rom s1=%08lx\n",rs1);
  printf("rom s2=%08lx\n",rs2);
  if(sum1==rs1 && sum2==rs2)
  {
    return 0;
  }
  printf("cart chksum err\n");
  show_center_str(2,"READ ERROR");
  return 1;
}
void test_eeprom()
{
  if(p_emu->test_cart())
  {
    return;
  }
  show_center_str(1,"TEST CART RECORD");
  show_center_str(2,"TEST");
  printf("test cart EEPROM\n");
  // test EEPROM
  p_emu->set_PC_access_EEPROM();
  p_emu->reset_EEPROM();
  if(p_emu->test_EEPROM())
  {
    p_emu->set_N64_access_EEPROM();
    show_center_str(2,"TEST ERROR");
    printf("detect error\n");
    test_error();
  }
  printf("detect ok\n");
  // read/write EEPROM
  const eeprom_size=512;
  static unsigned char eeprom_test_pat[eeprom_size];
  static unsigned char eeprom_read_pat[eeprom_size];
  // test 55
  memset(eeprom_test_pat,0x55,sizeof(eeprom_test_pat));
  printf("rw 0x55...");
  show_center_str(2,"RW 55");
  p_emu->write_EEPROM_page(0,eeprom_test_pat,eeprom_size/8);
  p_emu->read_EEPROM_page(0,eeprom_read_pat,eeprom_size/8);
  if(memcmp(eeprom_test_pat,eeprom_read_pat,eeprom_size)!=0)
  {
    show_center_str(2,"READ ERROR 55");
    printf("read error 0x55\n");
    test_error();
  }
  printf("ok\n");
  // test AA
  memset(eeprom_test_pat,0xaa,sizeof(eeprom_test_pat));
  printf("rw 0xAA...");
  show_center_str(2,"RW AA");
  p_emu->write_EEPROM_page(0,eeprom_test_pat,eeprom_size/8);
  p_emu->read_EEPROM_page(0,eeprom_read_pat,eeprom_size/8);
  if(memcmp(eeprom_test_pat,eeprom_read_pat,eeprom_size)!=0)
  {
    show_center_str(2,"READ ERROR AA");
    printf("read error 0xAA\n");
    test_error();
  }
  printf("ok\n");
}
