/*---Z64_UTIL.CPP---*/
#include"z64_util.h"
#include"z64const.h"
#include"z64_io.h"
#include"z64_file.h"
#include"n64_emu.h"
#include"z64_gbl.h"
#include"flash\flash.h"
#include<string.h>
#include<bios.h>
#include<dos.h>
#include<conio.h>
#include<fcntl.h>
int file_exist(const char *fn)
{
  int fh;
  if(_dos_open(fn,O_RDONLY,&fh)!=0)
    return 0;
  _dos_close(fh);
  return 1;
}
// N64 power detection functions
int wait_power(const char *prompt,int pwr)///
{
  // -- start
  int bHandleBackup = 0;
  if(pwr==3)
  {
    pwr=1;
    bHandleBackup = 1;
  }
  // -- end
  p_disp->clear();
  p_disp->gotoxy(1,1);
  p_disp->printf(prompt);
  p_fc->set_status(0);
  if(pwr==0 && p_emu->is_N64_power_on()!=0)
	prompt_sound();
  while(p_emu->is_N64_power_on()!=pwr)
  {
    // -- start
    int k = p_key->read_key(); ///
    if(bHandleBackup && k==EnterKey && (inp(0x16a)&0x08)==0)
    {
      return 1;
    }
    // -- end
    //p_key->read_key();
    int is_changed,new_status;
    p_fc->get_status(is_changed,new_status);
    {
      if(is_changed)
      {
        if(new_status)
        {
          if(pwr)
            show_center_str(2,"TURN ON N64 POWER");
          else
            show_center_str(2,"TURN OFF N64 POWER");
        }
        else
          show_center_str(2,"");
      }
    }
  }
  return 0;///
}
void wait_power_off(const char *prompt)
{
  wait_power(prompt,0);
}
void wait_power_on(const char *prompt)
{
  wait_power(prompt,1);
}
void wait_cart()
{
  if(p_emu->test_cart()==0)  // cartridge plugged
    return;
  p_disp->clear();
  p_fc->set_status(0);
  long clk,new_clk;
  clk=biostime(0,0);
  prompt_sound();
  while(1)
  {
    new_clk=biostime(0,0);
    if(p_emu->test_cart())  // not plugged
    {
      clk=new_clk;
    }
    else
    {
      if(new_clk-clk>18)
        break;
    }
    p_key->read_key();
    int is_changed,new_status;
    p_fc->get_status(is_changed,new_status);
    {
      if(is_changed)
      {
        if(new_status)
          show_center_str("TURN OFF POWER AND\nPLUG CARTRIDGE");
        else
          p_disp->clear();
      }
    }
  }
}
// wait user enter
void show_msg_wait_enter(const char *msg)
{
  show_center_str(2,msg);
  p_fc->set_status(0);
  while(p_key->read_key()!=EnterKey)
  {
    int is_changed,new_status;
    p_fc->get_status(is_changed,new_status);
    {
      if(is_changed)
      {
        if(new_status)
          show_center_str(2,msg);
        else
          show_center_str(2,"");
      }
    }
  }
}
// Display functions
void get_center_str(char *buf,const char *src)
{
  int len,leading_spc;
  len=strlen(src);
  leading_spc=(20-len)/2;
  memset(buf,' ',20);
  memcpy(buf+leading_spc,src,len);
  buf[20]='\0';
}
const char *get_center_str(const char *src)
{
  static char buf[24];
  get_center_str(buf,src);
  return buf;
}
void show_center_str(const char *src)
{
  char buf[24];
  const char *p;
  p=strchr(src,'\n');
  if(p==NULL)
  {
    show_center_str(1,src);
    return;
  }
  else
  {
    memcpy(buf,src,p-src);
    buf[p-src]='\0';
    show_center_str(1,buf);
    show_center_str(2,p+1);
  }
}
void show_center_str(int y,const char *src)
{
  char buf[24];
  get_center_str(buf,src);
  p_disp->gotoxy(1,y);
  p_disp->printf("%.20s",buf);
}
void clear_line(int line)
{
  show_center_str(line,"");
}
const char *get_center_title(const char *title)
{
  int title_len;
  for(title_len=20;title_len>0;title_len--)
  {
    if(title[title_len-1]!=' ')
      break;
  }
  static char title_buf[24];
  memset(title_buf,' ',sizeof(title_buf));
  memcpy(title_buf+(20-title_len)/2,title,title_len);
  title_buf[20]='\0';
  return title_buf;
}
void show_file_title(const char *title)
{
  p_disp->gotoxy(1,1);
  p_disp->printf("%.20s",get_center_title(title));
}
// ask user yes/no
static void disp_yn_opt(int x,int y,int opt,int st)
{
  char *yn[2][2]=
  {
    {"   ","YES"},
    {"  ","NO"}
  };
  if(opt==0)  // Y
  {
    p_disp->gotoxy(x,y);
  }
  else
  {
    p_disp->gotoxy(x+4,y);
  }
  p_disp->printf(yn[opt][st]);
}
int ask_yes_no(int x,int y,int def_opt)
{
  int opt;
  opt=def_opt;
  disp_yn_opt(x,y,0,1);
  disp_yn_opt(x,y,1,1);
  p_fc->set_status(0);
  int is_changed,new_status;
  while(1)
  {
    int k;
    k=p_key->read_key();
    if(k==NoKey)
    {
      p_fc->get_status(is_changed,new_status);
      if(is_changed)
      {
        disp_yn_opt(x,y,opt,new_status);
      }
    }
    else
    {
      if(k==EnterKey)
        return opt;
      if(k==UpKey || k==DownKey)
      {
        disp_yn_opt(x,y,opt,1);
        p_fc->set_status(0);
        opt=!opt;
        disp_yn_opt(x,y,opt,0);
      }
    }
  }
}
// error handling
void show_error_msg_wait_enter(const char *msg)
{
  p_disp->clear();
  show_center_str(msg);
  error_sound();
  p_fc->set_status(0);
  while(p_key->read_key()!=EnterKey)
  {
    int is_changed,new_status;
    p_fc->get_status(is_changed,new_status);
    {
      if(is_changed)
      {
        if(new_status)
          show_center_str(msg);
        else
          p_disp->clear();
      }
    }
  }
}
void load_file_error()
{
  show_error_msg_wait_enter("\nLOAD FILE ERROR");
}
void write_file_error()
{
  show_error_msg_wait_enter("\nWRITE FILE ERROR");
}
void disk_not_ready()
{
  show_error_msg_wait_enter("\nDISK NOT READY");
}
void disk_space_not_enough()
{
  show_error_msg_wait_enter("\nSPACE NOT ENOUGH");
}
void no_eeprom_warning()
{
  show_error_msg_wait_enter("NO CARTRIDGE EEPROM\nUNABLE TO SAVE");
}
// sounds
static void wait_clk(int l=1)
{
  long clk;
  clk=biostime(0,0);
  while(biostime(0,0)<clk+l);
}
void success_sound()
{
  nosound();
  wait_clk();
  sound(1000);
  wait_clk();
  sound(4000);
  wait_clk();
  sound(2000);
  wait_clk();
  sound(4000);
  wait_clk();
  nosound();
}
void prompt_sound()
{
  nosound();
  wait_clk();
  sound(2000);
  wait_clk();
  nosound();
  wait_clk(2);
  sound(2000);
  wait_clk();
  nosound();
}
void error_sound()
{
  nosound();
  wait_clk();
  sound(500);
  wait_clk(3);
  nosound();
  wait_clk(1);
  sound(500);
  wait_clk(3);
  nosound();
  wait_clk(1);
  sound(500);
  wait_clk(3);
  nosound();
}
void test_error_sound(int cnt)
{
  int i;
  while(1)
  {
    for(i=0;i<cnt;i++)
    {
      wait_clk();
      sound(500);
      wait_clk(3);
      nosound();
      wait_clk(1);
    }
    wait_clk(20);
    if(kbhit() && getch()==27)
      return;
  }
}
void test_ok()
{
  p_disp->clear();
  show_center_str(1,"SELF TEST OK");
  success_sound();
  delay(3000);
}
//
void shift_display_delay()
{
  long l;
  for(l=0;l<18000;l++);
}
void shift_display(int y,const char *new_buf,int dir)
{
  char prev_buf[20];
  p_disp->gotoxy(1,y);
  p_disp->read(prev_buf,20);
  int i,j;
  for(i=1;i<=20;i++)
  {
    p_disp->gotoxy(1,y);
    if(dir==0)  // R->L
    {
      for(j=i;j<20;j++)
      {
        p_disp->printf("%c",prev_buf[j]);
      }
      for(j=0;j<i;j++)
      {
        p_disp->printf("%c",new_buf[j]);
      }
    }
    else  // L->R
    {
      for(j=20-i;j<20;j++)
      {
        p_disp->printf("%c",new_buf[j]);
      }
      for(j=0;j<20-i;j++)
      {
        p_disp->printf("%c",prev_buf[j]);
      }
    }
    shift_display_delay();
  }
}
int wait_change_disk()
{
	int is_changed,new_status;
  int dev_rdy=p_filesys->is_device_ready();
  while(dev_rdy==p_filesys->is_device_ready())
  {
  	p_fc->get_status(is_changed,new_status);
    if(is_changed)
    {
     	if(new_status)
        show_center_str(2,"REMOVE DISK...");
    	else
		    show_center_str(2,"");
		}
    if(p_key->read_key()!=NoKey)
      return 1;
  }
  dev_rdy=p_filesys->is_device_ready();
  while(dev_rdy==p_filesys->is_device_ready())
  {
  	p_fc->get_status(is_changed,new_status);
    if(is_changed)
    {
	    if(new_status)
        show_center_str(2,"INSERT DISK...");
	    else
	      show_center_str(2,"");
   	}
	  if(p_key->read_key()!=NoKey)
      return 1;
  }
  p_filesys->get_disk_space();
  return 0;
}
void show_option(option &opt,int st)
{
  if(st)
  {
    p_disp->gotoxy(opt.x,2);
    p_disp->printf(opt.str);
  }
  else
  {
    int i,l;
    l=strlen(opt.str);
    for(i=0;i<l;i++)
    {
      p_disp->gotoxy(opt.x+i,2);
      p_disp->printf(" ");
    }
  }
}
int select_option(option *opt,int opt_num,int init_sel)
{
  int i;
  for(i=0;i<opt_num;i++)
  {
    show_option(opt[i],1);
  }
  int sel;
  sel=init_sel;
  p_fc->set_status(0);
  int is_changed,new_status;
  while(1)
  {
    int k;
    k=p_key->read_key();
    if(k==NoKey)
    {
      p_fc->get_status(is_changed,new_status);
      if(is_changed)
      {
        show_option(opt[sel],new_status);
      }
      continue;
    }
    if(k==EnterKey)
      break;
    show_option(opt[sel],1);
    if(k==UpKey)
    {
      sel--;
      if(sel<0)
        sel=opt_num-1;
    }
    else if(k==DownKey)
    {
      sel++;
      if(sel>=opt_num)
        sel=0;
    }
    p_fc->set_status(0);
    show_option(opt[sel],0);
  }
  return sel;
}
// Display process progress pencent
static unsigned long file_progress_sz;
static int file_progress_bar;
static int file_progress_percent;
void process_file_init_progress(unsigned long sz)
{
  // font pattern for progress display
  static char progress_bar_fnt[6][8]=
  {
    {0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00},
    {0x10,0x10,0x10,0x10,0x10,0x10,0x10,0x00},
    {0x18,0x18,0x18,0x18,0x18,0x18,0x18,0x00},
    {0x1c,0x1c,0x1c,0x1c,0x1c,0x1c,0x1c,0x00},
	  {0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x00},
    {0x1f,0x1f,0x1f,0x1f,0x1f,0x1f,0x1f,0x00}
  };
	file_progress_sz=sz;
  // set new font
  int i;
  for(i=0;i<6;i++)
    p_disp->set_font(i,progress_bar_fnt[i]);
  // clear display
  for(i=1;i<=16;i++)
  {
    p_disp->gotoxy(i,2);
		p_disp->printf("%c",8);
  }
  file_progress_bar=0;
  p_disp->gotoxy(17,2);
  p_disp->printf("  0%");
  file_progress_percent=0;
}
void process_file_update_progress(unsigned long sz)
{
  // show percent/bar
  int bar;
  bar=sz*16*5/file_progress_sz;
  // show bar
  if(bar!=file_progress_bar)  // need update?
  {
    int prev_loc;
    int loc,pat;  // character location/font pattern
    int i;
    prev_loc=(file_progress_bar-1)/5+1;
    loc=(bar-1)/5+1;
    pat=bar%5;
    if(pat==0)
      pat=5;
    for(i=prev_loc;i<loc;i++)
    {
      p_disp->gotoxy(i,2);
  		p_disp->printf("%c",5);
    }
    p_disp->gotoxy(loc,2);
		p_disp->printf("%c",pat);
		file_progress_bar=bar;
  }
  // show percent number
  int percent;
  percent=sz*100/file_progress_sz;
  if(percent!=file_progress_percent)
  {
    file_progress_percent=percent;
    p_disp->gotoxy(17,2);
		p_disp->printf("%3d%%",percent);
  }
}
void clear_file_update_progress()
{
	show_center_str(2,"");
}
int write_cart_SRAM_to_SRAM_file()
{
  // read cartridge SRAM data and write to file
  // need to open SRAM file first
  N64_loop loop;
  for(loop.init(SRAM_size,file_buf_size);loop.not_end();loop.next())
  {
    // read SRAM
    unsigned file_buf_ptr;
    for(file_buf_ptr=0;file_buf_ptr<file_buf_size;file_buf_ptr+=data_buf_size)
    {
      p_emu->read_cart_SRAM(loop.get_word_addr()+file_buf_ptr/2,file_buf+file_buf_ptr,data_buf_size/2);
    }
    // Write file
    if(p_filesys->write_sram_file(file_buf,file_buf_size)!=0)
    {
      p_filesys->close_sram_file();
      write_file_error();
      return 1;  // Write error
    }
  }
  // Write ok
  p_filesys->close_sram_file();
  return 0;
}
int write_DRAM_mapped_SRAM_to_SRAM_file()
{
  // read cartridge SRAM data and write to file
  // need to open SRAM file first
  N64_loop loop;
  for(loop.init(SRAM_size,file_buf_size);loop.not_end();loop.next())
  {
    // read SRAM
    unsigned file_buf_ptr;
    for(file_buf_ptr=0;file_buf_ptr<file_buf_size;file_buf_ptr+=data_buf_size)
    {
      p_emu->read_DRAM_mapped_SRAM(loop.get_word_addr()+file_buf_ptr/2,file_buf+file_buf_ptr,data_buf_size/2);
    }
    // Write file
    if(p_filesys->write_sram_file(file_buf,file_buf_size)!=0)
    {
      p_filesys->close_sram_file();
      write_file_error();
      return 1;  // Write error
    }
  }
  // Write ok
  p_filesys->close_sram_file();
  return 0;
}
int load_SRAM_file()
{
  // read SRAM file and write to cartridge
  // need to open SRAM file first
  N64_loop loop;
  for(loop.init(SRAM_size,file_buf_size);loop.not_end();loop.next())
  {
    // read file
    if(p_filesys->read_sram_file(file_buf,file_buf_size)!=0)
    {
      p_filesys->close_sram_file();
      load_file_error();
      return 1;  // Write error
    }
    // write SRAM
    unsigned file_buf_ptr;
    for(file_buf_ptr=0;file_buf_ptr<file_buf_size;file_buf_ptr+=data_buf_size)
    {
      p_emu->write_DRAM_mapped_SRAM(loop.get_word_addr()+file_buf_ptr/2,file_buf+file_buf_ptr,data_buf_size/2);
    }
  }
  // Write ok
  p_filesys->close_sram_file();
  return 0;
}
void save_original_DRAM_mapped_SRAM()
{
  N64_loop loop;
  for(loop.init(SRAM_size,file_buf_size);loop.not_end();loop.next())
  {
    // read SRAM
    unsigned file_buf_ptr;
    for(file_buf_ptr=0;file_buf_ptr<file_buf_size;file_buf_ptr+=data_buf_size)
    {
      p_emu->read_DRAM_mapped_SRAM(loop.get_word_addr()+file_buf_ptr/2,file_buf+file_buf_ptr,data_buf_size/2);
    }
    _fmemcpy(prev_SRAM_buf+loop.get_byte_addr(),file_buf,file_buf_size);
  }
}
int is_SRAM_modified()
{
  N64_loop loop;
  for(loop.init(SRAM_size,file_buf_size);loop.not_end();loop.next())
  {
    // read SRAM
    unsigned file_buf_ptr;
    for(file_buf_ptr=0;file_buf_ptr<file_buf_size;file_buf_ptr+=data_buf_size)
    {
      p_emu->read_DRAM_mapped_SRAM(loop.get_word_addr()+file_buf_ptr/2,file_buf+file_buf_ptr,data_buf_size/2);
    }
    // compare
    if(_fmemcmp(prev_SRAM_buf+loop.get_byte_addr(),file_buf,file_buf_size))
      return 1;
  }
  return 0;
}
void auto_backup_SRAM(Z64_file_info &info)
{
  p_disp->clear();
  show_file_title(info.get_title());
  show_center_str(2,"CHECKING SRAM...");
  if(is_SRAM_modified()==0)
  {
    return;
  }
  // Changed,ask to update SRAM file
  p_disp->clear();
  show_file_title(info.get_title());
  p_disp->gotoxy(1,2);
  p_disp->printf("UPDATE SRAM?");
  prompt_sound();
  int ret;
  ret=ask_yes_no(15,2);
  if(ret==0)  // Yes
  {
    if(p_filesys->open_sram_file_write(info)!=0)
    {
      write_file_error();
      return;  // Open error
		}
    if(write_DRAM_mapped_SRAM_to_SRAM_file())
    {
      return;
    }
    p_disp->clear();
    show_file_title(info.get_title());
    show_center_str(2,"UPDATE SRAM OK");
  }
}
