/*---APSIDRV.H---*/
#ifndef _INC_ASPIDRV
#define _INC_ASPIDRV
#include"typedef.h"
// Host adapter information
struct HostInquiryInfo
{
  BYTE HostSCSI_ID;    // Host Adapter SCSI ID
  BYTE ManagerID[17];  // SCSI Manager ID
  BYTE HostID[17];     // Host Adapter Identifier
  BYTE HostParam[17];  // Host Adaptor Unique Parameter
  WORD SuppExt;        // Supported Extensions
};
// Thread
struct Thread
{
  BYTE HostNum;   // Host Adaptor Number
  BYTE TargetID;  // Target ID
  BYTE LUN;       // LUN
};
// Sense Data
struct SenseData
{
  int ErrorCode:7;           // Error Code
  int Valid:1;               // Valid
  BYTE Resv1;                // Reserved
  unsigned int SenseKey:4;   // Sense Key
  int Resv2:4;               // Reserved
  BYTE InfoByte[4];          // Infomation Bytes
  BYTE AddSenseLen;          // Additional Sense Length
  BYTE CommSpecInfoByte[4];  // Command-specific Infomation Bytes
  BYTE ASC;                  // Additional Sense Code
  BYTE ASCQ;                 // Additional Sense Code Qualifier
  BYTE Resv3;                // Reserved
  BYTE Resv4;                // Reserved
  BYTE Resv5;                // Reserved
  BYTE Resv6;                // Reserved
  BYTE AddSenseByte[8];      // Additional Sense Byte
};
// Command Info
struct CommandInfo
{
  CommandInfo();  // Constructor
  void Reset();        // Reset all variables
  void SetThread(const Thread &thread);
  void SetThread(BYTE HostNum,BYTE TargetID,BYTE LUN);
  void SetCDB(FARPTR cdb,BYTE scsi_cdb_len);
  void SetDataBuffer(FARPTR BufferPtr,DWORD AllocLen);
  Thread InputThread;
  FARPTR CDB;
  BYTE SCSI_CDB_Len;
  FARPTR DataBufferPtr;
  DWORD DataAllocLen;
  BYTE HostStatus;
  BYTE TargetStatus;
  SenseData Sense;
};
// Test Unit Ready CDB
struct SCSI_CDB_TestUnitReady
{
  SCSI_CDB_TestUnitReady();  // Constructor
  void Reset();              // Reset all variables
  BYTE OpCode;               // Operation Code
  int Resv1:5;               // Reserved
  int LUN:3;                 // LUN
  BYTE Resv2;                // Reserved
  BYTE Resv3;                // Reserved
  BYTE Resv4;                // Reserved
  BYTE Control;              // Control Field
};
// Inquiry CDB
struct SCSI_CDB_Inquiry
{
  SCSI_CDB_Inquiry();  // Constructor
  void Reset();        // Reset all variables
  BYTE OpCode;         // Operation Code
  int EVPD:1;          // EVPD
  int Resv1:4;         // Reserved
  int LUN:3;           // LUN
  BYTE PageCode;       // Page code
  BYTE Resv2;          // Reserved
  BYTE DataAllocLen;   // Data Allocation Length
  BYTE Control;        // Control Field
};
// MODE SENSE
struct SCSI_CDB_MODE_SENSE
{
  BYTE OpCode;     // Operation Code
  int Resv1:5;     // Reserved
  int LUN:3;       // LUN
  int PageCode:6;  // Page Code
  int PC:2;        // Page Control
  BYTE Resv2;      // Reversed
  BYTE AllocLen;   // Allocation Lengthion
  BYTE Control;    // Control field
};
// MODE SELECT
struct SCSI_CDB_MODE_SELECT
{
  BYTE OpCode;        // Operation Code
  int SP:1;           // Save Parameter
  int Resv1:3;        // Reserved
  int PF:1;           // Page Format
  int LUN:3;          // LUN
  BYTE Resv2;         // Reserved
  BYTE Resv3;         // Reserved
  BYTE ParamListLen;  // Parameter List Length
  BYTE Control;       // Control field
};
// Block Descriptor(for MODE SENSE/SELECT)
struct BlkDes
{
  BYTE DensityCode;  // Density Code
  DWORD NumOfBlk;    // Number of Blocks
  DWORD BlkLen;      // Block Length
  BlkDes()
  {
    DensityCode=0;
    NumOfBlk=0;
    BlkLen=0;
  }
};
// ASPI Driver Class
class ASPI_Driver
{
  public:
    virtual int IsInstalled()=0;
    virtual int GetNumOfHosts()=0;
    virtual void HostInquiry(BYTE HostNum,HostInquiryInfo *info)=0;
    virtual BYTE GetDeviceType(BYTE Host,BYTE TargetID,BYTE LUN,BYTE *type)=0;
    virtual WORD SCSI_ExecuteInputData(CommandInfo &cmd)=0;
    virtual WORD SCSI_ExecuteOutputData(CommandInfo &cmd)=0;
    virtual WORD SCSI_ExecuteNoData(CommandInfo &cmd)=0;
    virtual WORD SCSI_ResetDevice(const Thread &thread)=0;
    virtual ~ASPI_Driver(){};
};
class Std_ASPI_Driver:public ASPI_Driver
{
  public:
    Std_ASPI_Driver();
    int IsInstalled();
    int GetNumOfHosts();
    void HostInquiry(BYTE HostNum,HostInquiryInfo *info);
    BYTE GetDeviceType(BYTE Host,BYTE TargetID,BYTE LUN,BYTE *type);
    WORD SCSI_ExecuteInputData(CommandInfo &cmd);
    WORD SCSI_ExecuteOutputData(CommandInfo &cmd);
    WORD SCSI_ExecuteNoData(CommandInfo &cmd);
    WORD SCSI_ResetDevice(const Thread &thread);
    ~Std_ASPI_Driver();
  private:
    void far (*m_ASPI_Entry)(void far *srb);  // ASPI Entry Pointer
    WORD SCSI_Execute(CommandInfo &cmd,BYTE ReqFlag);
    void SCSI_ExecuteNoWait(BYTE *SRB,CommandInfo &cmd,BYTE ReqFlag);
};
//
int aspi_inquiry_unit(ASPI_Driver &aspidrv,Thread &thread,BYTE *buf);
void cdb_write16bit(void *dst,WORD src);
void cdb_write32bit(void *dst,DWORD src);
#endif  // _INC_ASPIDRV
