
#include "flashrom\ATMEL.H"
#include "delay61h\delay61h.h"
#include <dos.h>

flash_atmel::flash_atmel()
{
 disk_seg=0x0e000;
 page_io=0x100;
 set_socket(0);
 set_page(0);
 idx=0;
 current_sec=0;
}

flash_atmel::~flash_atmel()
{
 set_page(0);
}

int flash_atmel::get_sector_size()
{
 return 512;
}

int flash_atmel::get_max_sector()
{
 return 1023;
}

int flash_atmel::get_current_sector()
{
 return current_sec;
}

void flash_atmel::set_disk_io(unsigned int io)
{
 page_io=io;
}

void flash_atmel::set_socket(char socket)
{
 p_num=(p_num & 0x0f) | (socket << 4);
 outportb(page_io,p_num);
}

void flash_atmel::set_page(int page_num)
{
 p_num=(p_num & 0xf0) | (page_num >> 7);
 outportb(page_io,p_num);
}

void flash_atmel::set_segment(unsigned int d_seg)
{
 disk_seg=d_seg;
}

unsigned int flash_atmel::write(unsigned char far *buf,int sec_num,unsigned int sector)
{
 unsigned int i1,flag1;
 current_sec=sec_num;
 do
 {
  get_sector(sec_num);
  for (i1=0; i1<2; i1++)
  {
   flag1=write_1(buf);
   if (flag1)
    break;
   verify(buf);
   if (flag1)
    break;
   buf+=256;
   idx+=256;
  }
  if (flag1)
   break;
  sec_num++;
  current_sec++;
  sector--;
 } while (sector);
 return flag1;
}

unsigned int flash_atmel::write_1(unsigned char far *buf)
{
 unsigned int flag1;
 unprotect();
 flag1=write_2(buf);
 protect();
 if (!flag1)
 {
  flag1=write_2(buf);
  delay_61port(500);
  //delay(10);
 };
 return flag1;
}

unsigned int flash_atmel::write_2(unsigned char far *buf)
{
 unsigned int i1,flag1;
 unsigned char far *c;
 c=(unsigned char far *)MK_FP(disk_seg,idx);
 for (i1=0; i1<256; i1++)
 {
  c[i1]=buf[i1];
  flag1=check_toggle_bit(c+i1);
  if (flag1)	// write fail
   break;
 };
 return flag1;
}

unsigned int flash_atmel::check_toggle_bit(unsigned char far *c)
{
 unsigned int i1,flag1;
 flag1=toggle_0(c);
 if (!flag1)
 {
  flag1=toggle_1(c);
  if (!flag1)
   flag1=toggle_0(c);
 };
 return flag1;
}

unsigned int flash_atmel::toggle_0(unsigned char far *c)
{
 unsigned int i1,flag1;
 flag1=-1;
 for (i1=0; i1<10000; i1++)
 {
  if (!(*c & 0x40))
  {
   flag1=0;
   break;
  };
 };
 return flag1;
}

unsigned int flash_atmel::toggle_1(unsigned char far *c)
{
 unsigned int i1,flag1;
 flag1=-1;
 for (i1=0; i1<10000; i1++)
 {
  if (*c & 0x40)
  {
   flag1=0;
   break;
  };
 };
 return flag1;
}

unsigned int flash_atmel::verify(unsigned char far *buf)
{
 unsigned int i1,flag1;
 unsigned char far *c;
 c=(unsigned char far *)MK_FP(disk_seg,idx);
 flag1=0;
 for (i1=0; i1<256; i1++)
 {
  if (buf[i1]!=c[i1])
  {
   flag1=-1;
   break;
  };
 };
 return flag1;
}

void flash_atmel::get_sector(int sec_num)
{
 p_num=(p_num&0xf0) | ((sec_num/128) & 0x0f);
 idx=(sec_num%128)*512;  // sector start address
 outportb(page_io,p_num);
 outportb(0x80,p_num);
}

void flash_atmel::protect()
{
 unsigned char p_reg;
 unsigned char far *c;
 p_reg=p_num;
 c=(unsigned char far *)MK_FP(disk_seg,0);
 set_page(0);
 c[0x5555]=0xaa;
 c[0x2aaa]=0x55;
 c[0x5555]=0xa0;
 outportb(page_io,p_reg);
 p_num=p_reg;
}

void flash_atmel::unprotect()
{
 unsigned char p_reg;
 unsigned char far *c;
 p_reg=p_num;
 c=(unsigned char far *)MK_FP(disk_seg,0);
 set_page(0);
 c[0x5555]=0xaa;
 c[0x2aaa]=0x55;
 c[0x5555]=0x80;
 c[0x5555]=0xaa;
 c[0x2aaa]=0x55;
 c[0x5555]=0x20;
 outportb(page_io,p_reg);
 p_num=p_reg;
}
