
#include "flashrom\SST.H"
#include "delay61h\delay61h.h"
#include <dos.h>

flash_sst::flash_sst()
{
 disk_seg=0xe000;
 page_io=0x100;
 set_socket(0);
 set_page(0);
 idx=0;
 current_sec=0;
}

flash_sst::~flash_sst()
{
 set_page(0);
}

int flash_sst::get_sector_size()
{
 return 512;
}

int flash_sst::get_max_sector()
{
 return 1023;
}

int flash_sst::get_current_sector()
{
 return current_sec;
}

void flash_sst::set_disk_io(unsigned int io)
{
 page_io=io;
}

void flash_sst::set_socket(char socket)
{
 p_num=(p_num & 0x0f) | (socket << 4);
 outportb(page_io,p_num);
}

void flash_sst::set_page(int page_num)
{
 p_num=(p_num & 0xf0) | (page_num >> 7);
 outportb(page_io,p_num);
}

unsigned int flash_sst::erase()
{
 unsigned int i1,i2,i3,idx1,flag1;
 unsigned char far *c;
 c=(unsigned char far *)MK_FP(disk_seg,0);
 idx1=idx;
 for (i3=0; i3<2; i3++)
 {
  asm {cli}
  c[idx1]=0x20;
  c[idx1]=0xd0;
  //asm{sti}
  delay_61port(100);
  //delay(2);

  for (i1=0; i1<256; i1++)
  {
   flag1=-1;
   for (i2=0; i2<100; i2++)     // wait erase
   {
    if (c[idx1]==0xff)                  // erase finish
    {
     flag1=0;
     break;
    };
   };
   if (flag1)                   // erase fail
    break;
   idx1++;
  };
 };
 return flag1;
}
void flash_sst::set_segment(unsigned int d_seg)
{
 disk_seg=d_seg;
}

unsigned int flash_sst::write(unsigned char far *buf,int sec_num,unsigned int sector)
{
 unsigned i1,i2,idx1,flag1;
 unsigned char far *c;
 idx1=0;
 current_sec=sec_num;
 c=(unsigned char far *)MK_FP(disk_seg,0);
 unprotect();
 do
 {
  get_sector(sec_num);
  if (erase())
  {
   flag1=-1;
   break;
  }
  for (i1=0; i1<512; i1++)
  {
   flag1=-1;
   asm{cli}
   c[idx]=0x10;
   c[idx]=buf[idx1];
   //asm{sti}
   for (i2=0; i2<20000; i2++)
   {
    if (c[idx]==buf[idx1])      // write finish
    {
     flag1=0;
     break;
    };
   };
   if (flag1)                   // write fail
    break;
   idx++;
   idx1++;
  };
  if (flag1)                    // write fail
   break;
  sec_num++;
  current_sec++;
  sector--;
 } while (sector);
 protect();
 return flag1;
}

void flash_sst::get_sector(int sec_num)
{
 p_num=(p_num&0xf0) | ((sec_num/128) & 0x0f);
 idx=(sec_num%128)*512; // sector start address
 outportb(page_io,p_num);
}

void flash_sst::protect()
{
 unsigned char p_reg,c1;
 unsigned char far *c;
 p_reg=p_num;
 c=(unsigned char far *)MK_FP(disk_seg,0);
 set_page(0);
 c1=c[0x1823];
 c1=c[0x1820];
 c1=c[0x1822];
 c1=c[0x418];
 c1=c[0x41b];
 c1=c[0x419];
 c1=c[0x40a];
 if (c1);
 outportb(page_io,p_reg);
 p_num=p_reg;
}

void flash_sst::unprotect()
{
 unsigned char p_reg,c1;
 unsigned char far *c;
 p_reg=p_num;
 c=(unsigned char far *)MK_FP(disk_seg,0);
 set_page(0);
 c1=c[0x1823];
 c1=c[0x1820];
 c1=c[0x1822];
 c1=c[0x418];
 c1=c[0x41b];
 c1=c[0x419];
 c1=c[0x41a];
 if (c1);
 outportb(page_io,p_reg);
 p_num=p_reg;
}

