/***************************************************************************
 *                                                                         *
 *  upbios.cpp                                                             *
 *                                                                         *
 *  Loads an as Overlay attached BIOS and writes it to the EEPROM,	   *
 *  without any CRC-checks                          			   *
 *                                                                         *
 *  Ravemax of DX!BDT                                                      *
 *                                                                         *
 ***************************************************************************/

#include <dos.h>
#include <io.h>
#include <fcntl.h>
#include <alloc.h>
#include <own/mytypes.h>
#include "flashrom\sst.h"
#include <lcd\textlcd.h>
#include "lcdfunc.cpp"

// LCD-variables are global

Text_LCD_if_8255 if8255;
Text_LCD_config cfg;
Text_LCD lcd(&if8255);

void puts_xy(int x,int y,char *txt)
{
   lcd.gotoxy(x,y);
   lcd.printf(txt);
}

int seek_to_bios(int fhandle)
{
   word esize[2];

   lseek(fhandle,2,SEEK_SET);
   _read(fhandle,(void *)esize,4);
   lseek(fhandle,(esize[1]-1)*512+esize[0],SEEK_SET);
   lcd.gotoxy(1,1);
   if (eof(fhandle))
      return(1);
   if (filelength(fhandle) != tell(fhandle)+524288)
      return(1);
   return(0);
}

int update_eeprom(int fhandle)
{
   const long bios_size = 512L*1024;

   byte puffer[512];
   flash_sst flash;
   dword baddr;
   int eret;

   flash.set_segment(0xE000);
   _asm cli
   for (baddr = 0; baddr < bios_size-64L*1024; baddr += 512)
   {
      _read(fhandle,puffer,512);
      eret = flash.write(puffer,baddr / 512,1);
      if (eret)
	 return(1);
   }
   _asm sti
   return(0);
}

void main(int argc,char *argv[])
{
   int fh;

   lcd.clear_screen();
   _dos_open(argv[0],O_RDONLY,&fh);
   if (seek_to_bios(fh))
   {
      _dos_close(fh);
      puts_xy(1,2,"NO OR INVALID BIOS!");
   }
   else
   {
      puts_xy(4,1,"BIOS-UPDATE...");
      if (update_eeprom(fh))
	 puts_xy(4,2,"UPDATE FAILED!");
      else
	 puts_xy(2,2,"TURN OFF YOUR Z64");
      _dos_close(fh);
      while(1)
	;
   }
}

