/***************************************************************************
 *                                                                         *
 *  keypad.cpp                                                             *
 *                                                                         *
 *  Routines to access the keypads                                         *
 *                                                                         *
 ***************************************************************************/

#include "z64unzip.h"

/*--------------------------------------------------------------------------
   KeypadRead - Returns the keypad-code (0 = Nothing pressed)
 --------------------------------------------------------------------------*/

U8 KeypadRead(void)
{
   U8 tKey;

   #ifndef USE_VGA
      if (!((tKey = ~inp(0x16A)) & KeyAnykey))
	 return(0);
      delay(100);
   #else
      U8 pcKey;

      if (!kbhit())
	 return(0);
      if (!(pcKey = getch()))
	 pcKey = getch();
      tKey = 0;
      if (pcKey == 27)
	 return(0xFF);
      if (pcKey == 13)
	 tKey |= KeyEnter;
      if (pcKey == 72)
	 tKey |= KeyUp;
      if (pcKey == 80)
	 tKey |= KeyDown;
      if (!tKey)
	 return(0);
   #endif

   return(tKey);
}

/*--------------------------------------------------------------------------
   KeypadWait - Waits until one (or more) certain key is pressed
 --------------------------------------------------------------------------*/

void KeypadWait(U8 validKeys)
{
   while (KeypadRead() & validKeys);      // Wait until released
   while (!(KeypadRead() & validKeys));   //  "     "   pressed
}


