#ifndef __LCD_H_
#define __LCD_H_

/***************************************************************************
 *                                                                         *
 *  lcd.h                                                                  *
 *                                                                         *
 ***************************************************************************/

#include "basetype.h"

/*--------------------------------------------------------------------------
   Constants
 --------------------------------------------------------------------------*/

/*
 *  Write-ids
 */

#define LCDWCmd    0
#define LCDWData   4

/*--------------------------------------------------------------------------
   External functions
 --------------------------------------------------------------------------*/

extern void LCDWrite(U8 id,unsigned char data);
extern void LCDCls(void);
extern void LCDSetFont(U8 from,U16 num,U8 *font);
extern void LCDPuts(const char *str);
extern void LCDPutsXY(U8 x,U8 y,const char *str);
extern void LCDPrintf(const char *fmt,...);
extern void LCDProgressBar(long base,long valueNow = 0);

/*--------------------------------------------------------------------------
   Macros
 --------------------------------------------------------------------------*/

#ifndef USE_VGA

   /*
    *  Use LCD
    */

   #define LCDHome()   \
      LCDWrite(LCDWCmd,2)

   #define LCDSetCGRAMAddr(addr)   \
      LCDWrite(LCDWCmd,0x40 | ((addr) & 0x3f))

   #define LCDSetDDRAMAddr(addr)   \
      LCDWrite(LCDWCmd,0x80 | ((addr) & 0x7f))

   #define LCDWriteChar(charCode)   \
      LCDWrite(LCDWData,(charCode))

   #define LCDGotoXY(x,y)   \
      LCDSetDDRAMAddr(((x)-1)+((y)-1)*0x40)

#else

   /*
    *  Use VGA
    */

   #define LCDHome()   \
      Gotoxy(1,1)

   #define LCDSetCGRAMAddr(addr)
   #define LCDSetDDRAMAddr(addr)

   #define LCDWriteChar(charCode)   \
      putchar((charCode))

   #define LCDGotoXY(x,y)   \
      gotoxy((x),(y))


#endif


#endif  // !__LCD_H_
