#include <stdio.h>
#include <string.h>
#include <io.h>
#include <fcntl.h>
#include <alloc.h>
#include "unzip.h"
#include "ram.h"
#include "lcd.h"

int main(int argc,char *argv[1])
{
   /*
    *  1. Commandline
    */

   unzFile zipFile;

   if (argc != 2)
   {
      puts("Z64UNZIP filename");
      return(1);
   }
   if ((zipFile = unzOpen(argv[1])) == NULL)
   {
      puts("ZIP I/O error");
      return(2);
   }


   /*
    *  2. Search ROM
    */

   unz_file_info zinfo;
   char fname[13];
   char found = 0;

   LCDCls();
   LCDPutsXY(3,2,"Searching .ROM");

   unzGoToFirstFile(zipFile);
   do
   {
      unzGetCurrentFileInfo(zipFile,&zinfo,fname,13,NULL,0,NULL,0);

      printf("%s,%p\n",fname,strstr(fname,".rom"));
      if (strstr(fname,".rom"))
      {
	 found = 1;
	 break;
      }
   } while (unzGoToNextFile(zipFile) != UNZ_END_OF_LIST_OF_FILE);

   if (!found)
   {
      puts("No ROM found!");
      unzClose(zipFile);
      return(3);
   }

   LCDPutsXY(1,1,fname);

   /*
    *  3. Extract file
    */

   #define BufferSize   8192

   char *buffer = (char *)malloc(BufferSize);
   int zipRes;
   U32 currOfs = 0;

   if (buffer == NULL)
   {
      puts("Not enough memory!");
      unzClose(zipFile);
      return(4);
   }

   unzOpenCurrentFile(zipFile);
   LCDProgressBar(0);
   while ((zipRes = unzReadCurrentFile(zipFile,buffer,BufferSize)) > 0)
   {
      RamWriteBuffer(Ram_Dram,currOfs,buffer,(U16)zipRes);
      currOfs += zipRes;
      LCDProgressBar(zinfo.uncompressed_size,currOfs);
      printf("Written: %d\N",zipRes);
   }
   printf("Last result: %d\n",zipRes);
   unzCloseCurrentFile(zipFile);
   free(buffer);
   RamSetMemoryRights(N64_Dram);


   /*
    *  4. Shotdown
    */

   unzClose(zipFile);
   return(0);
}

