/***************************************************************************
 *                                                                         *
 *  binutil.cpp                                                            *
 *                                                                         *
 *  The Bin-Util-support (extensions,emulator)                             *
 *                                                                         *
 ***************************************************************************/

#include "all_inc.h"

/*--------------------------------------------------------------------------
   Constants
 --------------------------------------------------------------------------*/

const char *const BinUtilPathStr = "/bin/desc.*";

#define BinUtilDescNum   5

const char *const BinUtilDescStr[BinUtilDescNum] = {
   "TYPE",
   "BIN",
   "ACTIONSTR",
   "TITLEADDR",
   "TITLELEN"
};

#define BinUtilTypeNum   4

const char *const BinUtilTypeStr[BinUtilTypeNum] = {
   "EFS",
   "EMU",
   "JBE",
   "Z64"
};

/*--------------------------------------------------------------------------
   Global variables
 --------------------------------------------------------------------------*/

BinUtil *binUtil;
int binUtilNumber;

/*--------------------------------------------------------------------------
   BinUtilInit - Bin-Util initialisation
 -------------------------------------------------------------------------*/

void BinUtilInit(void)
{
   int dfDone;
   struct ffblk fileBlock;
   FILE *f;
   char strBuf[129],*valuePtr;
   U8 loop;

   binUtil = NULL;
   binUtilNumber = 0;

   dfDone  = findfirst(BinUtilPathStr,&fileBlock,0x00);
   while (!dfDone)
   {
      strcpy(strBuf,BinUtilPathStr);
      strcpy(strBuf+5,fileBlock.ff_name);

      /*
       *  Load description
       */

      binUtil = (BinUtil *)realloc(binUtil,(binUtilNumber+1)*sizeof(BinUtil));
      strcpy(binUtil[binUtilNumber].ext,strchr(fileBlock.ff_name,'.'));
      binUtil[binUtilNumber].titleAddr = -1;   // No title (default)

      f = fopen(strBuf,"rt");
      while (!feof(f))
      {
	 fgets(strBuf,129,f);
	 if ((strBuf[0] != ';') && strBuf[0])
	 {
	    strBuf[strlen(strBuf)-1] = 0;  // Removing '\n'
	    for (loop = 0; loop < BinUtilDescNum; loop++)
	    {
	       if (strstr(strBuf,BinUtilDescStr[loop]))
	       {
		  valuePtr = strchr(strBuf,'=')+1;
		  switch (loop)
		  {
		     case 0 :   // Type
			for (char typeNum = 0; typeNum <= BinUtilTypeNum;
			     typeNum++)
			   if (!strcmpi(BinUtilTypeStr[typeNum],valuePtr))
			   {
			      binUtil[binUtilNumber].type = typeNum;
			      break;
			   }
			break;
		     case 1 :   // Bin
			binUtil[binUtilNumber].binStr = strdup(valuePtr);
			break;
		     case 2 :   // Actionstring
			binUtil[binUtilNumber].actionStr = strdup(valuePtr);
			break;
		     case 3 :   // Titleaddress
			binUtil[binUtilNumber].titleAddr =
			   strtol(valuePtr,NULL,0);
			break;
		      case 4 :   // Title-length
			binUtil[binUtilNumber].titleLen = (char)atoi(valuePtr);
			break;
		  }
	       }
	    }
	 }
      }
      fclose(f);
      binUtilNumber++;

      dfDone = findnext(&fileBlock);
   }
}

/*--------------------------------------------------------------------------
   BinUtilClose - Shutdown
 -------------------------------------------------------------------------*/

void BinUtilClose(void)
{
   for (; binUtilNumber; binUtilNumber--)
   {
      free(binUtil[binUtilNumber-1].binStr);
      free(binUtil[binUtilNumber-1].actionStr);
   }
   free(binUtil);
}

/*--------------------------------------------------------------------------
   BinUtilList - Lists all infos (just for debugging)
 -------------------------------------------------------------------------*/

void BinUtilList(void)
{
   int loop;

   for (loop = 0; loop < binUtilNumber; loop++)
   {
      printf("%d] %s\n",loop,binUtil[loop].ext);
      printf("  TYPE      = %s\n",BinUtilTypeStr[binUtil[loop].type]);
      printf("  BIN       = %s\n",binUtil[loop].binStr);
      printf("  ACTIONSTR = %s\n",binUtil[loop].actionStr);
      printf("  TITLEADDR = ");
      if (binUtil[loop].titleAddr == -1)
	 puts("NO TITLE");
      else
      {
	 printf("0x%lX\n",binUtil[loop].titleAddr);
	 printf("  TITLELEN  = %d\n",binUtil[loop].titleLen);
      }
   }
}

/*--------------------------------------------------------------------------
   BinUtilOpen - Opens the util (only if ROM) and returns the filehandle
 -------------------------------------------------------------------------*/

int BinUtilOpen(const char *fname)
{
   char tempfname[128];
   U8 pathChars;

   pathChars = strlen(BinUtilPathStr)-6;
   memcpy(tempfname,BinUtilPathStr,pathChars);
   strcpy(tempfname+pathChars,binUtil[MenuCurrentID].binStr);

   printf("Open (name) : %s\n",tempfname);

   return(open(tempfname,O_RDONLY));
}

