/***************************************************************************
 *                                                                         *
 *  config.cpp                                                             *
 *                                                                         *
 *  Configuration                                                          *
 *                                                                         *
 ***************************************************************************/

#include "all_inc.h"

/*--------------------------------------------------------------------------
   Constants
 --------------------------------------------------------------------------*/
/*
 *  Path and name of the configuration-file
 */

const char *const ConfigFilename = "ZCI.CFG";

/*
 *  The different choices
 */

const char *EnabledDisabledStr[] = {
   "Disabled",
   "Enabled"
};

const char *AskAlwaysNeverStr[] = {
   "Ask",
   "Always",
   "Never"
};

/*--------------------------------------------------------------------------
   Globale variables
 --------------------------------------------------------------------------*/

U8 config[ConfigNum];

/*--------------------------------------------------------------------------
   ConfigRead - Reads the configuration
 --------------------------------------------------------------------------*/

void ConfigRead(void)
{
   int fh;

   #ifdef USE_VGA
      if ((fh = open(ConfigFilename,O_RDONLY)) != -1)
      {
	  _read(fh,config,ConfigNum);
	  close(fh);
      }
      else
   #endif
   {
      config[Sound]    = True;
      config[Eeprom]   = Ask;
      config[Sram]     = Ask;
      config[CDSave]   = Ask;
      config[Fastload] = Ask;
      config[Palfix]   = False;
      config[Devmode]  = False;
   }
}

/*--------------------------------------------------------------------------
   ConfigWrite - Writes the configuration
 --------------------------------------------------------------------------*/

void ConfigWrite(void)
{
   #ifndef USE_VGA
   #else
      int fh;

      if ((fh = open(ConfigFilename,O_WRONLY)) != -1)
      {
	 _write(fh,config,ConfigNum);
	 close(fh);
      }
   #endif
}

/*--------------------------------------------------------------------------
   ConfigUpdateStr - Updates the string for the 2nd screenline
 --------------------------------------------------------------------------*/

void ConfigUpdateStr(U8 no)
{
   if (IsBool(no))
      CenterStr(screenLine[1],20,(char *)EnabledDisabledStr[config[no-128]],' ');
   else
      CenterStr(screenLine[1],20,(char *)AskAlwaysNeverStr[config[no]],' ');

   LCDPuts(screenLine[1]);
}

/*--------------------------------------------------------------------------
   ConfigChange - Changes a configuration-entry
 --------------------------------------------------------------------------*/

void ConfigChange(U8 no)
{
   if (IsBool(no))
   {
      no -= 128;
      config[no] ^= True;
      MfxIn4(1,EnabledDisabledStr[config[no]]);
   }
   else
   {
      if (config[no] == 2)
	 config[no] = 0;
      else
	 config[no]++;

      MfxIn4(1,AskAlwaysNeverStr[config[no]]);
   }
}

