/***************************************************************************
 *                                                                         *
 *  dosfile.cpp                                                            *
 *                                                                         *
 *  Drive and file I/O-functions                                           *
 *                                                                         *
 ***************************************************************************/

#include "all_inc.h"

/*--------------------------------------------------------------------------
   Globale variables
 --------------------------------------------------------------------------*/

Bool driveReadOnly;
U32  diskSerial = 0;

DiskInfo diskInfo;

/*--------------------------------------------------------------------------
   FileSize - Returns the size of a file
 --------------------------------------------------------------------------*/

long FileSize(int handle)
{
   long oldPos,tSize;

   oldPos = tell(handle);
   lseek(handle,0,SEEK_END);
   tSize = tell(handle);
   lseek(handle,oldPos,SEEK_SET);
   return(tSize);
}

/*--------------------------------------------------------------------------
   FileOpenExt - Opens a file with an other extension
 -------------------------------------------------------------------------*/

int FileOpenExt(char *fn,char *fext)
{
   int tfh;
   char *tempName,*tSuffix;

   tempName = (char *)malloc(strlen(fn)+5);
   strcpy(tempName,fn);
   if ((tSuffix = strchr(tempName,'.')) == NULL)
      strcat(tempName,fext);
   else
      strcpy(tSuffix,fext);
   tfh = open(tempName,O_RDWR);
   free(tempName);
   return(tfh);
}

/*--------------------------------------------------------------------------
   GetFileID - Returns the first word of a given file
 --------------------------------------------------------------------------*/

U16 FileGetID(char *fname)
{
   int fh;
   U16 id;

   if ((fh = open(fname,O_RDONLY)) == -1)
      return(0xFFFF);
   _read(fh,&id,2);
   close(fh);
   return(id);
}

/*--------------------------------------------------------------------------
   ReadFile - Reads data from a file
 --------------------------------------------------------------------------*/

Bool FileRead(char *fname,long offset,char *buffer,U16 size)
{
   int fh;

   if ((fh = open(fname,O_RDONLY)) == -1)
      return(False);
   if (lseek(fh,offset,SEEK_SET) == -1L)
      return(False);
   if (_read(fh,buffer,size) == -1)
      return(False);
   close(fh);
   return(True);
}

/*--------------------------------------------------------------------------
   DiskGetInfo - Checks a disk is ready and sets some variables
 --------------------------------------------------------------------------*/

Bool DiskGetInfo(U8 num)
{
   asm {
      mov   ah,0x69
      xor   al,al
      mov   bl,num    //bl = drive-num
      xor   bh,bh
      mov   dx,offset diskInfo
      int   0x21
      jnc   gdi_successfull
   }
      return(False);
   gdi_successfull:
      diskSerial = diskInfo.diskSerial;
      if (*diskInfo.fileType == 'C')   // "CDROM" or "CD001"
	 driveReadOnly = True;
      else
	 driveReadOnly = False;
      return(True);
}

/*--------------------------------------------------------------------------
   DiskGetFreeSpace
 --------------------------------------------------------------------------*/

U32 DiskGetFreeSpace(void)
{
   asm {
      mov   ah,0x36
      xor   dl,dl
      int   0x21
      dw    0x2566    // and eax,0x0000FFFFF
      dw    0xFFFF    // "
      dw    0x0000    // "
      db    0x66
      mul   bx
      db    0x66
      mul   cx
      db    0x66
      mov   dx,ax
      db    0x66
      shr   dx,16
   }
}

