/***************************************************************************
 *                                                                         *
 *  hw_misc.cpp                                                            *
 *                                                                         *
 *  Z64-Hardware-routines, which doesn't fit in the RAM-section            *
 *                                                                         *
 ***************************************************************************/

#include "all_inc.h"

/*--------------------------------------------------------------------------
   Global variables
 --------------------------------------------------------------------------*/

char Z64hwVersion;

/*--------------------------------------------------------------------------
   Z64InitChip - Initialise the Z64 chip (checks for version)
 --------------------------------------------------------------------------*/

void Z64InitChip(void)
{
   #ifdef USE_VGA
      Z64hwVersion = 0;
   #else
      asm {
	 mov   cx,0x0100    //ch = bool,cl = cnt
	 mov   bx,0x1000
      }
      izc_loop:
      asm {
	 mov   dx,0x162
	 mov   ax,bx
	 out   dx,ax
	 mov   dx,0x164
	 in    al,dx
	 and   al,0x0F
	 cmp   al,cl
	 jne   izc_end    // unable to read back cmd reg, version = 1
	 inc   bx
	 inc   cl
	 cmp   cl,16
	 jb    izc_loop
	 mov   ch,2       // able to read back cmd reg, version = 1
      }
      izc_end:
      asm {
	 mov   Z64hwVersion,ch
      }
    #endif
}

/*--------------------------------------------------------------------------
   Z64ClearRequestFlag - As the name says..
 --------------------------------------------------------------------------*/

void Z64ClearRequestFlag(void)
{
   asm {
      mov   dx,0x169   // set bit 6,keep bit 7(HOLD)
      mov   al,0xC0
      out   dx,al
      mov   al,0x80
      out   dx,al
   }
}

/*--------------------------------------------------------------------------
   Z64CheckRequestFlag - Checks if the request flag is set
 --------------------------------------------------------------------------*/

char Z64CheckRequestFlag(void)
{
   asm {
      mov   dx,0x016A
      in    al,dx
      and   al,0x20
   }
}

/*--------------------------------------------------------------------------
   N64IsPowerOn - Is N64 on or off
 --------------------------------------------------------------------------*/

char N64IsPowerOn(void)
{
   asm {
      mov   dx,0x016A
      in    al,dx
      and   al,0x40
   }
}

/*--------------------------------------------------------------------------
   N64IsCartridgePlugged - Cartridge available ?
 --------------------------------------------------------------------------*/

char N64IsCartridgePlugged(void)
{
   asm {
      mov   dx,0x016A
      in    al,dx
      and   al,0x80
   }
}

