/***************************************************************************
 *                                                                         *
 *  menu.cpp                                                               *
 *                                                                         *
 *  The menu (types,handler...)		                                   *
 *                                                                         *
 ***************************************************************************/

#include "all_inc.h"
#include "menu.inc"

/*--------------------------------------------------------------------------
   General constants
 --------------------------------------------------------------------------*/

const U16 TypeID[MenuTypesNum] = {
   0xAAAA,0xAAAA,0xAAAA,0xAA55,0x8037,0x3780,
   0x5A4D,0xAAAA,0x4150,0xAAAA,0xAAAA
};

char *ClearLine = "                    ";

char *TypeActionStr[MenuTypesNum][2] = {
   {ClearLine,NULL},
   {ClearLine,NULL},
   {ClearLine,NULL},      // Is set dynamicly
   {ClearLine+13,"Upgrade"},
   {ClearLine+16,"Load"},
   {ClearLine+16,TypeActionStr[M_V64ROM][1]},
   {ClearLine+13,"Execute"},
   {ClearLine+13,NULL},   // Is set dynamicly
   {ClearLine+15,"Apply"},
   {ClearLine+14,"Change"},
   {ClearLine+17,"Set"}
};

/*--------------------------------------------------------------------------
   Global variables
 --------------------------------------------------------------------------*/

MenuEntry **menu = NULL;
int entryNumber,currentMenu,menuPos;
int fileNumber;
char screenLine[2][21];

/*--------------------------------------------------------------------------
   MenuAddEntry - Adds a new entry (mem-alloc..)
 --------------------------------------------------------------------------*/

void MenuAddEntry(char *fname,U8 type,U8 id = 0)
{
   if (DefMenuParam[currentMenu].attrib & type)   // Only if allowed
   {
      menu = (MenuEntry **)realloc(menu,(entryNumber+1)*sizeof(MenuEntry *));
      menu[entryNumber] = (MenuEntry *)malloc(sizeof(MenuEntry));
      menu[entryNumber]->name = strdup(fname);
      menu[entryNumber]->type = type;
      menu[entryNumber]->id = id;
      entryNumber++;
   }
}

/*--------------------------------------------------------------------------
   MenuBuild - Creates the menu
 --------------------------------------------------------------------------*/

void MenuBuild(void)
{
   /*
    *  1. Create menu-base (add constant entries)
    */

   menu = (MenuEntry **)malloc(DefMenuParam[currentMenu].len*
			       sizeof(MenuEntry *));
   for (entryNumber = 0;
	entryNumber < DefMenuParam[currentMenu].len;
	entryNumber++)
      menu[entryNumber] = ((MenuEntry *)&DefMenu[entryNumber+
			  DefMenuParam[currentMenu].start]);

   /*
    *  2. Append files and directories
    */

   fileNumber = 0;   // Usage below

   if (DefMenuParam[currentMenu].attrib != B_OnlyMenu)
   {
      struct ffblk fileBlock;
      int findex,loop;
      char *fext;
      U16 fid;
      Bool found;

      findex = findfirst("*.*",&fileBlock,FA_NORMAL | FA_DIREC);
      while (!findex)
      {
	 if (fileBlock.ff_attrib & FA_DIREC)
	    MenuAddEntry(fileBlock.ff_name,M_Directory);
	 else
	 {
	    found = False;
	    // a. Check extension
	    fext = strchr(fileBlock.ff_name,'.');
	    for (loop = 0; loop < binUtilNumber; loop++)
	       if (!strcmp(fext,binUtil[loop].ext))
	       {
		  MenuAddEntry(fileBlock.ff_name,M_BinUtilFile,loop);
		  found = True;
		  break;
	       }
	    // b. Check file-ID
	    if (found == False)
	    {
	       fid = FileGetID(fileBlock.ff_name);
	       for (loop = M_BiosFile; loop < MenuTypesNum; loop++)
		  if (TypeID[loop] == fid)
		  {
		     MenuAddEntry(fileBlock.ff_name,loop);
		     break;
		  }
	    }
	 }
	 findex = findnext(&fileBlock);
      }

   /*
    *  3. Sort entries (bubble-sort)
    */

      MenuEntry *temp;

      for (loop = 0; loop < entryNumber; loop++)
	 for (findex = 1; findex < entryNumber-loop; findex++)
	    if (menu[findex-1]->type > menu[findex]->type)
	    {
	       temp = menu[findex];
	       menu[findex] = menu[findex-1];
	       menu[findex-1] = temp;
	    }

   /*
    *  4. Count files
    */

   for (loop = 0; loop < entryNumber; loop++)
      if ((menu[loop]->type >= M_BiosFile) &&
	  (menu[loop]->type <= M_IPSFile))
      {
	 fileNumber++;
	 menu[loop]->fileNo = fileNumber;
      }

   /*
    *  5. Append drive-character, if needed
    */

     // if (1) // Replace through real check
	 MenuAddEntry("D:",M_Drive);

   }   // End of "DefMenuParam[].attrib != B_OnlyMenu"
}

/*--------------------------------------------------------------------------
   MenuClose - Free the menu-memory
 --------------------------------------------------------------------------*/

void MenuClose(void)
{
   for (; entryNumber > DefMenuParam[currentMenu].len; entryNumber--)
   {
      free(menu[entryNumber-1]->name);
      free(menu[entryNumber-1]);
   }
   free(menu);
}

/*--------------------------------------------------------------------------
   MenuChange - Sets a new menu
  -------------------------------------------------------------------------*/

void MenuChange(U8 changeTo)
{
   MenuClose();
   currentMenu = changeTo;
   MenuBuild();

   if (fileNumber)
      menuPos = DefMenuParam[currentMenu].len;
   else
      menuPos = 0;
}

/*--------------------------------------------------------------------------
   MenuGetN64ROMTitle - Sets the first screenline to an N64 ROM-title
 --------------------------------------------------------------------------*/

Bool MenuGetN64ROMTitle(Bool swap,char *dest)
{
   U8 loop;

   FileRead(menu[menuPos]->name,0x20,dest,20);
   if (swap)
      SwapBytes(dest,20);

   /*
    *  1. Check if title exists and kill unneeded chars
    */

   for (loop = 0; loop < 19; loop++)
      if (dest[loop] != dest[loop+1])
	 break;
   if (loop == 19)
      return(False);

   /*
    *  2. Set termination
    */

   for (loop = 19; loop; loop--)
      if (dest[loop] != dest[loop-1])
	 break;
   dest[loop+1] = 0;
   return(True);
}

/*--------------------------------------------------------------------------
   MenuGetBinUtilTitle - Tries to return the title
 -------------------------------------------------------------------------*/

Bool MenuGetBinUtilTitle(BinUtil *bdesc,char *dst)
{
   if (bdesc->type == Bin_JBE)
      return(MenuGetN64ROMTitle(False,dst));
   if (bdesc->titleAddr == -1)
      return(False);
   FileRead(menu[menuPos]->name,bdesc->titleAddr,dst,bdesc->titleLen);
   dst[bdesc->titleLen] = 0;
   return(True);
}

/*--------------------------------------------------------------------------
   MenuGenerateOutput - Erstellt die zwei Zeilen und gibt sie aus
 --------------------------------------------------------------------------*/

void MenuGenerateOutput(void)
{
   Bool alreadySet = False;
   char ctemp[21];

   /*
    *  1. Build first screen-line
    */

   switch(menu[menuPos]->type)
   {
      case M_BiosFile :
	 FileRead(menu[menuPos]->name,0x70002,screenLine[1],2);
	 sprintf(ctemp,"BIOS v%d.%2d",screenLine[1][0],screenLine[1][1]);
	 if (screenLine[1][1] < 10)
	    ctemp[8] = '0';
	 alreadySet = True;
	 break;
      case M_V64ROM :
	 alreadySet = MenuGetN64ROMTitle(True,ctemp);
	 break;
      case M_Z64ROM :
	 alreadySet = MenuGetN64ROMTitle(False,ctemp);
	 break;
      case M_BinUtilFile :
	 alreadySet = MenuGetBinUtilTitle(&binUtil[menu[menuPos]->id],ctemp);
	 TypeActionStr[M_BinUtilFile][1] =
	    binUtil[menu[menuPos]->id].actionStr;
	 break;
   }

   if (!alreadySet)
      strcpy(ctemp,menu[menuPos]->name);

   /*
    *  2. Output of the 1st line
    */

   LCDCls();
   CenterStr(screenLine[0],20,ctemp,' ');
   LCDPuts(screenLine[0]);

   /*
    *  2. Output second screen-line
    */

   LCDGotoXY(1,2);

   if (menu[menuPos]->type <= M_MenuAction)
      TypeActionStr[menu[menuPos]->type][1] = screenLine[0];
   else if (menu[menuPos]->type == M_Config)
   {
      ConfigUpdateStr(menu[menuPos]->id);
      TypeActionStr[menu[menuPos]->type][1] = screenLine[1];
   }
   else if (menu[menuPos]->type < M_Directory)
   {
      sprintf(screenLine[1],"%-7s   %3d / %3d",
	      TypeActionStr[menu[menuPos]->type][1],
	      menu[menuPos]->fileNo,
	      fileNumber);
      LCDPuts(screenLine[1]);
   }
   else
      LCDPuts(TypeActionStr[menu[menuPos]->type][1]);
}

/*--------------------------------------------------------------------------
   MenuHandler - menu-handler (reacts on pressed keys)
 --------------------------------------------------------------------------*/

void MenuHandler(void (*func_type)(U8 type),void (*func_maction)(U8 num))
{
   U8 key;
   char statusByte = 0;

   MenuGenerateOutput();
   while (1)
   {
      /*
       *  1. Handle key
       */

      if (key = KeypadRead())
      {
	 #ifdef USE_VGA
	    if (key == 0xFF)   // Escape - quit program
	       break;
	 #endif
	 if (key & KeyUp)
	 {
	    if (!menuPos)
	       menuPos = entryNumber-1;
	    else
	       menuPos--;
	 }
	 else if (key & KeyDown)
	 {
	    menuPos++;
	    if (menuPos == entryNumber)
	       menuPos = 0;
	 }
	 else   // Enter
	 {
	    if (menu[menuPos]->type == M_SubMenu)
	       MenuChange(menu[menuPos]->id);
	    else if (menu[menuPos]->type == M_MenuAction)
	       func_maction(menu[menuPos]->id);
	    else
	       func_type(menu[menuPos]->type);
	 }
	 MenuGenerateOutput();
      }

      /*
       *  2. If no keypressed - Idle
       */

      else
      {
	 if (StatusChanged())
	 {
	    statusByte ^= 1;
	    if (menu[menuPos]->type >= M_Config)
	       LCDGotoXY(1,2);
	    else
	       LCDGotoXY(1,1);
	    LCDPuts(TypeActionStr[menu[menuPos]->type][statusByte]);
	 }


	 // Wenn Disk gewechselt wird..
	 //
	 // - BinUtils neu initialisieren
	 // - Verzeichnis neu einlesen

      }
   }
}

