#ifndef __MENU_H_
#define __MENU_H_

/***************************************************************************
 *                                                                         *
 *  menu.h                                                                 *
 *                                                                         *
 ***************************************************************************/

#include "basetype.h"

/*--------------------------------------------------------------------------
   The menu-entry-types (Type-number and bitmask)
 --------------------------------------------------------------------------*/

#define MenuTypesNum   11

/*
 *  Type-number
 */

enum {
   M_SubMenu = 0,M_MenuAction,M_Config,
   M_BiosFile,M_V64ROM,M_Z64ROM,M_EXEFile,M_BinUtilFile,M_IPSFile,
   M_Directory,M_Drive
};

/*
 *  Bitmask
 */

#define B_SubMenu       0x0001
#define B_MenuAction    0x0002
#define B_Config        0x0003
#define B_BiosFile      0x0008
#define B_V64ROM        0x0010
#define B_Z64ROM        0x0020
#define B_EXEFile       0x0040
#define B_BinUtilFile   0x0080
#define B_IPSFile       0x0100
#define B_Directory     0x0200
#define B_Drive         0x0400

/*
 *  Some default connected bitmasks
 */

#define B_AllTypes    0xFFFF
#define B_OnlyMenu    (B_SubMenu   | B_MenuAction | B_Config)
#define B_DirNDrive   (B_Directory | B_Drive)

/*--------------------------------------------------------------------------
   Structures
 --------------------------------------------------------------------------*/

/*
 *  A single menu-entry
 */

typedef struct _MenuEntry {
   char   *name;
   U8     type;
   U8     fileNo;
   U8     id;
} MenuEntry;

/*
 *  For the constant menu-entries
 */

typedef struct _MenuParam {
   U8    start;
   U8    len;
   U16   attrib;
} MenuParam;

/*--------------------------------------------------------------------------
   Externals
 --------------------------------------------------------------------------*/

/*
 *  Variables
 */

extern char screenLine[2][21];
extern int menuPos;
extern MenuEntry **menu;

/*
 *  Functions
 */

extern void MenuClose(void);
extern void MenuChange(U8 changeTo);
extern void MenuHandler(void (*func_type)(U8 type),
			void (*func_maction)(U8 num));

/*--------------------------------------------------------------------------
   Macros (use them!)
 --------------------------------------------------------------------------*/

#define MenuCurrentName   \
   menu[menuPos]->name

#define MenuCurrentType   \
   menu[menuPos]->type

#define MenuCurrentID   \
   menu[menuPos]->id


#endif   // !__MENU_H_

