/***************************************************************************
 *                                                                         *
 *  util.cpp                                                               *
 *                                                                         *
 *  Misc. functions							   *
 *                                                                         *
 ***************************************************************************/

#include "all_inc.h"

/*--------------------------------------------------------------------------
   WriteDebugMsg - Just for debugging-reasons (will be removed)
 --------------------------------------------------------------------------*/

void WriteDebugMsg(char *msg)
{
   #ifdef USE_VGA
      gotoxy(1,4);
      printf("%s\n<Press a key to continue>",msg);
      getch();
   #else
      puts(msg);   // Monitor
   #endif
}
/*--------------------------------------------------------------------------
   StatusChanged - Something like that is in the Z64-flash-routines
 --------------------------------------------------------------------------*/

Bool StatusChanged(void)
{
   static long lastTime = biostime(0,0L),currentTime;
   static char blinkDummy = 13;

   currentTime = biostime(0,0L);
   if ((currentTime-lastTime) >= (blinkDummy ^ 7))
   {
      lastTime = currentTime;
      blinkDummy ^= 7;
      return(True);
   }
   return(False);
}

/*--------------------------------------------------------------------------
   SwapBytes - Little <-> big-endian-byteorder
 --------------------------------------------------------------------------*/

void SwapBytes(U8 *buffer,U16 size)
{
   asm {
      mov   cx,size
      shr   cx,1
      mov   di,buffer
   }
   sb_nextbyte:
   asm {
      mov   ah,[di]
      mov   al,[di+1]
      mov   [di],ax
      add   di,2
      dec   cx
      jnz   sb_nextbyte
   }
}

/*--------------------------------------------------------------------------
   CenterStr - Centers string and fills the gap before the string
 --------------------------------------------------------------------------*/

void CenterStr(char *dest,U16 sizeDest,char *src,char ctos)
{
   asm {
      push   si
      mov    si,src
      mov    bx,si
   }
   ms_count_src:
   asm {
      mov    al,[si]
      inc    si
      or     al,al
      jnz    ms_count_src
      sub    si,bx
      dec    si
      mov    cx,si
      mov    si,bx
      mov    bx,cx
      mov    ax,sizeDest
      sub    ax,bx
      shr    ax,1
      mov    di,dest
      mov    bl,ctos
   }
   ms_fill:
   asm {
      or     ax,ax
      je     ms_next_char
      mov    [di],bl
      inc    di
      dec    ax
      jmp    ms_fill
   }
   ms_next_char:
   asm {
      mov    al,[si]
      inc    si
      mov    [di],al
      inc    di
      dec    cx
      jnz    ms_next_char
      pop    si
      mov    [di],cl
   }
}

/*--------------------------------------------------------------------------
   ShowCenterStr - Shows a string centered
 --------------------------------------------------------------------------*/

void ShowCenterStr(char *str,char line_no)
{
   CenterStr(screenLine[line_no-1],20,str,' ');
   LCDPutsXY(1,line_no,screenLine[line_no-1]);
}

/*--------------------------------------------------------------------------
   SoundBeep - Single Beeee(e)ep
 --------------------------------------------------------------------------*/

void SoundBeep(U16 sfreq)
{
   sound(sfreq);
   delay(20);
   nosound();
}

/*--------------------------------------------------------------------------
   AskYesNo - Asks a YN-question
 --------------------------------------------------------------------------*/

Bool AskYesNo(char *question)
{
   const char *const YnStr =
      " YES  or  NO ";
   const char YnPos[2][2] = {
      { 4,8 },
      {13,16}
   };

   U8 key,answer = 0;
   Bool changed = False;

   CenterStr(screenLine[0],20,question,' ');
   LCDCls();
   LCDPuts(screenLine[0]);
   LCDPutsXY(4,2,(char *)YnStr);
   while (!((key = KeypadRead()) & KeyEnter))
   {
      if (key & KeyBoth)
      {
	 answer ^= 1;
	 LCDPutsXY(4,2,(char *)YnStr);
	 SoundBeep(0x400);
      }
      else
      {
	 changed ^= True;
	 delay(250);
	 LCDGotoXY(YnPos[answer][0],2);
	 LCDWriteChar((changed) ? ' ' : '[');
	 LCDGotoXY(YnPos[answer][1],2);
	 LCDWriteChar((changed) ? ' ' : ']');
      }
   }
   return((Bool)(!answer));
}

/*--------------------------------------------------------------------------
   ShowMessage
 --------------------------------------------------------------------------*/

void ShowMessage(char *str)
{
   LCDPutsXY(1,2,str);
   KeypadWait(KeyEnter);
}

