/***************************************************************************
 *                                                                         *
 *  zci.cpp                                                                *
 *                                                                         *
 *  Main                 	                                           *
 *  v0.14                                                                  *
 *                                                                         *
 *  Written by Ravemax/Dextrose                                            *
 *                                                                         *
 *  ---------------------------------------------------------------------  *
 *                                                                         *
 ***************************************************************************/

#include "all_inc.h"

/*--------------------------------------------------------------------------
   LoadROM - Loads a ROM, the EEPROM or SRAM (if available)
 --------------------------------------------------------------------------*/

Bool LoadROM(const char *fname,Bool isV64ROM)
{
   const Size128MBit = 0x1000000;

   long fsize;

   fsize = RamLoadFile(open(fname,O_RDONLY),Ram_Dram,0x0,
		       isV64ROM,True);

   if (fsize == -1L)
      return(False);

   if (fsize > Size128MBit)
      RamSetMemoryRights(N64_32M_ROM);
   else
      RamSetMemoryRights(N64_Dram);

   return(True);
}

/*--------------------------------------------------------------------------
   LoadEmuFile
 --------------------------------------------------------------------------*/

Bool LoadEmuFile(const char *fname)
{
   const U32 V64FileAddr = 0x200000;

   if (binUtil[MenuCurrentID].type == Bin_Z64)
   {
      const char *const BinUtilPathStr = "/bin/desc.*";

      char tempfname[128],hwVer[2];
      U8 pathChars;

      pathChars = strlen(BinUtilPathStr)-6;
      memcpy(tempfname,BinUtilPathStr,pathChars);
      strcpy(tempfname+pathChars,binUtil[MenuCurrentID].binStr);
      itoa(Z64hwVersion,hwVer,10);

      spawnl(P_WAIT,tempfname,tempfname,hwVer,fname,NULL);
   }
   else
   {
      if (RamLoadFile(BinUtilOpen(fname),Ram_Dram,0x0,False,True) == -1L)
	 return(False);
      if (RamLoadFile(open(fname,O_RDONLY),Ram_Dram,V64FileAddr) == -1L)
	 return(False);
      RamSetMemoryRights(N64_Dram);
   }

   printf("EmuFile : %s\n",fname);
   return(True);
}

/*--------------------------------------------------------------------------
   BackupCartridge
 --------------------------------------------------------------------------*/

Bool BackupCartridge(void)
{
   char tfname[13];
   long index = 0;
   int tfh;
   long cartSize = RamGetROMSize();

   do {
      sprintf(tfname,"%08ld.ROM",index);
      index++;
   } while ((tfh = creatnew(tfname,0x0)) == -1);
   printf("BackupCart: %s\n",tfname);
/*
   LCDGotoXY(1,2);
   LCDPrintf("Size:%ld",cartSize);
   KeypadWait(KeyEnter);
  */
 //  RamSaveToFile(tfh,Ram_ROM,0x0,cartSize);
   RamSaveToFile(tfh,Ram_Dram,0x0,cartSize);
}

/*--------------------------------------------------------------------------
   TypeHandler - Routine to handle the different menu-types
 --------------------------------------------------------------------------*/

void TypeHandler(U8 type)
{
   switch (type)
   {
      case M_Z64ROM :
	 LoadROM(MenuCurrentName,False);
	 break;
      case M_V64ROM :
	 LoadROM(MenuCurrentName,True);
	 break;
      case M_EXEFile :
	 char hwVer[2];

	 itoa(Z64hwVersion,hwVer,10);
	 if (spawnl(P_WAIT,MenuCurrentName,MenuCurrentName,hwVer,NULL) == -1)
	    ShowMessage("EXEC FAILED!");
//	 MenuChange(0);
	 break;
      case M_BinUtilFile :
	 LoadEmuFile(MenuCurrentName);
	 break;
      case M_Config :
	 ConfigChange(MenuCurrentID);
	 break;
      case M_Directory :
	 chdir(MenuCurrentName);
	 MenuChange(0);
	 break;
 //     case M_DRIVE :
//	 _chdrive(x);
//	 break;
      default :
	 WriteDebugMsg("Type not implemented yet!");
   }
}

/*--------------------------------------------------------------------------
   MenuActionHandler - Handles the different menu-actions
 --------------------------------------------------------------------------*/

void MenuActionHandler(U8 num)
{
   switch(num)
   {
      case 1 :   // Backup cart
	 BackupCartridge();
	 MenuChange(0);
	 break;
      case 2 :   // Play cart
	 RamSetMemoryRights(N64_ROM);
	 break;
      case 3 :   // write config
	 ConfigWrite();
	 break;
      default :
	 WriteDebugMsg("Menuaction not implemented yet!");
   }
}

/*--------------------------------------------------------------------------
   MAIN
 --------------------------------------------------------------------------*/

void main(int argc,char *argv[])
{
   Z64InitChip();
   LCDInit();
   ConfigRead();
   BinUtilInit();
   MenuChange(0);
   MenuHandler(TypeHandler,MenuActionHandler);
   MenuClose();
   BinUtilClose();
}

