/***************************************************************************
 *                                                                         *
 *  zipfile.cpp                                                            *
 *                                                                         *
 *  Handles the ZIP-files                                                  *
 *                                                                         *
 ***************************************************************************/

#include "zipfile.h"

extern void MenuAddItem(const char *str);   // Like in the menu

/*--------------------------------------------------------------------------
   Global variables
 --------------------------------------------------------------------------*/

Bool    zipMode = False;
FILE    *zipFile;
ZipDesc *zipfileDesc;
int     zipfileNum = 0;

/*--------------------------------------------------------------------------
   EnterZipMode() - Adds the filenames & creates ZIP-descriptions
 --------------------------------------------------------------------------*/

Bool EnterZipMode(char *fname)
{
   /*
    *  Opens ZIP-archive
    */

   if ((zipFile = fopen(fname,"rb")) == NULL)
      return(False);
   fseek(zipFile,2,SEEK_SET);
   zipfileDesc = NULL;

   /*
    *  Get filenames and -data
    */

   ZipHeader zHdr;
   char cbuffer[13];

   while (fgetc(zipFile) == 3)
   {
      zipfileDesc = (ZipDesc *)realloc(zipfileDesc,(zipfileNum+1)
		    *sizeof(ZipDesc));
      fseek(zipFile,5,SEEK_CUR);
      fread(&zHdr,1,sizeof(ZipHeader),zipFile);

      fread(cbuffer,1,zHdr.lenFilename,zipFile);
      cbuffer[zHdr.lenFilename] = 0;
      MenuAddItem(strupr(cbuffer));

      zipfileDesc[zipfileNum].stored = (Bool)(zHdr.compression == 0);
      zipfileDesc[zipfileNum].compressed = zHdr.compressed;
      zipfileDesc[zipfileNum].uncompressed = zHdr.uncompressed;
      zipfileDesc[zipfileNum].offset = ftell(zipFile);

      fseek(zipFile,(zHdr.compressed+zHdr.lenExtra+2),SEEK_CUR);
      zipfileNum++;
   }

   /*
    *  Close
    */
   const char *ExitZipfile_Str = "<Exit ZIP-file>";

   MenuAddItem(ExitZipfile_Str);
   zipMode = True;

   return(True);
}

/*--------------------------------------------------------------------------
   ExitZipMode() - Quits the "ZIP"-mode
 --------------------------------------------------------------------------*/

void ExitZipMode(void)
{
   fclose(zipFile);
   free(zipfileDesc);
   zipMode = False;
}

